function [W_err, W, W_pct_error, W_n_err, P_X] = CGP_residual_computing(R_t_3d, A, c, M, X, method)
% Jan. 29, 2016
% compute the residual 
% method 1: use R_t_3d
% method 2: use A and c (default)
% This function could calculate both residual and predition error. 
% Jun. 30, 2016, vectorize the computing.

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



if nargin < 6
    method = 2;
end

[m, n] = size(X);
W = zeros(m, n - M);
P_X = zeros(m, n - M);

Y = X(:, M + 1 : end);
XX = zeros(m * M, n - M);
for i = 1 : M
    XX((i - 1) * m + 1 : i * m, :) = X(:, M + 1 - i : n - i);
end
if method == 1 
    P_X = reshape(R_t_3d, size(R_t_3d, 1), []) * XX;
    W = Y - P_X;     
elseif method == 2
    A_0_M = arrayfun(@(i) A ^ i, 0 : M, 'UniformOutput', false);
    A_0_M = cat(3, A_0_M{:});
    R_3d_Ac = arrayfun(@(i) reshape(A_0_M, [], size(A_0_M, 3)) * c(i, :)', 1 : size(c, 1), 'uniformoutput', false);
    R_3d_Ac = reshape(cat(2, R_3d_Ac{:}), m, m, M);
    P_X = reshape(R_3d_Ac, size(R_3d_Ac, 1), []) * XX;
    W = Y - P_X;       
else
    error('unrecognized method!')
end

W_err = norm(W, 'fro') / sqrt(numel(W));

W_pct_error = norm(W ./ X(:, M + 1 : end), 'fro') / sqrt(numel(W));

W_n_err = W_err ./ mean(mean(abs(X(:, M + 1 : end))));


end
