function [out_stat, deleted_files] = chunk_lock_clean(path, duration)
% This function is for cleaning tmp files whose jobs are not running in the folder.
%

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



% Author: Xiongtao Ruan
% Date: Feb. 24, 2016
% change datetime('now') to clock to be compatible with 8.1

if nargin < 2
    delete_by_time = false;
else
    delete_by_time = true;
end
hard_duration = 60;
deleted_files = {};  
out_stat = false;
keep_flag = true;

[status, output] = system(sprintf('find %s -type f -name "*.tmp"', path));
if ~isempty(output)
    % temp_filenames = strsplit(strtrim(output), '\n');
    % use more robust way
    temp_filenames = strsplit(strtrim(output), 'tmp'); 
    
    if numel(temp_filenames) > 0 
        for i = 1 : numel(temp_filenames)
            current_temp_filename = [strtrim(temp_filenames{i}), 'tmp'];
            if delete_by_time
                temp_file_info = dir(current_temp_filename);
                %if (datenum(datetime('now')) - [temp_file_info.datenum]) * 24 * 60 > duration
                if (datenum(clock) - [temp_file_info.datenum]) * 24 * 60 > duration
                    deleted_files{end + 1} = current_temp_filename;
                    delete(current_temp_filename);
                end
            elseif exist(current_temp_filename, 'file')
                % current_temp_filename
                try
                    job_info = textread(current_temp_filename, '%s', 'delimiter', '\n');
                catch ME
                    disp(ME);
                    if ~exist(current_temp_filename, 'file')
                        continue;
                    end
                end
                if numel(job_info) ~=2
                    deleted_files{end + 1} = current_temp_filename;
                    delete(current_temp_filename);
                    keep_flag = false;
                    continue
                end
                id_type = job_info{1};
                id = job_info{2};
                 
                % for the cluster to check job id
                if strcmp(id_type, 'jid')
                    [status, output] = system(sprintf('sacct -j %s --format=State', id));           
                
                    if isempty(regexp(output, 'RUNNING'))  
                        deleted_files{end + 1} = current_temp_filename;
                        delete(current_temp_filename);
                        keep_flag = false;
                    end
                elseif strcmp(id_type, 'pid')
                    [status, output] = system(sprintf('ps -o pid= -p %s', id));  
                    if isempty(output)
                        deleted_files{end + 1} = current_temp_filename;
                        delete(current_temp_filename);
                        keep_flag = false;
                    end
                else
                    deleted_files{end + 1} = current_temp_filename;
                    delete(current_temp_filename);
                    keep_flag = false;			
                end 
		% add hard clean condition, that is, if the tmp file exist for a certain time(here set as 1hr)
                if keep_flag
                    temp_file_info = dir(current_temp_filename);
                    if (datenum(clock) - [temp_file_info.datenum]) * 24 * 60 > hard_duration
                            deleted_files{end + 1} = current_temp_filename;
                            delete(current_temp_filename);
                    end
                end
            end
        end
    end
end

if ~isempty(deleted_files)
    out_stat = true;
end

end
    

