function [zero_mat, xcor_mat] = compare_cross_correlations(X, M)

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



% currently only implement M = 1
[n, m] = size(X);
zero_mat = zeros(n);
xcor_mat = zeros(n);

shift = M;
for i = 1 : n - 1
    for j = i + 1 : n
       acro = xcorr_compute(X(i, :), X(j, :), shift);
       [max_acro, ind] = max(abs(acro));
       acro_rem = acro;
       % acro_rem(M + 1) = [];
       abs(acro_rem(M + 1));
       xcor_mat(i, j) = acro(ind);
       if max_acro > 1.1 * abs(acro_rem(M + 1))
           if ind <= M
               zero_mat(i, j) = M - ind + 1;
               xcor_mat(i, j) = acro(ind);           
           else
               zero_mat(j, i) = ind - M - 1;
               xcor_mat(j, i) = acro(ind);               
           end
       end             
    end
end


end

function [acro, lag] = xcorr_compute(cause, effect, shift)
% assume cause, effect have the same length
% use unnormalized cross correlation

n = numel(effect);
N = 2 * shift + 1;

acro = zeros(N, 1);

for i = 1 : N
    if i >= 1 * shift + 1
        cause_i = cause(1 : end - i + shift + 1) ;
        effect_i = effect(i - shift : end);
    else
        cause_i = cause(shift + 2 - i : end) ;
        effect_i = effect(1 : end + i - shift - 1);
    end
    % acro(i) = (1 / numel(y_i) * sum(y_i .* x_i) - mean(y_i) * mean(x_i)) / (std(y_i, 1) * std(x_i, 1));
    acro(i) = (1 / numel(cause_i) * sum(cause_i .* effect_i) - mean(cause_i) * mean(effect_i)) ;

end

lag = -shift : shift;

end