function [t, ts_trend] = critical_timepoint_extract(I_x, I_y, param)

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



    % this function is for extraction of important time point for the time
    % series that can represent the start of the event. 
    % I_y is the time seris of cause, I_x is the time series of effect, lag
    % is the proposed lag time for the analysis. 
    
    % Jan. 26, 2016
    % Feb. 3, 2016
    % extract trend. 
    
    
    t = 1;
    if nargin < 3
        lag = 2;
        direction = 'pos';
        method = 'xcorr';
    else
        lag = param.lag;
        direction = param.direction;
        method = param.method;
    end
    
    n = numel(I_y);
    switch method
        case 'xcorr'
            xcorr_lag = zeros(n - 2 * lag, 1);
            ts_trend_all = zeros(n - 2 * lag, 2);

            for i = 1 : n - 2 * lag
                I_x_i = I_x( i : i + lag) ;
                I_y_i = I_y(i + lag : i + 2 * lag);
                xcorr_lag(i) = 1 / ( lag + 1) * sum(I_x_i .* I_y_i) - mean(I_x_i) * mean(I_y_i);
                
                % analysis trend, use linear trend analysis. 
                
                XT = [[1 : lag + 1]', ones(lag + 1, 1)];
                YT_x = I_x_i(:);
                YT_y = I_y_i(:);
                beta_y = (XT' * XT) \ XT' * YT_y;
                beta_x = (XT' * XT) \ XT' * YT_x;
                ts_trend_all(i, :) = [beta_x(1), beta_y(1)];                
            end
            if strcmp(direction, 'pos') 
                [~, ind] = max(xcorr_lag);
            else
                [~, ind] = min(xcorr_lag);
            end
            t = ind + lag;
            ts_trend = ts_trend_all(ind, :);
            
        case 'regression'
            % use linear regression model to pick the one with minimum MSE
            MSE = zeros(n - lag, 1);
            for i = 1 : n - lag
                I_y_i = I_y(i : i + lag - 1);
                I_x_i = I_x(i : i + lag - 1);
                
                XT = [I_y_i(:), ones(lag, 1)];
                yT = I_x_i(:);
                
                XTest = [I_y(i + lag), 1];
                yTest = I_x(i + lag);
                
                MSE_i = sqrt((yTest - XTest * ((XT' * XT) \ XT' * yT)) .^ 2);
                MSE(i) = MSE_i;          
            end 
            
            [~, ind] = min(MSE_i);
            t = ind + lag;
            % currently not implemented. 
            ts_trend = [];
        otherwise
            error('Unrecognized method.')
    end 
end