function [p_value_mat, h_mat, D] = cross_correlation_pair_granger_causality(X, M, param)
%  Use for determine which pair has significant causal relationship, based
%  on granger causality tests. M stands for the maximun lag time allowed,
%  which is chosen by the cross correlation method. 
%
% Date: 12/23/2016

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



alpha_thresh = param.alpha;
[N, T] = size(X);
M_max = M;

[zero_mat, xcor_mat] = compare_cross_correlations(X, M_max);
if false
    figure, imagesc(zero_mat);
    title('Lag with max xcorr')
    colorbar;
    figure, imagesc(xcor_mat);
    title('Max xcorr')
    colorbar;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do not include background information

RSS_1 = zeros(N);
RSS_2 = zeros(N);
Causal_mat = zeros(N);
p_value_mat = nan(N);
lag_mat = nan(N);
D = zeros(N, N, M_max);

for i = 1 : N
    for j = 1 : N
        if zero_mat(j, i) > 0
            M_ij = zero_mat(j, i);
            if M_ij == 0
                M_ij = M_max;
            end
            if i == j
                continue;
            end
            if xcor_mat(j, i) > 0
                lag_mat(j, i) = M_ij;
                D(j, i, M_ij) = -1;
            else
                lag_mat(j, i) = -M_ij;
                D(j, i, M_ij) = 1;
            end
            
            inds = bsxfun(@plus, 1 : M_ij, (0 : T - M_ij - 1)');
            XTrain_1 = arrayfun(@(k) reshape(X(sort([i, j]), inds(k, :)), 1, []), 1 : size(inds, 1), 'uniformoutput', false);
            % including Y
            XTrain_1 = cat(1, XTrain_1{:});
            % pad 1
            XTrain_1 = [XTrain_1, ones(size(XTrain_1, 1), 1)];
            yTrain_1 = X(j, M_ij + 1 : end)';
            
            % excluding Y        
            X_2 = X;
            % X_2(i, :) = [];
            XTrain_2 = arrayfun(@(k) reshape(X_2(j, inds(k, :)), 1, []), 1 : size(inds, 1), 'uniformoutput', false);
            XTrain_2 = cat(1, XTrain_2{:});
            % pad 1
            XTrain_2 = [XTrain_2, ones(size(XTrain_2, 1), 1)];
            yTrain_2 = X(j, M_ij + 1 : end)';

            [b_1, bint_1, r_1] = regress(yTrain_1, XTrain_1);
            [b_2, bint_2, r_2] = regress(yTrain_2, XTrain_2);
            RSS_1_ji = r_1' * r_1;
            RSS_2_ji = r_2' * r_2;
            RSS_ratio_ji = RSS_2_ji / RSS_1_ji;
            RSS_1(j, i) = RSS_1_ji;
            RSS_2(j, i) = RSS_2_ji;
            RSS_ratio(j, i) = RSS_ratio_ji;                 
            Log_likehold(j, i) = log(RSS_ratio_ji);
            v1 = size(XTrain_1, 2) - size(XTrain_2, 2);
            % actually not correct
            v2 = size(XTrain_1, 1) - size(XTrain_1, 2);
            F_test_ji = (RSS_2_ji - RSS_1_ji) / v1  / (RSS_1_ji ./ v2);
            alpha = 0.05;
            c_v = finv(1-alpha, v1, v2);
            Causal_mat(j, i) = F_test_ji > c_v;   
            p_value_mat(j, i) = fcdf(F_test_ji, v1, v2,'upper');
        end
    end
end

flag = 1;
test_inds = ~isnan(p_value_mat);
pvalues = p_value_mat(test_inds);
% alpha_thresh = 0.5;
[corrected_p, h]=bonf_holm(pvalues,alpha_thresh);

corrected_p_mat = nan(N);
corrected_p_mat(test_inds) = corrected_p;
h_mat = zeros(N);
h_mat(test_inds) = h;

D = D .* repmat(h_mat, 1, 1, M_max);
D = reshape(D, size(D, 1), []);

end

