function [X, tt_X] = cross_validation_graph_data_generation(t_cell_info, cv_index)   
%
% March 18, 2017 change to use the defined region using all the data. 

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




s = RandStream('mt19937ar', 'Seed', 892734);
RandStream.setGlobalStream(s);

param = t_cell_info.param;
current_timepoint = param.current_timepoint;
interporlation_time = t_cell_info.model_data.interporlation_time;
cv_fold = param.cv_param.cv_fold;
cluster_sensor = param.cluster_sensor;
cluster_type = param.cluster_type;
cluster_num = param.cluster_num;
vol_thresh = param.vol_thresh;

cv_data_location = t_cell_info.path_info.cv_data_location;
cv_clustering_location = t_cell_info.path_info.cv_clustering_location;
clustering_result_location = t_cell_info.path_info.clustering_result_location;

template_crop_function = t_cell_info.model_data.template_crop_function;
template_image_crop = t_cell_info.model_data.template_image_crop;

cv_data_filename_j = sprintf('%sCV_data_%dfold_%d.mat', cv_data_location, cv_fold, cv_index);
a = load(cv_data_filename_j);
cv_means_mat_j = a.cv_means_mat_j;    
clear a;

a = load(['subscripts_standard_voxels.mat']);
subscripts = a.subscripts;
clear a;

m = numel(current_timepoint);

[~, inds] = intersect(interporlation_time, current_timepoint);

tn_mean_t = reshape(cv_means_mat_j(:, 1, 1, inds, :), [], numel(inds), size(cv_means_mat_j, 5));
tt_mean_t = reshape(cv_means_mat_j(:, 1, 2, inds, :), [], numel(inds), size(cv_means_mat_j, 5));
        
% if strcmp(cluster_sensor, 'all')
%     cluster_filename = sprintf('%smethod_%s_cluster_num_%d_cv_%d.mat', cv_clustering_location, cluster_type, cluster_num, i);
% else
%     cluster_filename = sprintf('%smethod_%s_sensor_%s_cluster_num_%d_cv_%d.mat', cv_clustering_location, cluster_type, cluster_sensor, cluster_num, i);
% end
% 
% if ~exist(cluster_filename, 'file')
%     % Jan. 27, 2016 set streams to allow reproducible results
%     if strcmp(cluster_sensor, 'all')
%         X = reshape(permute(tn_mean_t, [3, 2, 1]), size(tn_mean_t, 3), []);
%     elseif ~strcmp(data_type, '2pt') && strcmp(cluster_sensor, 'Actin')
%         sensor_ind = ~cellfun(@isempty, regexpi(unique_sensor_names, cluster_sensor));
%         X = squeeze(tn_mean_t(sensor_ind, :, :))';
%     elseif strcmp(data_type, '2pt') && strcmp(cluster_sensor, 'Actin')
%         X = read_sensors_model_mean_data({cluster_sensor}, {current_condition}, current_timepoint);
%         X = X{1}';
%     end
%     if strcmp(cluster_type, 'kmeans')
%         s = RandStream('mt19937ar', 'seed', 1);
%         km_options.streams = s;
%         labels = kmeans(X, cluster_num, 'options', km_options);
%     elseif strcmp(cluster_type, 'gmmem')
%         [labels] = emgm(X, cluster_num);
%     end    
% 
%     save(cluster_filename, 'labels', 'X', 'cluster_num');        
% else    
%     cluster_data = load(cluster_filename);
%     labels = cluster_data.labels;
% end
% 
% % check cluster numbers
% 
% % label_reshape = reshape(labels, size(model_means_vec, 1), []);
% 
% label_reconstruct = model_reconstruction(labels, subscripts);
% label_reconstruct_cropped = template_crop_function(label_reconstruct);
% 
% region_count = 0;
% region_label_group = zeros(m, cluster_num);
% labeled_maps = {};
% region_mean = {};
% 
% map_label = zeros(size(label_reconstruct_cropped));
%         
% for i1 = 1 : cluster_num
%     map_i = (label_reconstruct_cropped == i1);
%     %[labels] = bwlabeln(map_ij, 6);
%     %label_vals = unique(labels);
%     %label_vals(label_vals == 0) = [];
%     % xruan 01/13/2016 change the connect component to 26-connection
%     % cc = bwconncomp(map_ij, 6);   
%     cc = bwconncomp(map_i, 26);
%     inds = find(cellfun(@numel, cc.PixelIdxList) >= vol_thresh);
%     for k = 1 : numel(inds)
%         map_label(cc.PixelIdxList{inds(k)}) = k + region_count;
%     end
%     if isempty(inds)
%         k = 0;
%     end
%     region_count = region_count + k;   
%     region_label_group(i1) = numel(inds);
% end   
% map_label;
% 
% % Feb. 4, 2016
% % sort the regions by their centroid with the order of y, x, z
% [region_centroids, region_labels, label_order, map_label_sorted] = region_centroid_compute(map_label);
% map_label = map_label_sorted;
% % save the map labels so that other postprocessing scripts can directly use
% % the map label for consistency. 
% map_label_filename = sprintf('%sclustering_ROI_region_label_map_cv_%d_cluster_num-%d_vol_thrsd-%d.mat', cv_clustering_location, cv_index, cluster_num, vol_thresh);
% if ~exist(map_label_filename, 'file')
%     save(map_label_filename, 'map_label', 'region_count', 'region_labels');
% end

map_label_filename = sprintf('%sclustering_ROI_region_label_map_cluster_num-%d_vol_thrsd-%d.mat', clustering_result_location, cluster_num, vol_thresh);
load(map_label_filename);

map_label_vec = map_label(template_image_crop > 0);
region_label_vals = unique(map_label_vec);
region_label_vals(region_label_vals == 0) = [];

X_mat = zeros(size(tn_mean_t, 1), size(tn_mean_t, 2), numel(region_label_vals));
tt_X_mat = zeros(size(tt_mean_t, 1), size(tt_mean_t, 2), numel(region_label_vals));

for i1 = 1 : numel(region_label_vals)
    label_i = region_label_vals(i1);
    inds_i = (map_label_vec == label_i);
    X_mat(:, :, i1) = sum(tn_mean_t(:, :, inds_i), 3)  ./ sum(inds_i) * numel(inds_i);     
    tt_X_mat(:, :, i1) = sum(tt_mean_t(:, :, inds_i), 3) ./ sum(inds_i) * numel(inds_i);    
end

% Dec 21, change X = reshape(permute(X_mat, [1, 3, 2]), [], size(X_mat, 2));
% to X = reshape(permute(X_mat, [3, 1, 2]), [], size(X_mat, 2));
X = reshape(permute(X_mat, [3, 1, 2]), [], size(X_mat, 2));
tt_X = reshape(permute(tt_X_mat, [3, 1, 2]), [], size(tt_X_mat, 2)); 
% Jan. 21, 2016 xruan
% don't normalize the time series. 
% granger causality tests
X_orig = X;
tt_X_orig = tt_X;

clearvars tn_mean_t tt_mean_t;

% merge nodes

if param.merge_proteins
    % by default, merge (cpalpha1, wasp, coronin1A,
    % wave2), actin, (arp3, hs1), cofilin, ezrin, pip2,
    % lat, mrlc 
    % we use th mean of the training and validation data.
    region_num = cluster_num;
    % group_inds = {[1, 7], 2, [3, 5, 11, 12], 4, 6, 8, 9, 10};
    group_inds = param.group_inds;
    X_new = zeros(region_num, numel(group_inds), size(X, 2));
    tt_X_new = zeros(region_num, numel(group_inds), size(tt_X, 2));
    X_reshape = reshape(X, region_num, [], size(X, 2));
    tt_X_reshape = reshape(tt_X, region_num, [], size(tt_X, 2));
    for i1 = 1 : numel(group_inds)
        X_new(:, i1, :) = mean(X_reshape(:, group_inds{i1}, :), 2);
        tt_X_new(:, i1, :) = mean(tt_X_reshape(:, group_inds{i1}, :), 2);  
    end
    % just keep them for tracking or debuging

    X = reshape(X_new, [], size(X_new, 3));
    tt_X = reshape(tt_X_new, [], size(tt_X_new, 3));
end

end