
%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



function [filenames] = dir_recursive(given_dir)

    % xruan 08/13/2015
    % get all filenames under the current dir and its subdiretories 
    % use DFS to recursively get all filenames (no directory)
    
    if strcmp(given_dir(end), filesep)
        given_dir(end) = [];
    end
      
    filenames = {};
    
    stack = {};
    
    if isdir(given_dir)
        stack{end + 1} = given_dir;
    end
    
    
    while ~isempty(stack)

        % pop one element
        current_dir = stack{end};
        stack(end) = [];

        fileinfo = dir(current_dir);
        if isempty(fileinfo)
            continue;
        end
        subdir_files = {fileinfo.name};
        sub_isdir = [fileinfo.isdir];
        subdir_files(1 : 2) = [];
        sub_isdir(1 : 2) = [];
        subdirs = cellfun(@(x) [current_dir, filesep, x], subdir_files(sub_isdir), 'UniformOutput', false);
        if length(subdirs) > 0
            stack(end + 1 : end + length(subdirs)) = subdirs;
        end

        subfiles = cellfun(@(x) [current_dir, filesep, x], subdir_files(~sub_isdir), 'UniformOutput', false);
        if length(subfiles) > 0
            filenames(end + 1 : end + length(subfiles)) = subfiles;
        end
    end
end


