function [MC] = extract_correlated_nodes_based_on_xcorr(X, M, param)
% The function is to determine which pair of node has large xcorr with lag
% time 0. The output is the maximal cliques the reflect such relationships.
% 
% Date: Jan. 4, 2017

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



if nargin < 1
    debug = true;
    M = 2;
    load('t_X.mat');
    len_per_sensor = 3;
    all_sensor_region_combine = cellfun(@(x) arrayfun(@(y) sprintf('%s - %d', x, y),  1 : len_per_sensor, 'uniformoutput', false), unique_sensor_names, 'uniformoutput', false);
    all_sensor_region_combine = reshape(cat(1, all_sensor_region_combine{:})', 1, []);   
end
   
addpath(genpath('maximalCliques'));

N_node = size(X, 1);
Xcorr_mat = zeros(N_node, N_node, 2 * M + 1);
for i = 1 : N_node
    for j = 1 : N_node
        if i == j
            continue;
        end
        [acro, lag] = xcorr_compute(X(i, :), X(j, :));
        Xcorr_mat(i, j, :) = acro(lag >= -2 & lag <= 2);
    end
end


Xcorr_m = Xcorr_mat(:, :, M + 1 : end);
Xcorr_m1 = abs(Xcorr_mat(:, :, M + 1 : end));
[max_xcorr, max_ind] = max(Xcorr_m1 , [], 3);
alpha = 2.5;
relation_mat = zeros(N_node);

for i = 1 : N_node
    for j = 1 : N_node
        if i == j
            continue;
        end        
        if max_ind(i, j) == 1 && Xcorr_m1(i, j, 1) > alpha * max(Xcorr_m1(i, j, 2 : end)) && Xcorr_m1(i, j, 1) > 3 * mean(max_xcorr(:))
            relation_mat(i, j) = Xcorr_m(i, j, 1);
        end
    end
end

if false
    figure, imagesc(relation_mat)
    set(gca,'YDir','reverse')
    matrix_plot_handle = gca;
    ytrick_labels = all_sensor_region_combine;
    xtrick_labels = all_sensor_region_combine;
    tick_values = 1 : size(relation_mat, 1);

    set(matrix_plot_handle, 'YTick', tick_values, 'YTickLabel', ytrick_labels, 'fontsize', 8, 'fontweight', 'bold')

    set(matrix_plot_handle, 'XTick', tick_values, 'XTickLabel', xtrick_labels, 'fontsize', 8, 'fontweight', 'bold')
    rotateXLabels(matrix_plot_handle, 90);

    h = colorbar;
    set(h, 'Position', positions(3, :))
    hold off      
end

% select pairs that has similar xcorr at zero to each other. 
a = (relation_mat - relation_mat') ./ relation_mat ;
b = abs(a) <1e-4;
if false 
    figure, imagesc(b)
    set(gca,'YDir','reverse')
    matrix_plot_handle = gca;
    ytrick_labels = all_sensor_region_combine;
    xtrick_labels = all_sensor_region_combine;
    tick_values = 1 : size(b, 1);

    set(matrix_plot_handle, 'YTick', tick_values, 'YTickLabel', ytrick_labels, 'fontsize', 8, 'fontweight', 'bold')

    set(matrix_plot_handle, 'XTick', tick_values, 'XTickLabel', xtrick_labels, 'fontsize', 8, 'fontweight', 'bold')
    rotateXLabels(matrix_plot_handle, 90);
end

% maximal cliques. 
[ MC ] = maximalCliques( b, 'v2' );

if false
    figure, imagesc(MC)
    set(gca,'YDir','reverse')
    matrix_plot_handle = gca;
    ytrick_labels = all_sensor_region_combine;
    % xtrick_labels = all_sensor_region_combine;
    tick_values = 1 : size(b, 1);

    set(matrix_plot_handle, 'YTick', tick_values, 'YTickLabel', ytrick_labels, 'fontsize', 8, 'fontweight', 'bold')

    % set(matrix_plot_handle, 'XTick', tick_values, 'XTickLabel', xtrick_labels, 'fontsize', 8, 'fontweight', 'bold')
    rotateXLabels(matrix_plot_handle, 90);
end



end