function [t_cell_info] = master_script_B7_permutation_testing(t_cell_info)
% choose the pairs that have larges xcorr with at least one time lag. And
% then test the granger causality between these pairs. Because we don't
% have enough data, we don't include the background information in the
% Granger causality testing. 
%
% change the method to only permute within each protein each region. 
% and also define chosen pairs for each permute 
% 
% Author: Xiongtao Ruan
% Date: 03/17/2017
%
% 
% copied from master_script_pair_granger_causality_B7_testing.m

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




                                  
param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;

N_test = param.permutation_tests_num;

graph_data = t_cell_info.graph_data;
X = graph_data.X;
tt_X = graph_data.tt_X;

model_sub_location = t_cell_info.path_info.model_sub_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;
permutation_sub_location = t_cell_info.path_info.permutation_sub_location;
permutation_computing_location = t_cell_info.path_info.permutation_computing_location;

% specific parameters
switch causal_method
    case 'CGP'
        N_perfile = 100;
        time_perfile = 0.3;    
    case  'CENR'
        N_perfile = 500;
        time_perfile = 1 / 30;
end

computing_models = true;
p_value_method = 'empirical';
shuffling_method = param.permutation_tests_shuffling_method;

N = size(X, 1);

optimal_param = t_cell_info.cv_info.optimal_param;
optimize_param.M = optimal_param.M;
optimize_param.lambda_1 = optimal_param.lambda_1;
optimize_param.lambda_2 = optimal_param.lambda_2;
optimize_param.lambda_3 = optimal_param.lambda_3;
optimize_param.self_prediction = optimal_param.self_prediction;

switch causal_method
    case  'CENR'
        optimize_param.alpha = optimal_param.alpha;
        b = optimal_param.b;
        gc_param.alpha = optimize_param.alpha;
    case 'CGP'
        if ~optimize_param.self_prediction
            zero_mat = kron(eye(size(X, 1) / cluster_num), ones(cluster_num));
            optimize_param.zero_mat = zero_mat;
        end    
end

% computing_models = true;

N_per_partition = N_perfile;
N_total = N_test;
N_partition = ceil(N_test / N_perfile);

method = param.residual_compute_method;

s = RandStream('mt19937ar', 'Seed', 2390785);
RandStream.setGlobalStream(s);

Rand_seeds = round(randperm(round(N_partition * 1.2))' * 2565 + (rand(round(N_partition * 1.2), 1) - 0.5) * 1000);
Rand_seeds = unique(Rand_seeds);
Rand_seeds = Rand_seeds(1 : N_partition);

Max_loops = 5;
loop_count = 0;

while computing_models && loop_count < Max_loops
    computed_mat = zeros(N_partition, 1);
    
    % use block partitioning for the computing. 
    chunk_lock_clean(permutation_computing_location, N_perfile * time_perfile);
    for k0 = 1 : N_partition
        % s = RandStream('mt19937ar', 'Seed', Rand_seeds(k0));
        % RandStream.setGlobalStream(s);
        % rng(Rand_seeds(k0));
        
        k_beg = (k0 - 1) * N_per_partition + 1;
        k_end = min(N_total, k0 * N_per_partition);

        results_filename = sprintf('%sclustering_ROI_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.mat', permutation_computing_location, current_timepoint(1), current_timepoint(end), vol_thresh, k_beg, k_end);
        temp_filename = sprintf('%sclustering_ROI_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.tmp', permutation_computing_location, current_timepoint(1), current_timepoint(end), vol_thresh, k_beg, k_end);
        % the definition here is different from other scripts, these are
        % for per protein errors rather than all proteins. 
        W_row_pct_err_mat = zeros(k_end - k_beg + 1, N);
        P_row_pct_err_mat = zeros(k_end - k_beg + 1, N);
        P_row_err_mat = zeros(k_end - k_beg + 1, N);
        P_row_protein_err_mat = zeros(k_end - k_beg + 1, N / 3);
        
        if exist(results_filename, 'file')
            fprintf('Partition %d has been computed.\n', k0);
            computed_mat(k0) = 1;
            continue;
        end
        if exist(temp_filename, 'file')
           fprintf('Partition %d is being computed.\n', k0);
           continue;
        else
            fclose(fopen(temp_filename, 'w'));
        end
        fprintf('Compute partition: %d\n', k0);
        for k = k_beg : k_end
            s = RandStream('mt19937ar', 'Seed', Rand_seeds(k0) + k);
            RandStream.setGlobalStream(s);
            
            % randperm(10)
            % X_shuffle = cell2mat(cellfun(@(x) x(randperm(numel(x))),  mat2cell(X, ones(size(X, 1), 1), size(X, 2)), 'uniformoutput', false));
            % X_shuffle = cell2mat(cellfun(@(x) x(randperm(numel(x))),  mat2cell(X, ones(size(X, 1), 1), size(X, 2)), 'uniformoutput', false));
            
            X_shuffle = shuffle_data_function(X, shuffling_method);
            
            switch causal_method
                case 'CENR'
                    [~, ~, D] = cross_correlation_pair_granger_causality(X_shuffle, M, gc_param);
                    tic
                    % [A] = master_script_spatiotemporal_group_lasso_admm(X, M, options);
                    [R_t_3d, A, c] = master_script_spatiotemporal_constraint_projected_admm(X_shuffle, M, D, b, optimize_param);
                    toc
                case 'CGP'
                    [R_t_3d, A, c] = master_script_causal_graph_process(X_shuffle, M, optimize_param);
            end
            
            [W_err, W, W_pct_error, W_n_err] = CGP_residual_computing(R_t_3d, A, c, M, X_shuffle, method);
            [P_err, P_W, P_pct_error, P_n_err] = CGP_residual_computing(R_t_3d, A, c, M, tt_X, method);                    
            fprintf(' W_n: %4f, W_pct: %4f\n P_n: %4f, P_pct: %4f\n\n', W_n_err, W_pct_error, P_n_err, P_pct_error);                    
            W_row_pct_err_mat(k - k_beg + 1, :) = sqrt(mean((W ./ abs(X_shuffle(:, M + 1 : end))) .^ 2, 2));
            Row_P_pct_error = sqrt(mean((P_W ./ abs(tt_X(:, M + 1 : end))) .^ 2, 2));
            Row_P_error = sqrt(mean(P_W  .^ 2, 2));
            Row_P_protein_error = sqrt(mean(reshape(reshape((P_W ./ tt_X(:, M + 1 : end))', size(P_W, 2), 3, []), [], 8) .^2)');
            
            P_row_pct_err_mat(k - k_beg + 1, :) = Row_P_pct_error;
            P_row_err_mat(k - k_beg + 1, :) = Row_P_error;
            P_row_protein_err_mat(k - k_beg + 1, :) = Row_P_protein_error;
        end
        save(results_filename, 'P_row_pct_err_mat',  'P_row_protein_err_mat', 'P_row_err_mat', 'k_beg', 'k_end', 'param');
        if exist(results_filename, 'file') && exist(temp_filename, 'file')
            delete(temp_filename);
        end
        computed_mat(k0) = 1;
    end
    if all(computed_mat(:) == 1) 
        fprintf('All computing has been finished!\n');
        break;
    end    
    pause(5 * 60);
    loop_count = loop_count + 1;        
end

compute_p_value = true;

model_result_filename = sprintf('%sclustering_ROI_results_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d', model_sub_location, causal_method, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh);

results = load(model_result_filename);
P_W = results.P_W;
P_row_pct_err = sqrt(mean((P_W ./ abs(tt_X(:, M + 1 : end))) .^ 2, 2));
P_protein_row_pct_err = sqrt(mean(reshape(reshape((P_W ./ tt_X(:, M + 1 : end))', size(P_W, 2), 3, []), [], 8) .^2)');

N_test_count = 0;
pct_p_value_upper = zeros(size(P_row_pct_err));
pct_p_value_lower = zeros(size(P_row_pct_err));
pct_p_value_tail = zeros(size(P_row_pct_err));

protein_p_value_upper = zeros(size(P_protein_row_pct_err));
protein_p_value_lower = zeros(size(P_protein_row_pct_err));
protein_p_value_tail = zeros(size(P_protein_row_pct_err));

% A_mat = zeros(X_size, X_size, N_test);

% now only implement empirical distribution based p-values. 
testing_result_file = sprintf('%sclustering_ROI_reltime_%d-%d_vol_thrd_%d_testing_summary.mat', permutation_sub_location, current_timepoint(1), current_timepoint(end), vol_thresh);
if compute_p_value && ~exist(testing_result_file, 'file')
    % count if skip any file 
    N_skip = 0;
    switch p_value_method            
        case 'empirical'
            for i = 1 : N_partition
                i_range = (i - 1) * N_perfile + 1 : min(N_test, i * N_perfile);
                result_file = sprintf('%sclustering_ROI_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.mat', permutation_computing_location, current_timepoint(1), current_timepoint(end), vol_thresh, i_range(1), i_range(end));
                try 
                    load(result_file);
                    % p-values for each row
                    pct_p_up_part = P_row_pct_err < P_row_pct_err_mat';
                    pct_p_value_upper = pct_p_value_upper + sum(pct_p_up_part, 2);
                    
                    pct_p_low_part = P_row_pct_err > P_row_pct_err_mat';
                    pct_p_value_lower = pct_p_value_lower + sum(pct_p_low_part, 2);  
                    
                    pct_p_tail_part = abs(P_row_pct_err) < abs(P_row_pct_err_mat');
                    pct_p_value_tail = pct_p_value_tail + sum(pct_p_tail_part, 2);  
                    
                    % p-values for each protein
                    protein_p_up_part = P_protein_row_pct_err < P_row_protein_err_mat';
                    protein_p_value_upper = protein_p_value_upper + sum(protein_p_up_part, 2);
                    
                    protein_p_low_part = P_protein_row_pct_err > P_row_protein_err_mat';
                    protein_p_value_lower = protein_p_value_lower + sum(protein_p_low_part, 2);  
                    
                    protein_p_tail_part = abs(P_protein_row_pct_err) < abs(P_row_protein_err_mat');
                    protein_p_value_tail = protein_p_value_tail + sum(protein_p_tail_part, 2);  
                           
                    N_test_count = N_test_count + numel(i_range);
                    
                catch ME
                    fprintf('The testing %d - %d do not exist!\n', i_range(1), i_range(end));
                    N_skip = N_skip + 1;
                    continue;
                end               
            end
            if N_skip == 0
                pct_p_value_upper = (pct_p_value_upper) / (N_test_count);
                pct_p_value_lower = (pct_p_value_lower) / (N_test_count);
                pct_p_value_tail = (pct_p_value_tail) / (N_test_count);
                
                protein_p_value_upper = (protein_p_value_upper) / (N_test_count);
                protein_p_value_lower = (protein_p_value_lower) / (N_test_count);
                protein_p_value_tail = (protein_p_value_tail) / (N_test_count);                
                save(testing_result_file, 'param', 'pct_p_value_upper', 'pct_p_value_lower', 'pct_p_value_tail', 'protein_p_value_upper', 'protein_p_value_lower', 'protein_p_value_tail');
            end
        otherwise
            error('Unrecognized testing method!');
    end
else
    load(testing_result_file);
end

testing_info = struct();
testing_info.test_name = 'B7_testing';
testing_info.permutation_tests_num = param.permutation_tests_num;
testing_info.P_row_pct_err = P_row_pct_err;
testing_info.P_protein_row_pct_err = P_protein_row_pct_err;

testing_info.pct_p_value_upper = pct_p_value_upper;
testing_info.pct_p_value_lower = pct_p_value_lower;
testing_info.pct_p_value_tail = pct_p_value_tail;

testing_info.protein_p_value_upper = protein_p_value_upper;
testing_info.protein_p_value_lower = protein_p_value_lower;
testing_info.protein_p_value_tail = protein_p_value_tail;

t_cell_info.testing_info = testing_info;


end

function [X_shuffle] = shuffle_data_function(X, method)

switch method
    case 'Same_protein_region'
        % original method
        X_shuffle = cell2mat(cellfun(@(x) x(randperm(numel(x))),  mat2cell(X, ones(size(X, 1), 1), size(X, 2)), 'uniformoutput', false));
        
    case 'Same_protein'
        X_1 = permute(reshape(X, 3, [], size(X, 2)), [2, 3, 1]);
        X_1 = reshape(X_1, size(X_1, 1), []);
        X_1_shuffle = cell2mat(cellfun(@(x) x(randperm(numel(x))),  mat2cell(X_1, ones(size(X_1, 1), 1), size(X_1, 2)), 'uniformoutput', false));
        X_shuffle = permute(reshape(X_1_shuffle, size(X_1_shuffle, 1), 3, []), [2, 1, 3]);
        X_shuffle = reshape(X_shuffle, [], size(X_shuffle, 3));
    case 'All_data'
        X_shuffle = reshape(X(randperm(numel(X))), size(X));
end

end

