function [t_cell_info] = master_script_B7_permutation_testing_postprocess(t_cell_info)                                      
% permutation testing just computing all files first. 
% 
% Visualize p-value time series. 
% we only consider the left-side p-values
% 
% 

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




                                  
param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;
suffix = param.suffix;

N_test = param.permutation_tests_num;

graph_data = t_cell_info.graph_data;
X = graph_data.X;
tt_X = graph_data.tt_X;

model_sub_location = t_cell_info.path_info.model_sub_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;
permutation_sub_location = t_cell_info.path_info.permutation_sub_location;
permutation_computing_location = t_cell_info.path_info.permutation_computing_location;

graph_data = t_cell_info.graph_data;
kept_nodes = graph_data.kept_nodes;
kept_node_names = graph_data.kept_node_names;
kept_sensor_names = graph_data.kept_sensor_names;
region_num = graph_data.region_num;
len_per_sensor = region_num;

results_filename = sprintf('%sclustering_ROI_results_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d%s', model_sub_location, causal_method, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh, suffix);
results = load(results_filename);
A = results.A;
try
    c = results.c;
    R_t_3d = results.R_t_3d;
end
method = param.residual_compute_method;

[P_err, P_W, P_pct_error, P_n_err, B7_P_X] = CGP_residual_computing(R_t_3d, A, c, M, tt_X, method);

testing_info = t_cell_info.testing_info;
P_protein_row_pct_err = testing_info.P_protein_row_pct_err;
protein_p_value_lower = testing_info.protein_p_value_lower;

[corrected_p_low, h_low] = bonf_holm(protein_p_value_lower);
P_protein_row_pct_err = P_protein_row_pct_err .* h_low;

sg_inds = find(h_low == 1);
sg_pvalues = corrected_p_low(sg_inds);
sg_thrsh = max(P_protein_row_pct_err(h_low == 1));
pvalue_sg_thrsh_org = max(protein_p_value_lower(h_low == 1));
pvalue_sg_thrsh = max(sg_pvalues);

% use protein level permutation tests, 
% [sg_region_inds, sg_sensor_inds] = ind2sub([region_num, numel(kept_node_names) / region_num], sg_inds);
sg_sensor_inds = sg_inds;
sg_region_inds = 1 : region_num;

unique_sg_sensor_inds = unique(sg_sensor_inds);
Num_col = region_num;

% [B, sort_inds] = sortrows([sg_sensor_inds, sg_region_inds]);

if ~isempty(sg_sensor_inds)
    figure, 
    set(gcf, 'color', 'w')
    set(gcf, 'Visible', 'on')
    % set(gcf, 'Visible', 'off')

    per_image_size = 200;
    m = numel(unique_sg_sensor_inds);
    n = Num_col;

    figure_width = per_image_size * Num_col * 1.5;
    figure_height = per_image_size * numel(unique_sg_sensor_inds) * 1.5;
    border_width = 0.3;
    border_hight = 0.2;

    set(gcf, 'Position', [1, 1, figure_width, figure_height]);
    axis off;

    cm = @jet;

    colors = cm(9);
    y_color = [0, 0, 0];

    figure_handles = {};

    for i = 1 : numel(unique_sg_sensor_inds)
        curr_sg_sensor_ind = unique_sg_sensor_inds(i);
        % inds = find(B(:, 1) == curr_sg_sensor_ind);
        sg_sensor = kept_sensor_names{curr_sg_sensor_ind};        
        curr_sg_region_inds = sg_region_inds;
        

        for j = 1 : numel(curr_sg_region_inds)

            curr_sg_region_ind = curr_sg_region_inds(j);
            
            org_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, tt_X, numel(kept_node_names) / len_per_sensor);
            prd_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, B7_P_X, numel(kept_node_names) / len_per_sensor);
            
            m_rate = 0.7;
            n_rate = 0.3;

            per_height = 1 / (m + ( m - 1) * m_rate);
            per_width = 1 / (n + (n - 1) * n_rate);

            % img_corner_1 = [(i - 1) * per_image_size + 1, [j - 1] * per_image_size + 1];
            % img_corner_2 = [i * per_image_size, j * per_image_size];

            curr_position = [per_width * ( 1 + n_rate) * (j - 1), per_height * ( 1 + m_rate) * (m - i) , per_width, per_height];
            curr_position(1) = curr_position(1) * (1 - border_width * 2) + border_width;
            curr_position(2) = curr_position(2) * (1 - border_hight * 2) + border_hight;
            curr_position(3) = curr_position(3) * (1 - border_width * 2);
            curr_position(4) = curr_position(4) * (1 - border_hight * 2);

            axes('Position', curr_position);

            h = plot(current_timepoint, org_sg_plot, 'Color', [0, 0, 0], 'linewidth', 2);    
            xlim([current_timepoint(1), current_timepoint(end)])
            figure_handles{end + 1, 1} = h;
            % set(gca,'YTickLabel',[]);
            ylim_1=get(gca,'ylim');
            title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 10);
            hold on 
            
            % plot the predicted one
            h = plot(current_timepoint(M + 1 : end), prd_sg_plot, ':r', 'LineWidth', 2);              
            figure_handles{end + 1, 2} = h;        
            xlim([current_timepoint(1), current_timepoint(end)])
            % set(gca,'YTickLabel',[]);
            ylim_2=get(gca,'ylim');
            % axis off
            % set(gca, 'ylim', [min([ylim_1, ylim_2]), max([ylim_1, ylim_2])])
            title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 10);
                        
            hold on
            
            if j == 1
                if cluster_num == 6
                    text(-7, sum(ylim_1) / 2, sprintf('%s', sg_sensor));
                else
                    text(-140, sum(ylim_1) / 2, sprintf('%s', sg_sensor));
                end
            end
           
        end               
    end


    hold off 

    dpi = 300;

    figure_filename = sprintf('%sclustering_ROI_B7_prediction_significant_kinetic_plots_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         
end




% hack version for the paper
if ~isempty(sg_sensor_inds)
    figure, 
    set(gcf, 'color', 'w')
    set(gcf, 'Visible', 'on')
    % set(gcf, 'Visible', 'off')

    per_image_size = 300;
    m = numel(unique_sg_sensor_inds);
    n = Num_col;

    figure_width = per_image_size * Num_col * 1.5;
    figure_height = per_image_size * numel(unique_sg_sensor_inds) * 1.5;
    border_width = 0.3;
    border_hight = 0.08;

    set(gcf, 'Position', [1, 1, figure_width, figure_height]);
    axis off;

    cm = @jet;

    colors = cm(9);
    y_color = [0, 0, 0];

    figure_handles = {};

    for i = 1 : numel(unique_sg_sensor_inds)
        curr_sg_sensor_ind = unique_sg_sensor_inds(i);
        % inds = find(B(:, 1) == curr_sg_sensor_ind);
        sg_sensor = kept_sensor_names{curr_sg_sensor_ind};        
        curr_sg_region_inds = sg_region_inds;
        
        for j = 1 : 3
            cur_ind = (i - 1) * 3 + j;
            if any(j == curr_sg_region_inds)
                norm_plot = true;
            else
                norm_plot = ~false;
            end
            
            m_rate = 0.6;
            n_rate = 0.2;

            per_height = 1 / (m + ( m - 1) * m_rate);
            per_width = 1 / (n + (n - 1) * n_rate);

            % img_corner_1 = [(i - 1) * per_image_size + 1, [j - 1] * per_image_size + 1];
            % img_corner_2 = [i * per_image_size, j * per_image_size];

            curr_position = [per_width * ( 1 + n_rate) * (j - 1), per_height * ( 1 + m_rate) * (m - i) , per_width, per_height];
            curr_position(1) = curr_position(1) * (1 - border_width * 2) + border_width;
            curr_position(2) = curr_position(2) * (1 - border_hight * 2) + border_hight;
            curr_position(3) = curr_position(3) * (1 - border_width * 2);
            curr_position(4) = curr_position(4) * (1 - border_hight * 2);

            axes('Position', curr_position);
            
            curr_sg_region_ind = j;

            org_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, tt_X, numel(kept_node_names) / len_per_sensor);
            prd_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, B7_P_X, numel(kept_node_names) / len_per_sensor);
            if norm_plot
                h = plot(current_timepoint, org_sg_plot, 'Color', [0, 0, 0], 'linewidth', 2);   
            end
            xlim([current_timepoint(1), current_timepoint(end)])
            figure_handles{end + 1, 1} = h;
            % set(gca,'YTickLabel',[]);
            ylim_1=get(gca,'ylim');
            if i == 1
                title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 15);
            end
            hold on 

            % plot the predicted one
                            if norm_plot
            h = plot(current_timepoint(M + 1 : end), prd_sg_plot, ':r', 'LineWidth', 2);          
                            end
            figure_handles{end + 1, 2} = h;        
            xlim([current_timepoint(1), current_timepoint(end)])
            % set(gca,'YTickLabel',[]);
            ylim_2=get(gca,'ylim');
            % axis off
            ylim_scale = [0.4, 2.2];
            ylim(ylim_scale);
            ylim_int = 0.6;
            set(gca, 'ytick', ylim_scale(1) : ylim_int : ylim_scale(2));
            if ~norm_plot
                axis off;
                axes('Position', curr_position);
                xlim([current_timepoint(1), current_timepoint(end)])
                set(gca, 'ytick', ylim(1) : ylim_int : ylim(2));
            end
            % title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 10);
                        
            hold on
            
            if j == 1 && norm_plot 
                if cluster_num == 3
                    text(-220, sum(ylim) / 2, sprintf('%s', sg_sensor), 'fontsize', 12, 'fontweight', 'bold');
                else
                    text(-60, sum(ylim) / 2, sprintf('%s', sg_sensor));
                end
            end
            if ~norm_plot && j == 1
                text(-220, 0.5, sprintf('%s', sg_sensor), 'fontsize', 12, 'fontweight', 'bold');
                axis off;
            end
            if j ~= 1 &&  i < numel(unique_sg_sensor_inds) && norm_plot 
                set(gca,'xtick',[])
                set(gca,'ytick',[])
            end
            if j == 1 &&  i < numel(unique_sg_sensor_inds) && norm_plot 
                set(gca,'xtick',[])
            end
            if j ~= 1 && i == numel(unique_sg_sensor_inds) && norm_plot 
                set(gca,'ytick',[])
            end
            set(findobj(gca, '-property', 'fontsize'), 'fontsize', 14)
            
        end               
    end

    hold off 
    dpi = 300;
    figure_filename = sprintf('%sclustering_ROI_B7_protein_prediction_significant_kinetic_plots_1_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         
end


end
