function [t_cell_info] = master_script_CENR_build_model(t_cell_info, param)
% build CENR model, 
% copied from master_script_CGP_build_model.m
% 
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Mar. 16, 2017

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu





current_timepoint = param.current_timepoint;
lag_time = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;
suffix = param.suffix;

cv_data_location = t_cell_info.path_info.cv_data_location;
model_sub_location = t_cell_info.path_info.model_sub_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;

M = lag_time;

% Jan 23, 2016 automatically set work directory, because sometimes,
% /projects directory is also need to be used. 
illustration_colormap_to_use = t_cell_info.param.illustration_colormap_to_use;

graph_data = t_cell_info.graph_data;
kept_nodes = t_cell_info.graph_data.kept_nodes;
all_sensor_region_combine = graph_data.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);

X = graph_data.X;
tt_X = graph_data.tt_X;

optimal_param = t_cell_info.cv_info.optimal_param;
optimize_param.M = optimal_param.M;
optimize_param.lambda_1 = optimal_param.lambda_1;
optimize_param.lambda_2 = optimal_param.lambda_2;
optimize_param.lambda_3 = optimal_param.lambda_3;
optimize_param.alpha = optimal_param.alpha;
optimize_param.b = optimal_param.b;
optimize_param.self_prediction = optimal_param.self_prediction;
min_CV_P_pct_error = optimal_param.min_P_pct_error;

if false
    switch opt_std
        case 'CV_M2'
            
        case 'CV_M2_merge_214'
            optimize_param.lambda_1 = 1.258930e-03;
            optimize_param.lambda_2 = 1.000000e-02;          
            optimize_param.alpha = 0.3;
            optimize_param.b = -0.3;
    end   
end

% granger causality tests
gc_param = struct();
gc_param.alpha = optimize_param.alpha;
[~, h_mat, D] = cross_correlation_pair_granger_causality(X, M, gc_param);

if ~false
    figure, 
    set(gcf, 'Visible', 'off')
    set(gcf, 'Visible', 'on')
    set(gcf, 'Color', 'w')
    set(gcf, 'Renderer', 'opengl');
    figure_height = 1200;
    figure_width = 1500;
    set(gcf, 'Position', [1, 1, figure_width, figure_height]);
    imagesc(h_mat);
    title('Significant relationships after correction');
    xlabel('Cause')
    ylabel('Effect');
    kept_node_names = all_sensor_region_combine(kept_nodes);
    set(gca, 'YTick',  1 : size(X, 1), 'YTickLabel', kept_node_names);
    set(gca, 'XTick',  1 : size(X, 1), 'XTickLabel', kept_node_names);
    ylabel('Effect');
    xlabel('Cause');
    ax = gca;
    ax.FontWeight = 'bold';
    ax.FontSize = 18;
    ax.XTickLabelRotation=90;
    dpi = 150;
    figure_filename = sprintf('%spair_granger_chosen_pairs', figure_sub_location);
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);  
end

A = h_mat;
R_t_3d = A;
c = [0, 1];

param.optimize_param = optimize_param;
method = param.residual_compute_method;

tic,
[R_t_3d, A, c, f_mat, f_terms] = master_script_spatiotemporal_constraint_projected_admm(X, M, D, optimize_param.b, optimize_param);
toc
[W_err, W, W_pct_error, W_n_err] = CGP_residual_computing(R_t_3d, A, c, M, X, method);
[P_err, P_W, P_pct_error, P_n_err] = CGP_residual_computing(R_t_3d, A, c, M, tt_X, method);
% W_g_err = norm(W, 'fro') / sqrt(numel(W));
% W_gn_err = W_g_err ./ mean(mean(abs(X(:, M + 1 : end))))
% P_g_err = norm(P_W, 'fro') / sqrt(numel(P_W));
% P_gn_err = P_g_err ./ mean(mean(abs(tt_X(:, M + 1 : end))))

fprintf(' W_n: %4f, W_pct: %4f\n P_n: %4f, P_pct: %4f\n\n', W_n_err, W_pct_error, P_n_err, P_pct_error);

results_filename = sprintf('%sclustering_ROI_results_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d%s', model_sub_location, causal_method, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh, suffix);
save(results_filename, 'R_t_3d', 'A', 'c','M', 'X', 'tt_X', 'W_n_err', 'W_pct_error', 'W', 'P_W', 'P_pct_error', 'P_err', 'P_n_err', 'min_CV_P_pct_error', 'optimize_param');

model_info.optimize_param = optimize_param;
model_info.X = X;
model_info.tt_X = tt_X;
model_info.R_t_3d = R_t_3d;
model_info.A = A;
model_info.c = c;
model_info.min_CV_error = min_CV_P_pct_error;

t_cell_info.model_info = model_info;

end
