function [t_cell_info, param] = master_script_CGP_B7_testing_postprocess(t_cell_info, param)
% process p-values, make figures, tables. 

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



if nargin < 1
    t_cell_info = struct();
end

if nargin < 2
    param = struct();
end

param = process_options_structure( struct('current_timepoint', -40 : 5: 180, ...
                                          'time_interval', 5, ... 
                                          'causal_method', 'CGP', ...                                            
                                          'optm_method', 'original', ...
                                          'time_series_type', 'interpolation', ... 
                                          'interpolation_method', 'spline', ...                                          
                                          'M', 1, ...
                                          'opt_std', 'CV_M2', ...                                         
                                          'cluster_num', 3, ...
                                          'cluster_nums', 3, ...
                                          'cluster_type', 'kmeans', ...
                                          'cluster_sensor', 'all', ...
                                          'data_type', '1pt', ...
                                          'vol_thresh', 4, ...
                                          'work_dir_setting', '1pt_synpase_correction', ...
                                          'computing_sufix', '_12202016001', ...
                                           'suffix', '', ...
                                          'N_test', 10000000, ...
                                          'N_perfile', 500, ...
                                          'p_value_method', 'empirical', ...  
                                          'computing_models', true, ...
                                          'compute_p_value', ~false ...
                                      ), param);
                                        
current_timepoint = param.current_timepoint;
M = param.M;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
work_dir_setting = param.work_dir_setting;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
optm_method = param.optm_method;
opt_std = param.opt_std;
causal_method = param.causal_method;
computing_sufix = param.computing_sufix;
suffix = param.suffix;
time_series_type = param.time_series_type;
interpolation_method = param.interpolation_method;
N_test = param.N_test;
N_perfile = param.N_perfile;
p_value_method = param.p_value_method;
computing_models = param.computing_models;



model_means =  t_cell_info.model_data.model_means;
unique_sensor_names = t_cell_info.model_data.unique_sensor_names;
used_model_names = t_cell_info.model_data.used_model_names;
used_model_reltime = t_cell_info.model_data.used_model_reltime;
used_model_conditions = t_cell_info.model_data.used_model_conditions;

model_info = t_cell_info.model_info;
kept_nodes = model_info.kept_nodes;
all_sensor_region_combine = model_info.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);
kept_sensor_names =unique(cellfun(@(x) x{1}{1}, regexp(kept_node_names, '(^[^\s^-]+)', 'tokens'), 'Uniformoutput', false));


illustration_colormap_to_use = pmkmp(1024, 'CubicL'); 

dir_info = param.dir_info;
home_dir = dir_info.home_dir;
model_dir = dir_info.model_dir;
work_dir = dir_info.work_dir;
work_cluster_sensor_dir = dir_info.work_cluster_sensor_dir;
work_subdir = dir_info.work_subdir;
figure_dir = sprintf('%sfigures_OPT_%s_std_%s_volthresh-%d%s/', work_subdir, optm_method, opt_std, vol_thresh, suffix);
if ~exist(figure_dir, 'dir')
    mkdir(figure_dir);
end

B7_testing_dir = [work_subdir, 'B7_testing/'];
if ~exist(B7_testing_dir, 'dir')
    mkdir(B7_testing_dir);
end

B7_testing_subdir = [B7_testing_dir, 'permutation_test', computing_sufix, '/'];
if ~exist(B7_testing_subdir, 'dir')
    mkdir(B7_testing_subdir);
end

% total data point for training (building the model)
m = numel(current_timepoint);

if ~strcmp(opt_std, 'toy_data')
    [X, tt_X] = Full_B7_data_extraction(param);
else
    N = 36;
    T = 100;
    flag = 3;
    [X, A_M_3d, Omg, A_M, c_M] = RandDataGenerator_graphs(M, N, T, flag);
    [tt_X, A_M_3d, Omg, A_M, c_M] = RandDataGenerator_graphs(M, N, T, flag);    
    X = X(:, 5 : 50);
    tt_X = tt_X(:, 5 : 50);
end

N = size(X, 1);

testing_info = t_cell_info.testing_info;
p_value_upper = testing_info.p_value_upper;
p_value_lower = testing_info.p_value_lower;
p_value_tail = testing_info.p_value_tail;

results_filename = sprintf('%sclustering_ROI_results_OPT_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d%s', work_subdir, optm_method, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh, suffix);
% save(results_filename, 'R_t_3d', 'A', 'c','M', 'X');
% save([work_subdir, 'R_A_c.mat'], 'R_t_3d', 'A', 'c', 'M', 'X');

results = load(results_filename);
A = results.A;
try
    c = results.c;
    R_t_3d = results.R_t_3d;
end
if ~strcmp(opt_std, 'toy_data')
    [X, tt_X] = Full_B7_data_extraction(param);
else
    N = 36;
    T = 100;
    flag = 3;
    [X, A_M_3d, Omg, A_M, c_M] = RandDataGenerator_graphs(M, N, T, flag);
    [tt_X, A_M_3d, Omg, A_M, c_M] = RandDataGenerator_graphs(M, N, T, flag);    
    X = X(:, 5 : 50);
    tt_X = tt_X(:, 5 : 50);
end

kept_nodes = 1 : size(X, 1);
X = X(kept_nodes, :);
tt_X = tt_X(kept_nodes, :);

switch opt_std
    case 'CV_M2_merge_214'
        param.merge_proteins = true;
        param.group_inds = {[1, 7], 2, [3, 5, 11, 12], 4, 6, 8, 9, 10};
        param.sufix = sprintf('_CGP_CV_M2_merge_214_01222017001');
        param.computing_sufix = sprintf('_CGP_CV_M2_merge_214_01222017001');
    case 'CV_M2_merge_7'
        param.merge_proteins = true;
        param.group_inds = {[1, 2, 3, 5, 7, 11, 12], 4, 6, 8, 9, 10};
    case 'CV_M2_merge_25'
        param.merge_proteins = true;
        param.group_inds = {[1, 7], [2, 3, 5, 11, 12], 4, 6, 8, 9, 10};
end

if param.merge_proteins
    % by default, merge (cpalpha1, wasp, coronin1A,
    % wave2), actin, (arp3, hs1), cofilin, ezrin, pip2,
    % lat, mrlc 
    % we use th mean of the training and validation data.
    region_num = cluster_num;
    group_inds = param.group_inds;
    X_new = zeros(region_num, numel(group_inds), size(X, 2));
    tt_X_new = zeros(region_num, numel(group_inds), size(tt_X, 2));
    X_reshape = reshape(X, region_num, [], size(X, 2));
    tt_X_reshape = reshape(tt_X, region_num, [], size(tt_X, 2));
    kept_nodes = zeros(region_num, numel(group_inds));
    for i1 = 1 : numel(group_inds)
        X_new(:, i1, :) = mean(X_reshape(:, group_inds{i1}, :), 2);
        tt_X_new(:, i1, :) = mean(tt_X_reshape(:, group_inds{i1}, :), 2);  
        kept_nodes(:, i1) = (1 : region_num) + min(group_inds{i1} - 1) * region_num;
    end
    % just keep them for tracking or debuging

    X = reshape(X_new, [], size(X_new, 3));
    tt_X = reshape(tt_X_new, [], size(tt_X_new, 3));
    kept_nodes = reshape(kept_nodes, [], 1);
end
N = size(X, 1);
kept_node_names = all_sensor_region_combine(kept_nodes);

len_per_sensor = cluster_num;

method = 2;
% just directly compute Err 
[~, W] = CGP_residual_computing(R_t_3d, A, c, M, X, method);            
W_err = sqrt(sum(W .^ 2, 2)) / sqrt(size(W, 2));
W_n_err = W_err ./ (mean(abs(X(:, M + 1 : end)), 2));
[~, P_W,~, B7_P_X] = CGP_residual_computing(R_t_3d, A, c, M, tt_X, method);
P_err = sqrt(sum(P_W .^ 2, 2)) / sqrt(size(P_W, 2));
P_n_err = P_err ./ (mean(abs(tt_X(:, M + 1 : end)), 2));
P_pct_err = sqrt(mean((P_W ./ abs(tt_X(:, M + 1 : end))) .^ 2, 2));
Err_predict = P_pct_err;

% permutation testing just computing all files first. 

% Visualize p-value time series. 
% we only consider the left-side p-values

[corrected_p_low, h_low] = bonf_holm(p_value_lower);
Err_lower = Err_predict .* h_low;

sg_inds = find(h_low == 1);
sg_pvalues = corrected_p_low(sg_inds);
sg_thrsh = max(Err_predict(h_low == 1));
pvalue_sg_thrsh_org = max(p_value_lower(h_low == 1));
pvalue_sg_thrsh = max(sg_pvalues);

[sg_region_inds, sg_sensor_inds] = ind2sub([len_per_sensor, numel(kept_node_names) / len_per_sensor], sg_inds);

unique_sg_sensor_inds = unique(sg_sensor_inds);
Num_col = max(histc(sg_sensor_inds, unique_sg_sensor_inds));

[B, sort_inds] = sortrows([sg_sensor_inds, sg_region_inds]);

if ~isempty(sort_inds)
    figure, 
    set(gcf, 'color', 'w')
    set(gcf, 'Visible', 'on')
    % set(gcf, 'Visible', 'off')

    per_image_size = 300;
    m = numel(unique_sg_sensor_inds);
    n = Num_col;

    figure_width = per_image_size * Num_col * 1.5;
    figure_height = per_image_size * numel(unique_sg_sensor_inds) * 1.5;
    border_width = 0.3;
    border_hight = 0.08;

    set(gcf, 'Position', [1, 1, figure_width, figure_height]);
    axis off;

    cm = @jet;

    colors = cm(9);
    y_color = [0, 0, 0];

    figure_handles = {};

    for i = 1 : numel(unique_sg_sensor_inds)
        curr_sg_sensor_ind = unique_sg_sensor_inds(i);
        inds = find(B(:, 1) == curr_sg_sensor_ind);
        sg_sensor = kept_sensor_names{curr_sg_sensor_ind};
        
        curr_sg_region_inds = B(inds, 2);

        % text(-0.2, 1 - i * 0.1 + 0.2, pos_y_sensor);

        for j = 1 : numel(inds)

            curr_sg_region_ind = curr_sg_region_inds(j);
            
            org_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, tt_X, numel(kept_node_names) / len_per_sensor);
            prd_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, B7_P_X, numel(kept_node_names) / len_per_sensor);
            
            m_rate = 0.7;
            n_rate = 0.3;

            per_height = 1 / (m + ( m - 1) * m_rate);
            per_width = 1 / (n + (n - 1) * n_rate);

            % img_corner_1 = [(i - 1) * per_image_size + 1, [j - 1] * per_image_size + 1];
            % img_corner_2 = [i * per_image_size, j * per_image_size];

            curr_position = [per_width * ( 1 + n_rate) * (j - 1), per_height * ( 1 + m_rate) * (m - i) , per_width, per_height];
            curr_position(1) = curr_position(1) * (1 - border_width * 2) + border_width;
            curr_position(2) = curr_position(2) * (1 - border_hight * 2) + border_hight;
            curr_position(3) = curr_position(3) * (1 - border_width * 2);
            curr_position(4) = curr_position(4) * (1 - border_hight * 2);

            axes('Position', curr_position);

            h = plot(current_timepoint, org_sg_plot, 'Color', [0, 0, 0], 'linewidth', 2);    
            xlim([current_timepoint(1), current_timepoint(end)])
            figure_handles{end + 1, 1} = h;
            % set(gca,'YTickLabel',[]);
            ylim_1=get(gca,'ylim');
            if i == 1
                title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 15);
            end
            hold on 
            
            % plot the predicted one
            h = plot(current_timepoint(M + 1 : end), prd_sg_plot, ':r', 'LineWidth', 2);              
            figure_handles{end + 1, 2} = h;        
            xlim([current_timepoint(1), current_timepoint(end)])
            % set(gca,'YTickLabel',[]);
            ylim_2=get(gca,'ylim');
            % axis off
            ylim_method = 'same_region';
            
            switch ylim_method
                case 'original'
                    ylim = [min([ylim_1, ylim_2]), max([ylim_1, ylim_2])];
                case 'same_region' 
                    data = [reshape(B7_P_X, 3, []), reshape(tt_X, 3, [])];
                    ylims = [min(data, [], 2), max(data, [], 2)];
                    ylim = ylims(j, :);                
                case 'all_regions'
                    ylim = [min([tt_X(:); B7_P_X(:)]), max([tt_X(:); B7_P_X(:)])];                    
            end
            ylim(:, 1) = floor(ylim(:, 1) * 10) / 10;
            ylim(:, 2) = ceil(ylim(:, 2) * 10) / 10;
            
            set(gca, 'ylim', ylim)
            
            ylim =get(gca,'ylim');
            if diff(ylim) / 0.1 <= 2
                ylim_int = 0.1;
            elseif diff(ylim) / 0.1 <= 4
                ylim_int = 0.15;
            elseif diff(ylim) / 0.1 <= 7
                 ylim_int = 0.2;
            else
                 ylim_int = 0.3;
            end        
            set(gca, 'ytick', ylim(1) : ylim_int : ylim(2));
            
            % title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 10);
                        
            hold on
            
            if j == 1
                if cluster_num == 3
                    text(-220, sum(ylim) / 2, sprintf('%s', sg_sensor), 'fontsize', 12, 'fontweight', 'bold');
                else
                    text(-60, sum(ylim) / 2, sprintf('%s', sg_sensor));
                end
            end
           
        end               
    end
    
    hold off 
    dpi = 150;
    figure_filename = sprintf('%sclustering_ROI_B7_prediction_significant_kinetic_plots_k-%d_M_%d_vol_thrsd_%d', figure_dir, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         
end
% hack version for the paper
if ~isempty(sort_inds)
    figure, 
    set(gcf, 'color', 'w')
    set(gcf, 'Visible', 'on')
    % set(gcf, 'Visible', 'off')

    per_image_size = 300;
    m = numel(unique_sg_sensor_inds);
    n = Num_col;

    figure_width = per_image_size * Num_col * 1.5;
    figure_height = per_image_size * numel(unique_sg_sensor_inds) * 1.5;
    border_width = 0.3;
    border_hight = 0.08;

    set(gcf, 'Position', [1, 1, figure_width, figure_height]);
    axis off;

    cm = @jet;

    colors = cm(9);
    y_color = [0, 0, 0];

    figure_handles = {};

    for i = 1 : numel(unique_sg_sensor_inds)
        curr_sg_sensor_ind = unique_sg_sensor_inds(i);
        inds = find(B(:, 1) == curr_sg_sensor_ind);
        sg_sensor = kept_sensor_names{curr_sg_sensor_ind};
        
        curr_sg_region_inds = B(inds, 2);

        % text(-0.2, 1 - i * 0.1 + 0.2, pos_y_sensor);

        for j = 1 : 3
            cur_ind = (i - 1) * 3 + j;
            if any(j == curr_sg_region_inds)
                norm_plot = true;
            else
                norm_plot = ~false;
            end
            
            m_rate = 0.7;
            n_rate = 0.3;

            per_height = 1 / (m + ( m - 1) * m_rate);
            per_width = 1 / (n + (n - 1) * n_rate);

            % img_corner_1 = [(i - 1) * per_image_size + 1, [j - 1] * per_image_size + 1];
            % img_corner_2 = [i * per_image_size, j * per_image_size];

            curr_position = [per_width * ( 1 + n_rate) * (j - 1), per_height * ( 1 + m_rate) * (m - i) , per_width, per_height];
            curr_position(1) = curr_position(1) * (1 - border_width * 2) + border_width;
            curr_position(2) = curr_position(2) * (1 - border_hight * 2) + border_hight;
            curr_position(3) = curr_position(3) * (1 - border_width * 2);
            curr_position(4) = curr_position(4) * (1 - border_hight * 2);

            axes('Position', curr_position);
            
                curr_sg_region_ind = j;

                org_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, tt_X, numel(kept_node_names) / len_per_sensor);
                prd_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, B7_P_X, numel(kept_node_names) / len_per_sensor);
                if norm_plot
                    h = plot(current_timepoint, org_sg_plot, 'Color', [0, 0, 0], 'linewidth', 2);   
                end
                xlim([current_timepoint(1), current_timepoint(end)])
                figure_handles{end + 1, 1} = h;
                % set(gca,'YTickLabel',[]);
                ylim_1=get(gca,'ylim');
                if i == 1
                    title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 15);
                end
                hold on 

                % plot the predicted one
                                if norm_plot
                h = plot(current_timepoint(M + 1 : end), prd_sg_plot, ':r', 'LineWidth', 2);          
                                end
                figure_handles{end + 1, 2} = h;        
                xlim([current_timepoint(1), current_timepoint(end)])
                % set(gca,'YTickLabel',[]);
                ylim_2=get(gca,'ylim');
                % axis off
                ylim_method = 'same_region';

                switch ylim_method
                    case 'original'
                        ylim = [min([ylim_1, ylim_2]), max([ylim_1, ylim_2])];
                    case 'same_region' 
                        data = [reshape(B7_P_X, 3, []), reshape(tt_X, 3, [])];
                        ylims = [min(data, [], 2), max(data, [], 2)];
                        ylim = ylims(j, :);                
                    case 'all_regions'
                        ylim = [min([tt_X(:); B7_P_X(:)]), max([tt_X(:); B7_P_X(:)])];                    
                end
                ylim(:, 1) = floor(ylim(:, 1) * 10) / 10;
                ylim(:, 2) = ceil(ylim(:, 2) * 10) / 10;

                set(gca, 'ylim', ylim)

                ylim =get(gca,'ylim');
                if diff(ylim) / 0.1 <= 2
                    ylim_int = 0.1;
                elseif diff(ylim) / 0.1 <= 4
                    ylim_int = 0.15;
                elseif diff(ylim) / 0.1 <= 7
                     ylim_int = 0.2;
                else
                     ylim_int = 0.3;
                end        
                set(gca, 'ytick', ylim(1) : ylim_int : ylim(2));
            if ~norm_plot
                axis off;
                axes('Position', curr_position);
                xlim([current_timepoint(1), current_timepoint(end)])
                set(gca, 'ytick', ylim(1) : ylim_int : ylim(2));
            end
            % title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 10);
                        
            hold on
            
            if j == 1 && norm_plot 
                if cluster_num == 3
                    text(-220, sum(ylim) / 2, sprintf('%s', sg_sensor), 'fontsize', 12, 'fontweight', 'bold');
                else
                    text(-60, sum(ylim) / 2, sprintf('%s', sg_sensor));
                end
            end
            if ~norm_plot && j == 1
                text(-220, 0.5, sprintf('%s', sg_sensor), 'fontsize', 12, 'fontweight', 'bold');
                axis off;
            end
        end               
    end
    
    hold off 
    dpi = 150;
    figure_filename = sprintf('%sclustering_ROI_B7_prediction_significant_kinetic_plots_1_k-%d_M_%d_vol_thrsd_%d', figure_dir, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         
end

if ~isempty(sort_inds)
    figure, 
    set(gcf, 'color', 'w')
    set(gcf, 'Visible', 'on')
    % set(gcf, 'Visible', 'off')

    per_image_size = 300;
    m = numel(unique_sg_sensor_inds);
    n = Num_col;

    figure_width = per_image_size * Num_col * 1.5;
    figure_height = per_image_size * numel(unique_sg_sensor_inds) * 1.5;
    border_width = 0.3;
    border_hight = 0.08;

    set(gcf, 'Position', [1, 1, figure_width, figure_height]);
    axis off;

    cm = @jet;

    colors = cm(9);
    y_color = [0, 0, 0];

    figure_handles = {};

    for i = 1 : numel(unique_sg_sensor_inds)
        curr_sg_sensor_ind = unique_sg_sensor_inds(i);
        inds = find(B(:, 1) == curr_sg_sensor_ind);
        sg_sensor = kept_sensor_names{curr_sg_sensor_ind};
        
        curr_sg_region_inds = B(inds, 2);

        % text(-0.2, 1 - i * 0.1 + 0.2, pos_y_sensor);

        for j = 1 : numel(inds)

            curr_sg_region_ind = curr_sg_region_inds(j);
            
            org_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, tt_X, numel(kept_node_names) / len_per_sensor);
            prd_sg_plot = kinetic_extract(curr_sg_sensor_ind, curr_sg_region_ind, B7_P_X, numel(kept_node_names) / len_per_sensor);
            
            m_rate = 0.7;
            n_rate = 0.3;

            per_height = 1 / (m + ( m - 1) * m_rate);
            per_width = 1 / (n + (n - 1) * n_rate);

            % img_corner_1 = [(i - 1) * per_image_size + 1, [j - 1] * per_image_size + 1];
            % img_corner_2 = [i * per_image_size, j * per_image_size];

            curr_position = [per_width * ( 1 + n_rate) * (j - 1), per_height * ( 1 + m_rate) * (m - i) , per_width, per_height];
            curr_position(1) = curr_position(1) * (1 - border_width * 2) + border_width;
            curr_position(2) = curr_position(2) * (1 - border_hight * 2) + border_hight;
            curr_position(3) = curr_position(3) * (1 - border_width * 2);
            curr_position(4) = curr_position(4) * (1 - border_hight * 2);

            axes('Position', curr_position);

            h = plot(current_timepoint, org_sg_plot, 'Color', [0, 0, 0], 'linewidth', 2);    
            xlim([current_timepoint(1), current_timepoint(end)])
            figure_handles{end + 1, 1} = h;
            % set(gca,'YTickLabel',[]);
            ylim_1=get(gca,'ylim');
            if i == 1
                title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 15);
            end
            hold on 
            
            % plot the predicted one
            h = plot(current_timepoint(M + 1 : end), prd_sg_plot, ':r', 'LineWidth', 2);              
            figure_handles{end + 1, 2} = h;        
            xlim([current_timepoint(1), current_timepoint(end)])
            % set(gca,'YTickLabel',[]);
            ylim_2=get(gca,'ylim');
            % axis off
            set(gca, 'ylim', [min([ylim_1, ylim_2]), max([ylim_1, ylim_2])])
            ylim =get(gca,'ylim');
            if diff(ylim) / 0.1 <= 3
                ylim_int = 0.1;
            elseif diff(ylim) / 0.1 <= 5
                ylim_int = 0.15;
            elseif diff(ylim) / 0.1 <= 7
                 ylim_int = 0.2;
            else
                 ylim_int = 0.3;
            end        
            set(gca, 'ytick', ylim(1) : ylim_int : ylim(2));
            
            % title(sprintf('Region %d', curr_sg_region_ind), 'FontSize', 10);
                        
            hold on
            
            if j == 1
                if cluster_num == 3
                    text(-220, sum(ylim_1) / 2, sprintf('%s', sg_sensor), 'fontsize', 12, 'fontweight', 'bold');
                else
                    text(-60, sum(ylim_1) / 2, sprintf('%s', sg_sensor));
                end
            end
           
        end               
    end
    
    hold off 
    dpi = 150;
    figure_filename = sprintf('%sclustering_ROI_B7_prediction_significant_kinetic_plots_2_k-%d_M_%d_vol_thrsd_%d', figure_dir, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         
end

% plot the contribution of all sensors to the significant nodes. 
N_sg = numel(sg_inds);

figure,
set(gcf, 'color', 'w')
set(gcf, 'Visible', 'on')
per_image_size = 150;
figure_width = per_image_size  * 10;
figure_height = per_image_size * N_sg * 1.5;
set(gcf, 'Position', [1, 1, figure_width, figure_height]);

for i = 1 : N_sg
    subplot(N_sg, 1, i);
    cur_ind = sg_inds(i);
    cur_causes = squeeze(R_t_3d(i, :, :));
    [cur_region_ind, cur_sensor_ind] = ind2sub([len_per_sensor, numel(kept_node_names) / len_per_sensor], cur_ind);
    
    bar(cur_causes);
    title(sprintf('%s', kept_node_names{cur_ind}), 'FontSize', 10); 
    set(gca, 'XTick',  len_per_sensor / 2 + [0 : numel(kept_node_names)-1] * len_per_sensor, 'XTickLabel', kept_node_names, 'XTickLabelRotation', 45);

end
dpi = 150;

figure_filename = sprintf('%sclustering_ROI_B7_prediction_significant_causes_k-%d_M_%d_vol_thrsd_%d', figure_dir, cluster_num, M, vol_thresh);
export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         



N_sensor = numel(kept_sensor_names);
% write errors csv files. 
csv_field_matrix = {};
Err_cell = num2cell(reshape(Err_predict, len_per_sensor, []));
csv_field_matrix{1, 1} = 'Region';
csv_field_matrix(1, 2 : N_sensor + 1) = cellfun(@(x) x, kept_sensor_names, 'UniformOutput', false);
csv_field_matrix(2 : len_per_sensor + 1, 1) = mat2cell([1 : len_per_sensor]', ones(len_per_sensor, 1));
csv_field_matrix(2 : len_per_sensor + 1, 2 : N_sensor + 1) = Err_cell;

table_filename = sprintf('%sclustering_ROI_B7_prediction_error_table_sg_level-%f_k-%d_M_%d_vol_thrsd_%d.csv', figure_dir, sg_thrsh, cluster_num, M, vol_thresh);

fid = fopen(table_filename, 'wt');
for i = 1: size(csv_field_matrix, 1)
    for j = 1 : size(csv_field_matrix, 2)
        if j > 1
            fprintf(fid, ',');
        end
        if ischar(csv_field_matrix{i, j})
          fwrite(fid, csv_field_matrix{i, j});
        else
          fprintf(fid, '%e', csv_field_matrix{i, j});
        end        
    end
    fprintf(fid, '\n');
end
fclose(fid);

pause(5);
% write original p-value csv files
csv_field_matrix = {};
pvalue_cell = mat2cell(reshape(p_value_lower, len_per_sensor, []), ones(1, len_per_sensor), ones(1 , N_sensor));
csv_field_matrix{1, 1} = 'Region';
% add region number for the first column
csv_field_matrix(1, 2 : N_sensor + 1) = cellfun(@(x) x, kept_sensor_names, 'UniformOutput', false);
% add sensor names
csv_field_matrix(2 : len_per_sensor + 1, 1) = mat2cell([1 : len_per_sensor]', ones(len_per_sensor, 1));
% add p-values
csv_field_matrix(2 : len_per_sensor + 1, 2 : N_sensor + 1) = pvalue_cell;

table_filename_1 = sprintf('%sclustering_ROI_B7_prediction_pvalue_table_sg_level-%.3f_k-%d_M_%d_vol_thrsd_%d.csv', figure_dir, pvalue_sg_thrsh_org, cluster_num, M, vol_thresh);

fid = fopen(table_filename_1, 'wt');
for i = 1: size(csv_field_matrix, 1)
    for j = 1 : size(csv_field_matrix, 2)
        if j > 1
            fprintf(fid, ',');
        end
        if ischar(csv_field_matrix{i, j})
          fwrite(fid, csv_field_matrix{i, j});
        else
          fprintf(fid, '%e', csv_field_matrix{i, j});
        end        
    end
    fprintf(fid, '\n');
end
fclose(fid);

pause(5);
% write corrected p-value csv files
csv_field_matrix = {};
pvalue_cell = num2cell(reshape(corrected_p_low, len_per_sensor, []));
% add region number for the first column
csv_field_matrix{1, 1} = 'Region';
csv_field_matrix(1, 2 : N_sensor + 1) = cellfun(@(x) x, kept_sensor_names, 'UniformOutput', false);
% add sensor names
csv_field_matrix(2 : len_per_sensor + 1, 1) = mat2cell([1 : len_per_sensor]', ones(len_per_sensor, 1));
% add pvalues
csv_field_matrix(2 : len_per_sensor + 1, 2 : N_sensor + 1) = pvalue_cell;

table_filename_2 = sprintf('%sclustering_ROI_B7_prediction_BH-pvalue_table_sg_level-%.3f_k-%d_M_%d_vol_thrsd_%d.csv', figure_dir, pvalue_sg_thrsh, cluster_num, M, vol_thresh);

fid = fopen(table_filename_2, 'wt');
for i = 1: size(csv_field_matrix, 1)
    for j = 1 : size(csv_field_matrix, 2)
        if j > 1
            fprintf(fid, ',');
        end
        if ischar(csv_field_matrix{i, j})
          fwrite(fid, csv_field_matrix{i, j});
        else
          fprintf(fid, '%e', csv_field_matrix{i, j});
        end        
    end
    fprintf(fid, '\n');
end
fclose(fid);



end
