function [t_cell_info, param] = master_script_CGP_Full_testing(t_cell_info, param)
% choose the pairs that have larges xcorr with at least one time lag. And
% then test the granger causality between these pairs. Because we don't
% have enough data, we don't include the background information in the
% Granger causality testing. 

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




% Author: Xiongtao Ruan
% Date: 11/07/2016

if nargin < 1
    t_cell_info = struct();
end

if nargin < 2
    param = struct();
end

param = process_options_structure( struct('current_timepoint', -40 : 5: 180, ...
                                          'time_interval', 5, ... 
                                          'causal_method', 'CGP', ...                                            
                                          'optm_method', 'original', ...
                                          'time_series_type', 'interpolation', ... 
                                          'interpolation_method', 'spline', ...                                          
                                          'M', 1, ...
                                          'opt_std', 'CV_M2', ...                                         
                                          'cluster_num', 3, ...
                                          'cluster_nums', 3, ...
                                          'cluster_type', 'kmeans', ...
                                          'cluster_sensor', 'all', ...
                                          'data_type', '1pt', ...
                                          'vol_thresh', 4, ...
                                          'work_dir_setting', '1pt_synpase_correction', ...
                                          'computing_sufix', '_12202016001', ...
                                          'N_test', 500000, ...
                                          'N_perfile', 500, ...
                                          'p_value_method', 'empirical', ...  
                                          'computing_models', true, ...
                                          'compute_p_value', ~false ...
                                      ), param);
                                        
current_timepoint = param.current_timepoint;
M = param.M;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
work_dir_setting = param.work_dir_setting;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
optm_method = param.optm_method;
opt_std = param.opt_std;
causal_method = param.causal_method;
computing_sufix = param.computing_sufix;
time_series_type = param.time_series_type;
interpolation_method = param.interpolation_method;
N_test = param.N_test;
N_perfile = param.N_perfile;
p_value_method = param.p_value_method;
computing_models = param.computing_models;

% Jan 23, 2016 automatically set work directory, because sometimes,
% /projects directory is also need to be used. 

filepath = which(mfilename);
paths = regexp(filepath, 'code', 'split');
home_dir = paths{1};

model_dirset = {'/projects/cellorganizer/xruan/tcell_project/tcell_excel_results_2pt_simulation_85_sn_st/two-point_landmark-smoothing-method_none_intensity-morphing-method_none_models/', ...
    '/projects/cellorganizer/xruan/tcell_project/intermediate_results_final/tcell_excel_results_1pt_repro_85_sn_st/landmark-smoothing-method_none_intensity-morphing-method_none_models/'};
cellorganizer_dirset = {'~/cellorganizer_bu', '~/cellorganizer'};
formatOut = 'yyyymmdd HH:MM:SS';

for i = 1 : length(model_dirset)
    if exist(model_dirset{i}, 'dir')
        model_dir = model_dirset{i};
        break;
    end
end
for i = 1 : length(cellorganizer_dirset)
    if exist(cellorganizer_dirset{i}, 'dir')
        cellorganizer_dir = cellorganizer_dirset{i};
        break;
    end
end
% addpath(genpath('~/cellorganizer'));
% addpath(genpath(home_dir));
% addpath(genpath([home_dir, filesep, 'code']));
addpath(genpath(['/projects/cellorganizer/xruan/tcell_project/tcell_clean/']));
% addpath(genpath(cellorganizer_dir));

illustration_colormap_to_use = pmkmp(1024, 'CubicL'); 

segmentation_dir = [home_dir, 'segmentation/'];
if ~exist(segmentation_dir, 'dir')
    mkdir(segmentation_dir);
end

% if strcmp(work_dir_setting, 'unnormalized')
%     work_dir = [home_dir, 'results/per_voxel_CGP_temporal_unnormalized/'];
% elseif strcmp(work_dir_setting, 'revise')
%     work_dir = [home_dir, 'results/per_voxel_CGP_temporal_revise/'];
% else
%     work_dir = [home_dir, 'results/per_voxel_CGP_temporal/'];
% end
data_dir = sprintf('%sdata/', home_dir);
mkdir(data_dir);
data_subdir = sprintf('%sTS_interpolation_method_%s_10162016/', data_dir, interpolation_method);
mkdir(data_subdir);

work_dir = [home_dir, 'results/per_voxel_CGP_temporal_', work_dir_setting, '/'];

if ~exist(work_dir, 'dir')
    mkdir_recursive(work_dir);
end

work_dir = sprintf('%s%s/', work_dir, cluster_type);

if ~exist(work_dir, 'dir')
    mkdir_recursive(work_dir);
end

work_cluster_sensor_dir = sprintf('%scausal_method_%s_cluster_sensor_%s_%s/', work_dir, causal_method, cluster_sensor, data_type);
if ~exist(work_cluster_sensor_dir, 'dir')
    mkdir_recursive(work_cluster_sensor_dir);
end

work_subdir = sprintf('%smethod-%s_num_%d_lagtime-%d/', work_cluster_sensor_dir, cluster_type, cluster_num, M);
if ~exist(work_subdir, 'dir')
    mkdir(work_subdir);
end

testing_dir = [work_subdir, 'Full_testing/'];
if ~exist(testing_dir, 'dir')
    mkdir(testing_dir);
end

testing_subdir = [testing_dir, 'permutation_test', computing_sufix, '/'];
if ~exist(testing_subdir, 'dir')
    mkdir(testing_subdir);
end

dir_info = struct();
dir_info.home_dir = home_dir;
dir_info.data_dir = data_dir;
dir_info.data_subdir = data_subdir;
dir_info.model_dir = model_dir;
dir_info.work_dir = work_dir;
dir_info.work_cluster_sensor_dir = work_cluster_sensor_dir;
dir_info.work_subdir = work_subdir;
dir_info.testing_dir = testing_dir;

param.dir_info = dir_info;
if isempty(fieldnames(t_cell_info))
    t_cell_info_filename = sprintf('t_cell_info_%s_method_%s.mat', time_series_type, interpolation_method);
    t_cell_info_fullpath =[work_dir, t_cell_info_filename];

    if exist(t_cell_info_fullpath, 'file')
        load(t_cell_info_fullpath);
    elseif exist([home_dir, 'code/', t_cell_info_filename], 'file')
        load([home_dir, 'code/', t_cell_info_filename]);
    elseif strcmp(time_series_type, 'original')
        [t_cell_info] = master_script_get_model_data([], param);
        save(t_cell_info_fullpath, 't_cell_info');
    elseif strcmp(time_series_type, 'interpolation')
        [t_cell_info] = master_script_get_model_time_series_data([], param);
        save(t_cell_info_fullpath, 't_cell_info');   
    end

    % total data point for training (building the model)
    m = numel(current_timepoint);

    current_condition = 'Full Stimulus';

    model_means =  t_cell_info.model_data.model_means;
    unique_sensor_names = t_cell_info.model_data.unique_sensor_names;
    used_model_names = t_cell_info.model_data.used_model_names;
    used_model_reltime = t_cell_info.model_data.used_model_reltime;
    used_model_conditions = t_cell_info.model_data.used_model_conditions;
end

if ~strcmp(opt_std, 'toy_data')
    [X, tt_X] = Full_B7_data_extraction(param);
else
    N = 36;
    T = 100;
    flag = 3;
    [X, A_M_3d, Omg, A_M, c_M] = RandDataGenerator_graphs(M, N, T, flag);
    [tt_X, A_M_3d, Omg, A_M, c_M] = RandDataGenerator_graphs(M, N, T, flag);    
    X = X(:, 5 : 50);
    tt_X = tt_X(:, 5 : 50);
end

N = size(X, 1);

switch opt_std
    case 'CV_M2'
        % original time points 
        optimize_param.lambda_1 = 6.309570e-03;
        optimize_param.lambda_2 = 1.584893e-02;
        optimize_param.lambda_3 = 1.995300e-04;
        optimize_param.self_prediction = false;         
    case 'CV_M2_1'
        optimize_param.lambda_1 = 1.122000e-04;
        optimize_param.lambda_2 = 0;          
        optimize_param.self_prediction = false;        
      
end
% computing_models = true;

N_per_partition = N_perfile;
N_total = N_test;
N_partition = ceil(N_test / N_perfile);


s = RandStream('mt19937ar', 'Seed', 2390785);
RandStream.setGlobalStream(s);

Rand_seeds = round(randperm(round(N_partition * 1.2))' * 2565 + (rand(round(N_partition * 1.2), 1) - 0.5) * 1000);
Rand_seeds = unique(Rand_seeds);
Rand_seeds = Rand_seeds(1 : N_partition);

if computing_models
    % causal graph process 
    % X = reshape(permute(mean_t_downsample(:, 1 : 10, :), [1, 3, 2]), [], 10);
    testing_result_subdir = sprintf('%sMat_files/', testing_subdir);
    if ~exist(testing_result_subdir, 'dir')
        mkdir(testing_result_subdir);
    end
    
    options = struct();            
    if isfield(param, 'computing_options')
        options = param.computing_options;
    end
    options.M = M;

    options.lambda_1 = optimize_param.lambda_1;
    options.lambda_2 = optimize_param.lambda_2;
    options.lambda_3 = optimize_param.lambda_3;
    options.self_prediction = optimize_param.self_prediction;
    % use block partitioning for the computing. 

    chunk_lock_clean(testing_result_subdir, N_perfile * 0.1);
    for k0 = 1 : N_partition
        s = RandStream('mt19937ar', 'Seed', Rand_seeds(k0));
        RandStream.setGlobalStream(s);
        % rng(Rand_seeds(k0));
        
        k_beg = (k0 - 1) * N_per_partition + 1;
        k_end = min(N_total, k0 * N_per_partition);

        results_filename = sprintf('%sclustering_ROI_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.mat', testing_result_subdir, current_timepoint(1), current_timepoint(end), vol_thresh, k_beg, k_end);
        temp_filename = sprintf('%sclustering_ROI_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.tmp', testing_result_subdir, current_timepoint(1), current_timepoint(end), vol_thresh, k_beg, k_end);
        % the definition here is different from other scripts, these are
        % for per protein errors rather than all proteins. 
%         W_err_mat = zeros(k_end - k_beg + 1, N);
%         W_n_err_mat = zeros(k_end - k_beg + 1, N);
%         W_pct_err_mat = zeros(k_end - k_beg + 1, N);
%         P_err_mat = zeros(k_end - k_beg + 1, N);
%         P_n_err_mat = zeros(k_end - k_beg + 1, N);
        P_pct_err_mat = zeros(k_end - k_beg + 1, N);
        R_t_3d_mat = zeros(size(X, 1), size(X, 1), M, k_end - k_beg + 1);
        A_mat = zeros(size(X, 1), size(X, 1), k_end - k_beg + 1);
        if exist(results_filename, 'file')
            fprintf('Partition %d has been computed.\n', k0);
            continue;
        end
        if exist(temp_filename, 'file')
           fprintf('Partition %d is being computed.\n', k0);
           continue;
        else
            fclose(fopen(temp_filename, 'w'));
        end
        fprintf('Compute partition: %d\n', k0);
        for k = k_beg : k_end
            
            % conduct permutation.          
            X_shuffle = zeros(size(X));
            rand_order = randperm(numel(X));
            X_shuffle(1 : numel(X)) = X(rand_order);

            tic
            % [A] = master_script_spatiotemporal_group_lasso_admm(X, M, options);
            [R_t_3d, A, c] = master_script_causal_graph_process(X_shuffle, M, options);           
            toc
            
            method = 2;
            [~, W] = CGP_residual_computing(R_t_3d, A, c, M, X_shuffle, method);
            W_err = norm(W, 'fro') / sqrt(numel(W));
            W_pct_error = norm(W ./ X_shuffle(:, M + 1 : end), 'fro') / sqrt(numel(W));
            W_n_err = W_err ./ mean(mean(abs(X(:, M + 1 : end))));
            [~, P_W] = CGP_residual_computing(R_t_3d, A, c, M, tt_X, method);
            P_err = norm(P_W, 'fro') / sqrt(numel(P_W));
            P_pct_error = norm(P_W ./ tt_X(:, M + 1 : end), 'fro') / sqrt(numel(P_W));                
            P_n_err = P_err ./ mean(mean(abs(tt_X(:, M + 1 : end))));
            fprintf(' W_n: %4f, W_pct: %4f\n P_n: %4f, P_pct: %4f\n\n', W_n_err, W_pct_error, P_n_err, P_pct_error);
            
            R_t_3d_mat(:, :, :, k - k_beg + 1) = R_t_3d;
            A_mat(:, :, k - k_beg + 1) = A;
%             W_err_mat(k - k_beg + 1, :) = W_err;
%             W_n_err_mat(k - k_beg + 1, :) = W_n_err;
%             W_pct_err_mat(k - k_beg + 1, :) = W_pct_err;
%             P_err_mat(k - k_beg + 1, :) = P_err;
%             P_n_err_mat(k - k_beg + 1, :) = P_n_err;
             P_pct_err_mat(k - k_beg + 1, :) = P_pct_error;
        end
        save(results_filename, 'R_t_3d_mat', 'A_mat', 'P_pct_err_mat', 'k_beg', 'k_end', 'param');
        if exist(results_filename, 'file') && exist(temp_filename, 'file')
            delete(temp_filename);
        end
    end
end

testing_info_filename = sprintf('%sclustering_ROI_B7_testing_info.mat', testing_result_subdir);

if ~exist(testing_info_filename, 'file')
    save(testing_info_filename, 'param');
end


options = struct();            
if isfield(param, 'computing_options')
    options = param.computing_options;
end
options.M = M;

options.lambda_1 = optimize_param.lambda_1;
options.lambda_2 = optimize_param.lambda_2;
options.lambda_3 = optimize_param.lambda_3;
options.self_prediction = optimize_param.self_prediction;

% finish calculation of p-values
tic
% [A] = master_script_spatiotemporal_group_lasso_admm(X, M, options);
[R_t_3d, A, c] = master_script_causal_graph_process(X, M, options);
toc

method = 2;
% just directly compute Err 
[~, W] = CGP_residual_computing(R_t_3d, A, c, M, X, method);            
W_err = sqrt(sum(W .^ 2, 2)) / sqrt(size(W, 2));
W_n_err = W_err ./ (mean(abs(X(:, M + 1 : end)), 2));
[~, P_W] = CGP_residual_computing(R_t_3d, A, c, M, tt_X, method);
P_err = sqrt(sum(P_W .^ 2, 2)) / sqrt(size(P_W, 2));
P_n_err = P_err ./ (mean(abs(tt_X(:, M + 1 : end)), 2));
Err_predict = P_err;

compute_p_value = param.compute_p_value;
N_test_count = 0;
p_value_upper = zeros(size(Err_predict));
p_value_lower = zeros(size(Err_predict));
p_value_tail = zeros(size(Err_predict));
% A_mat = zeros(X_size, X_size, N_test);

% now only implement empirical distribution based p-values. 
testing_result_file = sprintf('%sclustering_ROI_reltime_%d-%d_vol_thrd_%d_testing_summary.mat', testing_subdir, current_timepoint(1), current_timepoint(end), vol_thresh);
if compute_p_value && ~exist(testing_result_file, 'file')
    % count if skip any file 
    N_skip = 0;
    switch p_value_method            
        case 'empirical'
            for i = 1 : N_partition
                i_range = (i - 1) * N_perfile + 1 : min(N_test, i * N_perfile);
                result_file = sprintf('%sclustering_ROI_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.mat', testing_result_subdir, current_timepoint(1), current_timepoint(end), vol_thresh, i_range(1), i_range(end));
                try 
                    load(result_file);
                    p_up_part = A < A_mat;
                    p_value_upper = p_value_upper + sum(p_up_part, 3);
                    
                    p_low_part = A > A_mat;
                    p_value_lower = p_value_lower + sum(p_low_part, 3);  
                    
                    p_tail_part = abs(A) < abs(A_mat);
                    p_value_tail = p_value_tail + sum(p_tail_part, 3);  
                    
                    N_test_count = N_test_count + numel(i_range);
                    
                catch ME
                    delete(result_file);
                    fprintf('The testing %d - %d do not exist!\n', i_range(1), i_range(end));
                    N_skip = N_skip + 1;
                    continue;
                end               
            end
            if N_skip == 0
                p_value_upper = (p_value_upper + 1) / (N_test_count + 1);
                p_value_lower = (p_value_lower + 1) / (N_test_count + 1);
                p_value_tail = (p_value_tail + 1) / (N_test_count + 1);
                save(testing_result_file, 'param', 'p_value_upper', 'p_value_lower', 'p_value_tail');
            end
        otherwise
            error('Unrecognized testing method!');
    end
else
    load(testing_result_file);
end

testing_info = struct();
testing_info.test_name = 'B7_testing';
testing_info.p_value_upper = p_value_upper;
testing_info.p_value_lower = p_value_lower;
testing_info.p_value_tail = p_value_tail;
t_cell_info.testing_info = testing_info;


end
