function [t_cell_info] = master_script_CGP_cross_validation(t_cell_info, param)
% This is for cross validation of CGP model 
% Copied from master_script_CGP_temporal_clustering_cross_validation.m
% 
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% March 15, 2017 Refactor
% clear all;

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



s = RandStream('mt19937ar', 'Seed', 892734);
RandStream.setGlobalStream(s);

current_timepoint = param.current_timepoint;
lag_time = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
causal_method = param.causal_method;
% computing_sufix = param.computing_sufix;

cv_param = param.cv_param;
cv_fold = cv_param.cv_fold;

% total data point for training (building the model)
m = numel(current_timepoint);

[x1, x2, x3, x4] = ndgrid(cv_param.M_group, cv_param.lambda_1_group, cv_param.lambda_2_group, cv_param.lambda_3_group);
parameter_combinations = [x1(:), x2(:), x3(:), x4(:)];
cv_param.parameter_combinations = parameter_combinations;
is_computing = cv_param.is_computing;

process_cross_validation_data_splitting(t_cell_info);

cv_data_location = t_cell_info.path_info.cv_data_location;
cv_computing_location = t_cell_info.path_info.cv_computing_location;
cv_sub_location = t_cell_info.path_info.cv_sub_location;

Max_loops = 5;
loop_count = 0;
N_per_partition = 100;
N_total = size(parameter_combinations, 1);

N_partition = ceil(N_total / N_per_partition);

while is_computing && loop_count < Max_loops
    computed_mat = zeros(cv_fold, N_partition);
    for i = 1 : cv_fold

        cv_index = i;

        [X, tt_X] = cross_validation_graph_data_generation(t_cell_info, cv_index);

        % use block partitioning for the computing. 
        chunk_lock_clean(cv_computing_location, N_per_partition * 0.3);
        for k0 = 1 : N_partition
            k_beg = (k0 - 1) * N_per_partition + 1;
            k_end = min(N_total, k0 * N_per_partition);

            results_filename = sprintf('%sclustering_ROI_cv_%d_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.mat', cv_computing_location, i, current_timepoint(1), current_timepoint(end), vol_thresh, k_beg, k_end);
            temp_filename = sprintf('%sclustering_ROI_cv_%d_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.tmp', cv_computing_location, i, current_timepoint(1), current_timepoint(end), vol_thresh, k_beg, k_end);
            W_err_mat = zeros(k_end - k_beg + 1, 1);
            W_n_err_mat = zeros(k_end - k_beg + 1, 1);
            W_pct_err_mat = zeros(k_end - k_beg + 1, 1);
            P_err_mat = zeros(k_end - k_beg + 1, 1);
            P_n_err_mat = zeros(k_end - k_beg + 1, 1);
            P_pct_err_mat = zeros(k_end - k_beg + 1, 1);

            if exist(results_filename, 'file')
                fprintf('Partition %d has been computed.\n', k0);
                computed_mat(cv_index, k0) = 1;
                continue;
            end
            if exist(temp_filename, 'file')
               fprintf('Partition %d is being computed.\n', k0);
               continue;
            else
                fclose(fopen(temp_filename, 'w'));
            end

            for k = [k_beg : k_end]
                cur_comb = parameter_combinations(k, :);
                M = cur_comb(1);
                lambda_1 = cur_comb(2);
                lambda_2 = cur_comb(3);
                lambda_3 = cur_comb(4);

                options = struct();
                options.M = M;
                options.lambda_1 = lambda_1;
                options.lambda_2 = lambda_2;
                options.lambda_3 = lambda_3;

                [R_t_3d, A, c] = master_script_causal_graph_process(X, M, options);
                method = param.residual_compute_method;
                [W_err, W, W_pct_error, W_n_err] = CGP_residual_computing(R_t_3d, A, c, M, X, method);
                [P_err, P_W, P_pct_error, P_n_err] = CGP_residual_computing(R_t_3d, A, c, M, tt_X, method);

                fprintf(' W_n: %4f, W_pct: %4f\n P_n: %4f, P_pct: %4f\n\n', W_n_err, W_pct_error, P_n_err, P_pct_error);
                W_err_mat(k - k_beg + 1) = W_err;
                W_n_err_mat(k - k_beg + 1) = W_n_err;
                W_pct_err_mat(k - k_beg + 1) = W_pct_error;                
                P_err_mat(k - k_beg + 1) = P_err;
                P_n_err_mat(k - k_beg + 1) = P_n_err;
                P_pct_err_mat(k - k_beg + 1) = P_pct_error;                
            end
            save(results_filename, 'W_err_mat', 'W_n_err_mat', 'W_pct_err_mat', 'P_err_mat', 'P_n_err_mat', 'P_pct_err_mat', 'k_beg', 'k_end', 'param');
            if exist(results_filename, 'file') && exist(temp_filename, 'file')
                delete(temp_filename);
            end
            computed_mat(cv_index, k0) = 1;
        end
    end
    if all(computed_mat(:) == 1) 
        is_computing = false;
        break;
    end    
    pause(5 * 60);
    loop_count = loop_count + 1;    
end

collect_results = true;
% collect results. 
if collect_results
    W_mat = zeros(size(parameter_combinations, 1), 1);
    W_N_mat = zeros(size(parameter_combinations, 1), 1);
    W_pct_mat = zeros(size(parameter_combinations, 1), 1);
    P_mat = zeros(size(parameter_combinations, 1), 1);
    P_n_mat = zeros(size(parameter_combinations, 1), 1);
    P_pct_mat = zeros(size(parameter_combinations, 1), 1);
    if false
        dir_info = dir(cv_computing_location);
        filenames = {dir_info.name};
        pattern = '.*mat';
        filenames = filenames(~cellfun(@isempty, regexp(filenames, pattern)));
        for i = 1 : numel(filenames)
            try 
                a = load([cv_computing_location, filenames{i}]);
            catch
                delete([cv_computing_location, filenames{i}]);
            end
        end
    end
    final_result = sprintf('%sclustering_num_%d_ROI_cv_final_results.mat', cv_sub_location, cluster_num);
    if(~exist(final_result, 'file'))
        for i = 1 : cv_fold         

            for k0 = 1 : N_partition
                k_beg = (k0 - 1) * N_per_partition + 1;
                k_end = min(N_total, k0 * N_per_partition);

                results_filename = sprintf('%sclustering_ROI_cv_%d_reltime_%d-%d_vol_thrd_%d_comb_par_%d_%d.mat', cv_computing_location, i, current_timepoint(1), current_timepoint(end), vol_thresh, k_beg, k_end);
                W_err_mat = zeros(k_end - k_beg + 1, 1);
                W_n_err_mat = zeros(k_end - k_beg + 1, 1);
                P_err_mat = zeros(k_end - k_beg + 1, 1);
                P_n_err_mat = zeros(k_end - k_beg + 1, 1);               

                if exist(results_filename, 'file')  
                    a = load(results_filename);
                    W_mat([k_beg : k_end], i) = a.W_err_mat;
                    W_N_mat([k_beg : k_end], i) = a.W_n_err_mat(:);
                    W_pct_mat([k_beg : k_end], i) = a.W_pct_err_mat(:);

                    P_mat([k_beg : k_end], i) = a.P_err_mat;
                    P_n_mat([k_beg : k_end], i) = a.P_n_err_mat(:);                
                    P_pct_mat([k_beg : k_end], i) = a.P_pct_err_mat(:);                

                    % A_T_mat(:, :, [k_beg : k_end]) = A_mat;                   
                    % c_T_mat(1, 1 : 2, [k_beg : k_end]) = c_mat(1, 1 : 2, :);
                else
                    error('The parition %d - %d does not exist.', k_beg, k_end);
                end            
            end   
        end
        % save(final_result, 'W_mat', 'W_N_mat', 'W_pct_mat', 'P_mat', 'P_n_mat', 'P_pct_mat', 'A_T_mat', 'c_T_mat', 'parameter_combinations', 'M_group', 'lambda_1_group', 'lambda_2_group', 'lambda_3_group');
        save(final_result, 'W_mat', 'W_N_mat', 'W_pct_mat', 'P_mat', 'P_n_mat', 'P_pct_mat', 'parameter_combinations', 'param');
     else
        a = load(final_result);
        P_pct_mat = a.P_pct_mat;
        parameter_combinations = a.parameter_combinations;
    end  
end

% choose optimal parameters
mean_P_pct_mat = mean(P_pct_mat, 2);
[min_P_pct_error, min_ind] = min(mean_P_pct_mat);
if numel(min_ind) > 1
    min_ind = min_ind(1);
end
optimal_param_comb = parameter_combinations(min_ind, :);
optimal_param.M = optimal_param_comb(1);
optimal_param.lambda_1 = optimal_param_comb(2);
optimal_param.lambda_2 = optimal_param_comb(3);
optimal_param.lambda_3 = optimal_param_comb(4);
optimal_param.self_prediction = cv_param.self_prediction;
optimal_param.min_P_pct_error = min_P_pct_error;
fprintf('minimum P_pct error: %4f, M: %d lambda_1: %d lambda_2: %d  lambda_3: %d\n', min_P_pct_error, optimal_param_comb(1), optimal_param_comb(2), optimal_param_comb(3), optimal_param_comb(4))

t_cell_info.cv_info = struct();
t_cell_info.cv_info.cv_param = param.cv_param;
t_cell_info.cv_info.optimal_param = optimal_param;


end