function [t_cell_info] = master_script_clustering_Full_B7_data_extraction(t_cell_info, param)
% clustering on the raw model data and generate data input for full
% stimulus and B7 conditions. 
%
%copied from B7_testing_error_computing.,
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Mar. 15, 2017 Refactor

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu





param = t_cell_info.param;
tcell_models_location = t_cell_info.path_info.tcell_models_location;
results_location = t_cell_info.path_info.results_location;
data_location = t_cell_info.path_info.data_location;
clustering_result_location = t_cell_info.path_info.clustering_result_location;
cross_validation_location = t_cell_info.path_info.cross_validation_location;
model_location = t_cell_info.path_info.model_location;
permutation_test_location = t_cell_info.path_info.permutation_test_location;
figure_location = t_cell_info.path_info.figure_location;


current_timepoint = param.current_timepoint;
lag_time = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;

illustration_colormap_to_use = t_cell_info.param.illustration_colormap_to_use; 


% total data point for training (building the model)
m = numel(current_timepoint);

train_condition = 'Full Stimulus';
test_condition = 'B7 Blockade';

model_data = t_cell_info.model_data.model_data;
model_means =  t_cell_info.model_data.model_means;
unique_sensor_names = t_cell_info.model_data.unique_sensor_names;
used_model_names = t_cell_info.model_data.used_model_names;
used_model_reltime = t_cell_info.model_data.used_model_reltime;
used_model_conditions = t_cell_info.model_data.used_model_conditions;
num_b7_sensor = numel(unique_sensor_names);
mean_t = zeros(numel(unique_sensor_names), numel(current_timepoint),  size(model_means, 3));
tt_mean_t = zeros(numel(unique_sensor_names), numel(current_timepoint),  size(model_means, 3));

model_means_vec = permute(mean_t, [3, 2, 1]);

% get the relative positions of the voxels in the model: subscripts
load('subscripts_standard_voxels.mat');

template_image = model_reconstruction(ones(size(subscripts, 1), 1), subscripts);

template_crop_x = sum(sum(template_image, 1), 3) > 0;
template_crop_y = sum(sum(template_image, 2), 3) > 0;
template_crop_z = sum(sum(template_image, 1), 2) > 0;
template_crop_function = @(given_image)given_image(template_crop_y, template_crop_x, template_crop_z);
template_crop_image = template_crop_function(template_image);

model_data_used = {};

for i = 1 : numel(unique_sensor_names)
    current_sensor = unique_sensor_names{i};   
    for j = 1 : numel(current_timepoint)          
        ind_y = strcmp(used_model_conditions, train_condition) ...
            .* (~cellfun(@isempty, regexp(used_model_names, current_sensor)));
        ind_x =  (used_model_reltime == current_timepoint(j));
        mean_t(i, j, :) = model_means(ind_y > 0, ind_x, :);
        ind_y_t = strcmp(used_model_conditions, test_condition) ...
            .* (~cellfun(@isempty, regexp(used_model_names, current_sensor)));
        ind_x_t =  (used_model_reltime == current_timepoint(j));
        if any(ind_y_t > 0)
            tt_mean_t(i, j, :) = model_means(ind_y_t > 0, ind_x_t, :);
        end
    end
end

tn_mean_t = mean_t;

if strcmp(cluster_sensor, 'all')
    cluster_filename = sprintf('%smethod_%s_cluster_num_%d.mat', clustering_result_location, cluster_type, cluster_num);
else
    cluster_filename = sprintf('%smethod_%s_sensor_%s_cluster_num_%d.mat', cluster_dir, cluster_type, cluster_sensor, cluster_num);
end

% Apr. 7, 2017 add computing for clustering criterion
if ~false && strcmp(param.opt_std, 'CV_M2')

    if strcmp(cluster_sensor, 'all')
        X = reshape(permute(mean_t, [3, 2, 1]), size(mean_t, 3), []);
    elseif ~strcmp(data_type, '2pt') && strcmp(cluster_sensor, 'Actin')
        sensor_ind = find(~cellfun(@isempty, regexpi(unique_sensor_names, cluster_sensor)) == 1);
        X = squeeze(mean_t(sensor_ind, :, :))';
    elseif strcmp(data_type, '2pt') && strcmp(cluster_sensor, 'Actin')
        X = read_sensors_model_mean_data({cluster_sensor}, {current_condition}, current_timepoint);
        X = X{1}';
    end
    cluster_nums = [2 : 9, 10 : 2 : 20, 25 : 5 : 90];
    eval_1 = evalclusters(X,'kmeans','CalinskiHarabasz','KList',cluster_nums);
    figure, plot(eval_1.InspectedK, eval_1.CriterionValues);
    fprintf('optimal cluster number: %d\n', eval_1.OptimalK);
end

if ~exist(cluster_filename, 'file')
    % Jan. 27, 2016 set streams to allow reproducible results
    if strcmp(cluster_sensor, 'all')
        X = reshape(permute(mean_t, [3, 2, 1]), size(mean_t, 3), []);
    elseif ~strcmp(data_type, '2pt') && strcmp(cluster_sensor, 'Actin')
        sensor_ind = find(~cellfun(@isempty, regexpi(unique_sensor_names, cluster_sensor)) == 1);
        X = squeeze(mean_t(sensor_ind, :, :))';
    elseif strcmp(data_type, '2pt') && strcmp(cluster_sensor, 'Actin')
        X = read_sensors_model_mean_data({cluster_sensor}, {current_condition}, current_timepoint);
        X = X{1}';
    end
    if strcmp(cluster_type, 'kmeans')
        s = RandStream('mt19937ar', 'seed', 1);
        km_options.streams = s;
        labels = kmeans(X, cluster_num, 'options', km_options);
    elseif strcmp(cluster_type, 'gmmem')
        [labels] = emgm(X, cluster_num);
    end    

    save(cluster_filename, 'labels', 'X', 'cluster_num');        
else    
    cluster_data = load(cluster_filename);
    labels = cluster_data.labels;
end

% check cluster numbers

label_reconstruct = model_reconstruction(labels, subscripts);
label_reconstruct_cropped = template_crop_function(label_reconstruct);

region_count = 0;
region_label_group = zeros(m, cluster_num);
labeled_maps = {};
region_mean = {};
    
map_label = zeros(size(label_reconstruct_cropped));

for i = 1 : cluster_num
    map_i = (label_reconstruct_cropped == i);
    % xruan 01/13/2016 change the connect component to 26-connection
    % cc = bwconncomp(map_ij, 6);   
    cc = bwconncomp(map_i, 26);
    inds = find(cellfun(@numel, cc.PixelIdxList) >= vol_thresh);
    for k = 1 : numel(inds)
        map_label(cc.PixelIdxList{inds(k)}) = k + region_count;
    end
    if isempty(inds)
        k = 0;
    end
    region_count = region_count + k;   
    region_label_group(i) = numel(inds);
end   
map_label;

% Feb. 4, 2016
% sort the regions by their centroid with the order of y, x, z
[region_centroids, region_labels, label_order, map_label_sorted] = region_centroid_compute(map_label);
map_label = map_label_sorted;

% visualize label map
if ~false  
    figure, 

    set(gcf, 'Visible', 'off')
    set(gcf, 'color', 'w')
    set(gcf, 'Visible', 'on')
    set(gcf, 'Renderer', 'opengl');
    dpi = 300;
    figure_height = 50;
    figure_width = 1500;
    set(gcf, 'Position', [1920, 1, figure_width, figure_height]);

    image_to_show = [];
    image_to_show = reshape_2d(map_label);
    max_intensity = max(image_to_show(:));
    
    imagesc(image_to_show);
    axis off;
    colormap([illustration_colormap_to_use]), caxis([0, max_intensity]);
    % ylabel_group = arrayfun(@(x) num2str(x), current_timepoint, 'UniformOutput', false);
    % set(gca, 'YTick',  -12 + 24 * [1 : numel(current_timepoint)], 'YTickLabel', ylabel_group)   
    set(gca, 'YTick', []);
    set(gca, 'XTick', []);
    % ylabel('Relative time');
    % title(sprintf('Regions of clusters - Method: %s - Cluster number: %d', cluster_type, cluster_num));
    % title('Region labels', 'fontsize', 21, 'fontweight', 'bold');    
    h = colorbar;
    xtick_group = arrayfun(@(x) num2str(x), round(linspace(1, cluster_num, 3)), 'UniformOutput', false);
    set(h, 'XTick',  round(linspace(1, cluster_num, 3)), 'XTickLabel', xtick_group, 'fontsize', 15, 'fontweight', 'bold');
    % xlabel(h, 'Cluster number');
    figure_filename = sprintf('%scluster_map_method-%s_cluster_sensor_%s_cluster_number_%d_reltime_%d-%d', clustering_result_location, cluster_type, cluster_sensor, cluster_num, current_timepoint(1), current_timepoint(end));
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);    
    
end

% save the map labels so that other postprocessing scripts can directly use
% the map label for consistency. 
map_label_filename = sprintf('%sclustering_ROI_region_label_map_cluster_num-%d_vol_thrsd-%d.mat', clustering_result_location, cluster_num, vol_thresh);
if ~exist(map_label_filename, 'file')
    save(map_label_filename, 'map_label', 'region_count', 'region_labels');
end

map_label_vec = map_label(template_crop_image > 0);
region_label_vals = unique(map_label_vec);
region_label_vals(region_label_vals == 0) = [];

X_mat = zeros(size(tn_mean_t, 1), size(tn_mean_t, 2), numel(region_label_vals));
tt_X_mat = zeros(size(tt_mean_t, 1), size(tt_mean_t, 2), numel(region_label_vals));

for i1 = 1 : numel(region_label_vals)
    label_i = region_label_vals(i1);
    inds_i = (map_label_vec == label_i);
    X_mat(:, :, i1) = sum(tn_mean_t(:, :, inds_i), 3) / sum(inds_i) * numel(inds_i);    
    tt_X_mat(:, :, i1) = sum(tt_mean_t(:, :, inds_i), 3) / sum(inds_i) * numel(inds_i);    
end

X = reshape(permute(X_mat, [3, 1, 2]), [], size(X_mat, 2)); 
tt_X = reshape(permute(tt_X_mat, [3, 1, 2]), [], size(tt_X_mat, 2)); 
region_num = numel(region_label_vals);

kept_nodes = 1 : numel(unique_sensor_names) * region_num;
orig_X = X;
orig_tt_X = tt_X;

if param.merge_proteins
    % by default, merge (cpalpha1, wasp, coronin1A,
    % wave2), actin, (arp3, hs1), cofilin, ezrin, pip2,
    % lat, mrlc 
    % we use th mean of the training and validation data.
    group_inds = param.group_inds;
    X_new = zeros(region_num, numel(group_inds), size(X, 2));
    tt_X_new = zeros(region_num, numel(group_inds), size(tt_X, 2));
    X_reshape = reshape(X, region_num, [], size(X, 2));
    tt_X_reshape = reshape(tt_X, region_num, [], size(tt_X, 2));
    kept_nodes = zeros(region_num, numel(group_inds));
    for i1 = 1 : numel(group_inds)
        X_new(:, i1, :) = mean(X_reshape(:, group_inds{i1}, :), 2);
        tt_X_new(:, i1, :) = mean(tt_X_reshape(:, group_inds{i1}, :), 2);  
        kept_nodes(:, i1) = (1 : region_num) + min(group_inds{i1} - 1) * region_num;
    end
    % just keep them for tracking or debuging
    X = reshape(X_new, [], size(X_new, 3));
    tt_X = reshape(tt_X_new, [], size(tt_X_new, 3));
    kept_nodes = reshape(kept_nodes, [], 1);
end
all_sensor_region_combine = cellfun(@(x) arrayfun(@(y) sprintf('%s - %d', x, y),  1 : region_num, 'uniformoutput', false), unique_sensor_names, 'uniformoutput', false);
all_sensor_region_combine = reshape(cat(1, all_sensor_region_combine{:})', 1, []);
kept_node_names = all_sensor_region_combine(kept_nodes);
kept_sensor_names =unique(cellfun(@(x) x{1}{1}, regexp(kept_node_names, '(^[^\s^-]+)', 'tokens'), 'Uniformoutput', false));

graph_data = struct();
graph_data.X = X;
graph_data.tt_X = tt_X;
graph_data.region_num = region_num;
graph_data.kept_nodes = kept_nodes;
graph_data.kept_node_names = kept_node_names;
graph_data.kept_sensor_names = kept_sensor_names;

graph_data.orig_X = orig_X;
graph_data.orig_tt_X = orig_tt_X;
graph_data.all_sensor_region_combine = all_sensor_region_combine;


t_cell_info.graph_data = graph_data;

end