function [t_cell_info] = master_script_data_hierachical_clustering(t_cell_info)
% clustering the data to see the patterns 
% copied from master_script_visualize_model.m
% 
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Date: Mar. 17, 2017

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu





close all;

param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;

unique_sensor_names = t_cell_info.model_data.unique_sensor_names;

graph_data = t_cell_info.graph_data;
kept_nodes = t_cell_info.graph_data.kept_nodes;
all_sensor_region_combine = graph_data.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);

X = graph_data.X;

illustration_colormap_to_use = t_cell_info.param.illustration_colormap_to_use;

figure_location = t_cell_info.path_info.figure_location;


X_p = reshape(permute(reshape(X, 3, [], size(X, 2)), [2, 3, 1]), [], 3 * size(X, 2));
dpi = 300;

% use whole protein for clustering 
% L1 clustering
metric = 'cityblock';

A = pdist(X_p, metric);
Y = squareform(A);

[h, c] = plot_hierachical_cluster_figure(A, Y, unique_sensor_names, metric);

export_fig(sprintf('%sX_h_clustering_%s_cophenetic_%d.png', figure_location, metric, c), '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);

% use whole protein for clustering 
% correlation
metric = 'correlation';

A = pdist(X_p, metric);
Y = squareform(A);

[h, c] = plot_hierachical_cluster_figure(A, Y, unique_sensor_names, metric);

export_fig(sprintf('%sX_h_clustering_%s_cophenetic_%d.png', figure_location, metric, c), '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);

% use whole protein for clustering 
% correlation
metric = 'euclidean';

A = pdist(X_p, metric);
Y = squareform(A);

[h, c] = plot_hierachical_cluster_figure(A, Y, unique_sensor_names, metric);

export_fig(sprintf('%sX_h_clustering_%s_cophenetic_%d.png', figure_location, metric, c), '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);

end
