function [t_cell_info] = master_script_generate_error_comparison_table(t_cell_info)
% generate error comparison table for Table 1 in the paper
%
% Author: Xiongtao Ruan
% March 20, 2017

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu






close all;

param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;
suffix = param.suffix;

graph_data = t_cell_info.graph_data;
kept_nodes = t_cell_info.graph_data.kept_nodes;
all_sensor_region_combine = graph_data.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);

illustration_colormap_to_use = t_cell_info.param.illustration_colormap_to_use;

model_location = t_cell_info.path_info.model_location;
model_sub_location = t_cell_info.path_info.model_sub_location;
figure_location = t_cell_info.path_info.figure_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;

% [R_t_3d, A, c] = master_script_causal_graph_process(X, M) 
results_filename = sprintf('%sclustering_ROI_results_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d%s', model_sub_location, causal_method, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh, suffix);

cv_info = t_cell_info.cv_info;
% show current condition model info

results = load(results_filename);

% collect models for different conditions
min_CV_error = results.min_CV_P_pct_error;
min_B7_error = results.P_pct_error;

AUC_param = struct();
AUC_param.unique_sensor_names = t_cell_info.model_data.unique_sensor_names;
AUC_param.causal_method = causal_method;
AUC_param.merge_proteins = param.merge_proteins;
if AUC_param.merge_proteins
    AUC_param.group_inds = param.group_inds;
end

cur_AUC = model_AUC_computing(results.A, results.R_t_3d, AUC_param);

fprintf('causal method: %s opt_std: %s \n CV error: %d  B7 error: %d  AUC: %d\n', causal_method, opt_std, min_CV_error, min_B7_error, cur_AUC);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% collect results for all eight cases


cur_param = struct();
cur_param.unique_sensor_names = t_cell_info.model_data.unique_sensor_names;;

causal_methods_group = {'CGP', 'CENR'};
opt_std_group = {'CV_M2', 'CV_M2_merge_214', 'CV_M2_merge_7', 'CV_M2_merge_25'};

csv_mat = zeros(3, 2, 4);

for i = 1 : numel(opt_std_group)
    cur_opt_std = opt_std_group{i};
    switch cur_opt_std
        case 'CV_M2'     
            cur_param.merge_proteins = false;
            cur_param.group_inds = {1, 3, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};  
        case 'CV_M2_merge_214'
            cur_param.merge_proteins = true;
            cur_param.group_inds = {[1, 7], 2, [3, 5, 11, 12], 4, 6, 8, 9, 10};    
        case 'CV_M2_merge_7'
            cur_param.merge_proteins = true;
            cur_param.group_inds = {[1, 2, 3, 5, 7, 11, 12], 4, 6, 8, 9, 10};   
        case 'CV_M2_merge_25'
            cur_param.merge_proteins = true;
            cur_param.group_inds = {[1, 7], [2, 3, 5, 11, 12], 4, 6, 8, 9, 10};  
    end
    
    for j = 1 : numel(causal_methods_group)
        cur_causal_method = causal_methods_group{j};
        cur_param.causal_method = cur_causal_method;
        
        cur_results_filename = sprintf('%s%s_%s/clustering_ROI_results_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d%s.mat', ...
            model_location, cur_causal_method, cur_opt_std, cur_causal_method, cur_opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh, suffix);
        if ~exist(cur_results_filename, 'file')
            error('%s does not exist!', cur_results_filename);
        end
        
        a = load(cur_results_filename);

        % collect models for different conditions
        cur_min_CV_error = a.min_CV_P_pct_error;
        cur_min_B7_error = a.P_pct_error;
        cur_AUC = model_AUC_computing(a.A, a.R_t_3d, cur_param);
        
        csv_mat(:, j, i) = [cur_min_CV_error, cur_min_B7_error, cur_AUC]';
    end
end

csv_mat_reshape = reshape(csv_mat, size(csv_mat, 1), []);

[m, n] = size(csv_mat_reshape);

% write errors csv files. 
csv_field_matrix = {};
Err_cell = num2cell(reshape(csv_mat_reshape, m, []));
csv_field_matrix{1, 1} = ' ';
csv_field_matrix(1, 2 : 3) = {'Ungrouped', 'Ungrouped'};
csv_field_matrix(1, 4 : 5) = {'Case 1', 'Case 1'};
csv_field_matrix(1, 6 : 7) = {'Case 2(7)', 'Case 2(7)'};
csv_field_matrix(1, 8 : 9) = {'Case 3(25)', 'Case 3(25)'};
csv_field_matrix(2, 2 : 3) = causal_methods_group;
csv_field_matrix(2, 4 : 5) = causal_methods_group;
csv_field_matrix(2, 6 : 7) = causal_methods_group;
csv_field_matrix(2, 8 : 9) = causal_methods_group;
csv_field_matrix(3, 1) = {'CV error'};
csv_field_matrix(4, 1) = {'B7 error'};
csv_field_matrix(5, 1) = {'AUC'};

csv_field_matrix(3 : 5, 2 : 9) = Err_cell;

table_filename = sprintf('%sMethods_comparison_table.csv', figure_location);

fid = fopen(table_filename, 'wt');
for i = 1: size(csv_field_matrix, 1)
    for j = 1 : size(csv_field_matrix, 2)
        if j > 1
            fprintf(fid, ',');
        end
        if ischar(csv_field_matrix{i, j})
          fwrite(fid, csv_field_matrix{i, j});
        else
          fprintf(fid, '%e', csv_field_matrix{i, j});
        end        
    end
    fprintf(fid, '\n');
end
fclose(fid);



end