function [t_cell_info] = master_script_get_model_data(t_cell_info, param)
% this function is for computation of the correlations between any two
% time point and two proteins for all conditions of t cell models. 
% 
% Copy from master_script_tcell_correlation_computation.m
%
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Oct. 9, 2015
% clear all;
% Mar. 15, 2017 Refactor

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




close all;
debug = 0;

model_dirset = {'/projects/cellorganizer/xruan/tcell_project/tcell_excel_results_2pt_simulation_85_sn_st/two-point_landmark-smoothing-method_none_intensity-morphing-method_none_models/', ...
    '/projects/cellorganizer/xruan/tcell_project/intermediate_results_final/tcell_excel_results_1pt_repro_85_sn_st/landmark-smoothing-method_none_intensity-morphing-method_none_models/'};

for i = 1 : length(model_dirset)
    if exist(model_dirset{i}, 'dir')
        default_model_dir = model_dirset{i};
        break;
    end
end
param.dir_info = process_options_structure(struct('model_dir', default_model_dir), param.dir_info);
                             
model_dir = param.dir_info.model_dir;

path(pathdef);
% addpath(genpath('~/cellorganizer'));
% addpath(genpath(home_dir));
% addpath(genpath([home_dir, filesep, 'code']));
% addpath(genpath(['/projects/cellorganizer/xruan/t_cell_project/tcell_clean/']));
% addpath(genpath(['~/cellorganizer_bu']));
% illustration_colormap_to_use = pmkmp(1024, 'CubicL'); 

segmentation_dir = [home_dir, 'segmentation/'];
if ~exist(segmentation_dir, 'dir')
    mkdir(segmentation_dir);
end

model_dir_info = dir(model_dir);
model_dir_info(1 : 2) = [];

used_model_type = 'standardized_voxels';

used_model_pattern = ['[FB]\w+\s[SB]\w+_\w+_reltime..*' used_model_type, '.mat'];

model_names = {model_dir_info.name};

used_model_names = model_names(~cellfun(@isempty, (regexp(model_names, used_model_pattern))));

used_model_info = regexp(used_model_names, '_', 'split');

conditions = {'Full Stimulus', 'B7 Blockade'};

used_model_conditions = cellfun(@(x) x{1}, used_model_info, 'UniformOutput', false);

used_model_sensor_names = cellfun(@(x) x{2}, used_model_info, 'UniformOutput', false);

used_model_reltime = cell2mat(cellfun(@(x) str2num(x{3}(end - 1 :end)), used_model_info, 'UniformOutput', false));

unique_sensor_names = unique(used_model_sensor_names);

model_num = length(used_model_names);

model_means = zeros(model_num, 6628);

model_std = zeros(model_num, 6628);

model_data = {};

for i = 1 : model_num
    current_model_name = used_model_names{i};
    
    load([model_dir, current_model_name]);
    model_means(i, :) = current_mean;
    model_data{i} = current_data;
end

t_cell_info = struct;

t_cell_info.model_data = struct;

t_cell_info.model_data.model_data = model_data;

t_cell_info.model_data.model_means = model_means;

t_cell_info.model_data.model_std = model_std;

t_cell_info.model_data.model_num = model_num;

t_cell_info.model_data.model_num = model_num;

t_cell_info.model_data.unique_sensor_names = unique_sensor_names;

t_cell_info.model_data.used_model_reltime = used_model_reltime;

t_cell_info.model_data.used_model_conditions = used_model_conditions;

t_cell_info.model_data.used_model_names = used_model_names;

end






