function [t_cell_info] = master_script_get_model_time_series_data(t_cell_info, param)

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



% this function is for computation of the correlations between any two
% time point and two proteins for all conditions of t cell models. 
% 
% Copy from master_script_tcell_correlation_computation.m
%
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Oct. 12, 2016
% clear all;
% Mar. 15, 2017 Refactor

close all;
debug = 0;

if nargin < 2
    param = struct();
    param.dir_info = struct();
end
home_dirset = {'~/projects/tcell_simulation/'};
model_dirset = {'/projects/cellorganizer/xruan/tcell_project/tcell_excel_1pt_synapse_inference_2016092501/landmark-smoothing-method_none_intensity-morphing-method_none_models/'};
formatOut = 'yyyymmdd HH:MM:SS';

for i = 1 : length(home_dirset)
    if exist(home_dirset{i}, 'dir')
        home_dir = home_dirset{i};
        break;
    end
end

for i = 1 : length(model_dirset)
    if exist(model_dirset{i}, 'dir')
        default_model_dir = model_dirset{i};
        break;
    end
end

param = process_options_structure( struct('interpolation_method', 'spline' ...
                                      ), param);

param.dir_info = process_options_structure(struct('model_dir', default_model_dir), param.dir_info);
                             
model_dir = param.dir_info.model_dir;
data_dir = param.dir_info.data_dir;
data_subdir = param.dir_info.data_subdir;
mkdir(data_subdir);

% path(pathdef);
% addpath(genpath('~/cellorganizer'));
% addpath(genpath(home_dir));
% addpath(genpath([home_dir, filesep, 'code']));
% addpath(genpath(['/projects/cellorganizer/xruan/t_cell_project/tcell_clean/']));
% addpath(genpath(['~/cellorganizer_bu']));
% illustration_colormap_to_use = pmkmp(1024, 'CubicL'); 

load('subscripts_standard_voxels.mat');
template_image = zeros(71, 71, 35);
min_inds = min(subscripts);
max_inds = max(subscripts);
template_image(sub2ind(size(template_image), subscripts(:, 1), subscripts(:, 2), subscripts(:, 3))) = 1;
template_crop_function = @(I) I(min_inds(1) : max_inds(1), min_inds(2) : max_inds(2), min_inds(3) : max_inds(3));
template_crop_extend_function = @(I) I(min_inds(1) - 3 : max_inds(1) + 3, min_inds(2) - 3 : max_inds(2) + 3, min_inds(3) - 3: max_inds(3) + 3);
template_image_crop = template_crop_function(template_image);

model_dir_info = dir(model_dir);
model_dir_info(1 : 2) = [];

used_model_type = 'time_series_tracking';

used_model_pattern = [used_model_type, '.*','.mat'];

model_names = {model_dir_info.name};

used_model_names = model_names(~cellfun(@isempty, (regexp(model_names, used_model_pattern))));

used_model_info = regexp(used_model_names, '_', 'split');

conditions = {'Full Stimulus', 'B7 Blockade'};

used_model_conditions = cellfun(@(x) x{4}, used_model_info, 'UniformOutput', false);

used_model_sensor_names = cellfun(@(x) x{5}(1:end-4), used_model_info, 'UniformOutput', false);

unique_sensor_names = unique(used_model_sensor_names);

% manually set relative time and time 
relative_times = -2 : 9;
all_relative_times_seconds = [-40, -20, 0, 20, 40, 60, 80, 100, 120, 180, 300, 420];

time_interval = param.time_interval;

interporlation_time = all_relative_times_seconds(1) : time_interval : all_relative_times_seconds(end);

model_num = length(used_model_names);

model_means = zeros(model_num, numel(interporlation_time), 6628);

model_std = zeros(model_num, 6628);

model_data_interp = {};

[l, m, n] = size(template_image_crop);
[yo, xo, zo, to] = ndgrid(1 : l, 1 : m, 1 : n, all_relative_times_seconds);
[yq, xq, zq, tq] = ndgrid(1 : l, 1 : m, 1 : n, interporlation_time);

template_TS = repmat(template_image_crop, [1, 1, 1, 12]);
template_TS_interp = repmat(template_image_crop, [1, 1, 1, numel(interporlation_time)]);
for i = 1 : model_num
    current_model_name = used_model_names{i};
    cur_condition = used_model_conditions{i};
    cur_sensor = used_model_sensor_names{i};
    
    load([model_dir, current_model_name]);
    
    cell_num = numel(time_series_data);
    time_series_data_interp = cell(cell_num, 1);
    for j = 1 : cell_num
        cur_cell_TS = time_series_data{j};
        non_nan_inds =  ~isnan(sum(cur_cell_TS, 2));
        cur_cell_TS_non_nan = cur_cell_TS(non_nan_inds, :);
        cur_time_seconds = all_relative_times_seconds(non_nan_inds);
        [yo, xo, zo, to] = ndgrid(1 : l, 1 : m, 1 : n, cur_time_seconds);
        
        cur_cell_TS_4d = zeros(l, m, n, sum(non_nan_inds));
        cur_cell_TS_4d(template_TS(:, :, :, non_nan_inds) > 0) = reshape(cur_cell_TS_non_nan', [], 1);
        cur_cell_TS_4d_interp = interpn(yo, xo, zo, to, cur_cell_TS_4d, yq, xq, zq, tq, 'spline', NaN);
        cur_cell_TS_interp = reshape(cur_cell_TS_4d_interp(template_TS_interp > 0), [], numel(interporlation_time))';
        time_series_data_interp{j} = cur_cell_TS_interp;
        if false
            close all
            figure, imshow(reshape_2d(reshape(permute(cur_cell_TS_4d_interp, [1, 4, 2, 3]), [], size(cur_cell_TS_4d_interp, 2), size(cur_cell_TS_4d_interp, 3))), []);
        end
    end
    non_nan_inds = cell2mat(cellfun(@(x) ~isnan(sum(x, 2)), time_series_data_interp, 'UniformOutput', false)');
    time_series_data_interp_mat = cat(3, time_series_data_interp{:});
    time_series_data_interp_mat(isnan(time_series_data_interp_mat)) = 0;
    time_series_data_interp_mean = bsxfun(@rdivide, sum(time_series_data_interp_mat, 3), sum(non_nan_inds, 2));
    
    model_means(i, :, :) = time_series_data_interp_mean;
    model_data_i = time_series_data_interp;
    save(sprintf('%smodel_data_%s_%s', data_subdir, cur_condition, cur_sensor), 'model_data_i', 'time_series_data_interp_mean', 'all_relative_times_seconds', 'interporlation_time', 'param');
end

t_cell_info.model_data = struct;

t_cell_info.model_data.model_data = model_data_interp;

t_cell_info.model_data.model_means = model_means;

% t_cell_info.model_data.model_std = model_std;

t_cell_info.model_data.model_num = model_num;

t_cell_info.model_data.unique_sensor_names = unique_sensor_names;

t_cell_info.model_data.used_model_reltime = interporlation_time;

t_cell_info.model_data.all_relative_times_seconds = all_relative_times_seconds;

t_cell_info.model_data.interporlation_time = interporlation_time;

t_cell_info.model_data.used_model_conditions = used_model_conditions;

t_cell_info.model_data.used_model_names = used_model_names;

end






