function [t_cell_info] = master_script_get_tcell_model_data(t_cell_info, param)
% Get model data from t cell pipeline. 
% Copy from master_script_tcell_correlation_computation.m
%
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Oct. 12, 2016
% clear all;
% Mar. 15, 2017 Refactor
% Mar. 19, 2017 add check of data completion to make it more robust. 

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu





% Variables referenced more than once can be copied to local variables:
master_param = t_cell_info.param;
tcell_models_location = t_cell_info.path_info.tcell_models_location;
results_location = t_cell_info.path_info.results_location;
data_location = t_cell_info.path_info.data_location;
clustering_result_location = t_cell_info.path_info.clustering_result_location;
cross_validation_location = t_cell_info.path_info.cross_validation_location;
model_location = t_cell_info.path_info.model_location;
permutation_test_location = t_cell_info.path_info.permutation_test_location;
figure_location = t_cell_info.path_info.figure_location;

load('subscripts_standard_voxels.mat');
template_image = zeros(71, 71, 35);
min_inds = min(subscripts);
max_inds = max(subscripts);
template_image(sub2ind(size(template_image), subscripts(:, 1), subscripts(:, 2), subscripts(:, 3))) = 1;
template_crop_function = @(I) I(min_inds(1) : max_inds(1), min_inds(2) : max_inds(2), min_inds(3) : max_inds(3));
template_crop_extend_function = @(I) I(min_inds(1) - 3 : max_inds(1) + 3, min_inds(2) - 3 : max_inds(2) + 3, min_inds(3) - 3: max_inds(3) + 3);
template_image_crop = template_crop_function(template_image);

model_dir_info = dir(tcell_models_location);
model_dir_info(1 : 2) = [];

used_model_type = 'time_series_tracking';

used_model_pattern = [used_model_type, '.*','.mat'];

model_names = {model_dir_info.name};

used_model_names = model_names(~cellfun(@isempty, (regexp(model_names, used_model_pattern))));

used_model_info = regexp(used_model_names, '_', 'split');

conditions = {'Full Stimulus', 'B7 Blockade'};

used_model_conditions = cellfun(@(x) x{4}, used_model_info, 'UniformOutput', false);

used_model_sensor_names = cellfun(@(x) x{5}(1:end-4), used_model_info, 'UniformOutput', false);

unique_sensor_names = unique(used_model_sensor_names);

% manually set relative time and time 
all_relative_times = param.all_relative_times;
all_relative_times_seconds = param.all_timepoint;
is_interpolation = param.is_interpolation;

if ~is_interpolation
    interporlation_time = all_relative_times_seconds;
else
    error('The interpotation part has not been implemented!')
end

model_num = length(used_model_names);

model_means = zeros(model_num, numel(interporlation_time), 6628);

model_std = zeros(model_num, 6628);

model_data_interp = {};

[l, m, n] = size(template_image_crop);
[yo, xo, zo, to] = ndgrid(1 : l, 1 : m, 1 : n, all_relative_times_seconds);
[yq, xq, zq, tq] = ndgrid(1 : l, 1 : m, 1 : n, interporlation_time);

template_TS = repmat(template_image_crop, [1, 1, 1, 12]);
template_TS_interp = repmat(template_image_crop, [1, 1, 1, numel(interporlation_time)]);

is_computing = true;
Max_loops = 5;
loop_count = 0;
while is_computing && loop_count < Max_loops
    computed_mat = zeros(model_num, 1);
    chunk_lock_clean(data_location, 2 * 60);
    for i = 1 : model_num
        current_model_name = used_model_names{i};
        cur_condition = used_model_conditions{i};
        cur_sensor = used_model_sensor_names{i};
        cur_model_data_filename = sprintf('model_data_%s_%s', cur_condition, cur_sensor);
        [can_start, final_name, final_exists, temp_exists] = chunk_start_clean(data_location, cur_model_data_filename);
        if final_exists
            computed_mat(i) = 1;
            continue;
        end
        if ~can_start || final_exists
            continue;
        end

        load([tcell_models_location, current_model_name]);

        cell_num = numel(time_series_data);
        time_series_data_interp = cell(cell_num, 1);
        for j = 1 : cell_num
            cur_cell_TS = time_series_data{j};
            non_nan_inds =  ~isnan(sum(cur_cell_TS, 2));
            cur_cell_TS_non_nan = cur_cell_TS(non_nan_inds, :);
            cur_time_seconds = all_relative_times_seconds(non_nan_inds);
            [yo, xo, zo, to] = ndgrid(1 : l, 1 : m, 1 : n, cur_time_seconds);

            cur_cell_TS_4d = zeros(l, m, n, sum(non_nan_inds));
            cur_cell_TS_4d(template_TS(:, :, :, non_nan_inds) > 0) = reshape(cur_cell_TS_non_nan', [], 1);
            cur_cell_TS_4d_interp = interpn(yo, xo, zo, to, cur_cell_TS_4d, yq, xq, zq, tq, 'spline', NaN);
            cur_cell_TS_interp = reshape(cur_cell_TS_4d_interp(template_TS_interp > 0), [], numel(interporlation_time))';
            time_series_data_interp{j} = cur_cell_TS_interp;
            if false
                close all
                figure, imshow(reshape_2d(reshape(permute(cur_cell_TS_4d_interp, [1, 4, 2, 3]), [], size(cur_cell_TS_4d_interp, 2), size(cur_cell_TS_4d_interp, 3))), []);
            end
        end
        non_nan_inds = cell2mat(cellfun(@(x) ~isnan(sum(x, 2)), time_series_data_interp, 'UniformOutput', false)');
        time_series_data_interp_mat = cat(3, time_series_data_interp{:});
        time_series_data_interp_mat(isnan(time_series_data_interp_mat)) = 0;
        time_series_data_interp_mean = bsxfun(@rdivide, sum(time_series_data_interp_mat, 3), sum(non_nan_inds, 2));

        model_means(i, :, :) = time_series_data_interp_mean;
        model_data_i = time_series_data_interp;
        save([data_location, cur_model_data_filename], 'model_data_i', 'time_series_data_interp_mean', 'all_relative_times_seconds', 'interporlation_time', 'param');
        chunk_finish(data_location, cur_model_data_filename);
        computed_mat(i) = 1;
    end
    if all(computed_mat(:) == 1) 
        is_computing = false;
        break;
    end  
    pause(5 * 60);
    loop_count = loop_count + 1;    
end

model_data_info_filename = [data_location, 'model_data_info.mat'];
if ~exist(model_data_info_filename, 'file')
    for i = 1 : model_num
        cur_condition = used_model_conditions{i};
        cur_sensor = used_model_sensor_names{i};
        
        cur_model_data_filename = sprintf('model_data_%s_%s', cur_condition, cur_sensor);
        a = load([data_location, cur_model_data_filename]);
        model_means(i, :, :) = a.time_series_data_interp_mean;
    end
        
    model_data = struct;
    model_data.model_data = model_data_interp;
    model_data.model_means = model_means;
    model_data.model_num = model_num;
    model_data.unique_sensor_names = unique_sensor_names;
    model_data.used_model_reltime = interporlation_time;
    model_data.all_relative_times_seconds = all_relative_times_seconds;
    model_data.interporlation_time = interporlation_time;
    model_data.used_model_conditions = used_model_conditions;
    model_data.used_model_names = used_model_names;
    model_data.template_image = template_image;
    model_data.template_image_crop = template_image_crop;
    model_data.template_crop_function = template_crop_function;
    model_data.template_crop_extend_function = template_crop_extend_function;
    
    save(model_data_info_filename, 'model_data');
else
    load(model_data_info_filename);
end

t_cell_info.model_data = model_data;

end






