function [t_cell_info] = master_script_plot_adjacency_network_comparison(t_cell_info)
% compare networks for two methods
% 
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Date: Mar. 17, 2017
%
% copied from master_script_network_comparison.m

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




close all;

param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;
suffix = param.suffix;

graph_data = t_cell_info.graph_data;
kept_nodes = t_cell_info.graph_data.kept_nodes;
all_sensor_region_combine = graph_data.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);

model_location = t_cell_info.path_info.model_location;
model_sub_location = t_cell_info.path_info.model_sub_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;


template_crop_function = t_cell_info.model_data.template_crop_function;
model_region_function = @(x) reshape_2d(template_crop_function(model_reconstruction(map_label_vec == x, subscripts)));
model_region_3d_function = @(x) template_crop_function(model_reconstruction(map_label_vec == x, subscripts));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model_1_filename = sprintf('%sCGP_%s/clustering_ROI_results_CGP_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d.mat', model_location, opt_std, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh);
model_2_filename = sprintf('%sCENR_%s/clustering_ROI_results_CENR_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d.mat', model_location, opt_std, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh);

if ~exist(model_1_filename, 'file') 
    warning('%s does not exist!', model_1_filename);
    return;
end
    
if ~exist(model_2_filename, 'file') 
    warning('%s does not exist!', model_2_filename);
    return;
end
   
%load models

a = load(model_1_filename);
b = load(model_2_filename);

% for CGP, use A, and For elastic net, use the combined A

A_cgp = a.A;
R_t_3d = b.R_t_3d;
A_en = sign(sum(R_t_3d, 3));
A_en = max(A_en .* R_t_3d, [], 3) .* A_en;
% A_1 = A;
A_en(eye(size(A_en, 1)) == 1) = 0;
N_node = size(A_cgp, 1);

unique_vals = unique([abs(A_cgp(:)); abs(A_en(:))]);

N_thresh = numel(unique_vals)

A_cgp_adj = zeros(N_node);
A_en_adj = zeros(N_node);
net_dists = zeros(N_thresh, 2);
net_similarity = zeros(N_thresh, 1);

for i = 1 : N_thresh
    thresh_i = unique_vals(N_thresh - i + 1);
    A_cgp_adj = (A_cgp >= thresh_i) - (A_cgp <= -thresh_i);
    A_en_adj = (A_en >= thresh_i) - (A_en <= -thresh_i);
    net_dists(i, 1) = sum(abs(A_cgp_adj(:) - A_en_adj(:))) ./ sum(abs(A_cgp_adj(:)) + abs(A_en_adj(:)) > 0);
    net_dists(i, 2) = sum(abs(A_cgp_adj(:) - A_en_adj(:) ~= 0)) ./ sum(abs(A_cgp_adj(:)) + abs(A_en_adj(:)) > 0);
    net_similarity(i, 1) = sum(abs(A_cgp_adj(:) .* A_en_adj(:) == 1)) ./ sum(abs(A_cgp_adj(:)) + abs(A_en_adj(:)) > 0);
end

if ~false 
    dpi = 300;
    figure, plot(flip(unique_vals), net_similarity)
    xlabel('thresh');
    ylabel('Similarity');    
            title('Same thresh similarity')

    figure_filename = sprintf('%sclustering_cgp_elastic_same_thresh_similarity_%d_same_thresh_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, i, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      
    
    figure, plot(flip(unique_vals), net_dists(:, 1))
    xlabel('thresh');
    ylabel('Distance');
            title('Same thresh distance')

    figure_filename = sprintf('%sclustering_cgp_elastic_same_thresh_net_dism_1_same_thresh_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      

    figure, plot(flip(unique_vals), net_dists(:, 2))
    xlabel('thresh');
    ylabel('Distance');
        title('Same thresh binary distance')

    figure_filename = sprintf('%sclustering_cgp_elastic_same_thresh_dism_2_same_thresh_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      
end

% compare by the same number of edges.
N_edge_1 = sum(A_cgp(:) > 0);
N_edge_2 = sum(A_en(:) > 0);
unique_vals_1 = unique(abs(A_cgp(:)));
unique_vals_2 = unique(abs(A_en(:)));

N_edge_compare = min(N_edge_1, N_edge_2);

A_cgp_adj = zeros(N_node);
A_en_adj = zeros(N_node);
net_dists = zeros(N_edge_compare, 2);
net_similarity = zeros(N_edge_compare, 1);
for i = 1 : N_edge_compare
    thresh_i1 = unique_vals_1(N_edge_1 - i + 1);
    thresh_i2 = unique_vals_2(N_edge_2 - i + 1);
    A_cgp_adj = (A_cgp >= thresh_i1) - (A_cgp <= -thresh_i1);
    A_en_adj = (A_en >= thresh_i2) - (A_en <= -thresh_i2);
    net_dists(i, 1) = sum(abs(A_cgp_adj(:) - A_en_adj(:))) ./ sum(abs(A_cgp_adj(:)) + abs(A_en_adj(:)) > 0);
    net_dists(i, 2) = sum(abs(A_cgp_adj(:) - A_en_adj(:) ~= 0)) ./ sum(abs(A_cgp_adj(:)) + abs(A_en_adj(:)) > 0);
    net_similarity(i, 1) = sum(abs(A_cgp_adj(:) .* A_en_adj(:) == 1)) ./ sum(abs(A_cgp_adj(:)) + abs(A_en_adj(:)) > 0);
end
if ~false 
    dpi = 300;    
    figure, plot(1 : N_edge_compare, net_similarity)
    xlabel('Edge number');
    ylabel('Similarity');    
    title('Same edge similarity')
    figure_filename = sprintf('%sclustering_cgp_elastic_same_edge_similarity_%d_same_thresh_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, i, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      
    
    figure, plot(1 : N_edge_compare, net_dists(:, 1))
    xlabel('Edge number');
    ylabel('Distance');
    title('Same edge distance')
    
    figure_filename = sprintf('%sclustering_cgp_elastic_same_edge_net_dism_1_same_thresh_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      

    figure, plot(1 : N_edge_compare, net_dists(:, 2))
    xlabel('Edge number');
    ylabel('Distance');
    title('Same edge binary distance')
    
    figure_filename = sprintf('%sclustering_cgp_elastic_same_edge_dism_2_same_thresh_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      
    
end

% identify robust edges, colored by three kinds of colors, one is common
% edges, and the other two are different colors that indicate the
% differences. 

unique_vals = unique([abs(A_cgp(:)); abs(A_en(:))]);

N_thresh = numel(unique_vals)

A_cgp_adj = zeros(N_node);
A_en_adj = zeros(N_node);
net_dists = zeros(N_thresh, 2);
net_similarity = zeros(N_thresh, 1);
dpi = 300;

for i = 30
    thresh_i = unique_vals(N_thresh - i + 1);
    A_cgp_adj = A_cgp .* (abs(A_cgp) >= thresh_i);
    A_en_adj = A_en .* (abs(A_en) >= thresh_i);
    A_common_ind =  (abs(A_cgp_adj) > 0) .* (abs(A_en_adj) > 0) == 1;
    
    A_sign = sign((A_cgp_adj + A_en_adj) .* A_common_ind);
    A_common = max(A_sign .* reshape([A_cgp_adj, A_en_adj], [size(A_cgp_adj), 2]), [], 3) .* A_sign;

    A_cgp_diff = A_cgp_adj .* ((abs(A_cgp_adj) > 0) - A_common_ind);
    A_en_adj_diff = A_en_adj .* ((abs(A_en_adj) > 0) - A_common_ind);
    h = network_comparison_plot_network(A_common, A_cgp_diff, A_en_adj_diff, kept_node_names);
    title(sprintf('Common edge number:%d', sum(A_common_ind(:))));
    figure_filename = sprintf('%sclustering_network_combine_%d_same_thresh_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, i, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      
    if rem(i, 20) == 0
        close all;
    end

end

% same number of edges

unique_vals_1 = unique(abs(A_cgp(:)));
unique_vals_2 = unique(abs(A_en(:)));
N_edge_1 = numel(unique_vals_1);
N_edge_2 = numel(unique_vals_2);

N_edge_compare = min(N_edge_1, N_edge_2);

A_cgp_adj = zeros(N_node);
A_en_adj = zeros(N_node);
net_dists = zeros(N_edge_compare, 2);
net_similarity = zeros(N_edge_compare, 1);
for i = 20 : 30
    thresh_i1 = unique_vals_1(N_edge_1 - i + 1);
    thresh_i2 = unique_vals_2(N_edge_2 - i + 1);
    A_cgp_adj = A_cgp .* (abs(A_cgp) >= thresh_i1);
    A_en_adj = A_en .* (abs(A_en) >= thresh_i2);
    A_common_ind =  (abs(A_cgp_adj) > 0) .* (abs(A_en_adj) > 0) == 1;
    
    A_sign = sign((A_cgp_adj + A_en_adj) .* A_common_ind);
    A_common = max(A_sign .* reshape([A_cgp_adj, A_en_adj], [size(A_cgp_adj), 2]), [], 3) .* A_sign;

    A_cgp_diff = A_cgp_adj .* ((abs(A_cgp_adj) > 0) - A_common_ind);
    A_en_adj_diff = A_en_adj .* ((abs(A_en_adj) > 0) - A_common_ind);
    h = network_comparison_plot_network(A_common, A_cgp_diff, A_en_adj_diff, kept_node_names);
    title(sprintf('Common edge number:%d', sum(A_common_ind(:))));
    
    figure_filename = sprintf('%sclustering_network_combine_%d_same_edge_num_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, i, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      
    if rem(i, 20) == 0
        close all;
    end
end

end