function [t_cell_info] = master_script_plot_kinetics(t_cell_info)
% plot kinetics with high values
% 
% copied from master_script_CGP_clustering_ROI_postprocessing.m
% 
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Date: Mar. 17, 2017

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


   
close all;

param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;
suffix = param.suffix;

graph_data = t_cell_info.graph_data;
kept_nodes = t_cell_info.graph_data.kept_nodes;
all_sensor_region_combine = graph_data.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);

illustration_colormap_to_use = t_cell_info.param.illustration_colormap_to_use;

model_sub_location = t_cell_info.path_info.model_sub_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;

% [R_t_3d, A, c] = master_script_causal_graph_process(X, M) 
results_filename = sprintf('%sclustering_ROI_results_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d%s', model_sub_location, causal_method, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh, suffix);

results = load(results_filename);
A = results.A;
try
    c = results.c;
    R_t_3d = results.R_t_3d;
end
X = t_cell_info.graph_data.X;

% combine positive and negative effects in kinetics plots. 
% put positive and negative effect together. 

N_lag = 20;
pair_method = 'adjacent_mat';

N_plot_group = [8];

for i0 = 1 : numel(N_plot_group)
    N_plot = N_plot_group(i0);
    switch pair_method
        case 'adjacent_mat'
            A_od = A;
            diagonal_inds = find(eye(size(A)) == 1);
            A_od(eye(size(A)) == 1) = 0;

            [A_vec_sort, inds_sort] = sort(abs(A_od(:)), 1, 'descend');
            vals_together = A_vec_sort(1 : N_plot);
            inds_together = inds_sort(1 : N_plot);

            A_vec = A_od(:);

            pos_vals = vals_together(A_vec(inds_together) > 0);
            pos_inds = inds_together(A_vec(inds_together) > 0);

            neg_vals = -vals_together(A_vec(inds_together) < 0);
            neg_inds = inds_together(A_vec(inds_together) < 0);
        case 'prediction'
            [norm_diff_x] = most_influence_pairs_by_prediction(X, A, c);

            % use maximum as criterion
            max_diff_x = max(norm_diff_x, [], 3);
            min_diff_x = min(norm_diff_x, [], 3);
            diff_x_tog = (max_diff_x > -min_diff_x) .* max_diff_x + (max_diff_x <= -min_diff_x) .* min_diff_x;

            diagonal_inds = find(eye(size(A)) == 1);
            diff_x_tog(eye(size(A)) == 1) = 0;

            [P_vec_sort, inds_sort] = sort(abs(diff_x_tog(:)));
            vals_together = P_vec_sort(end - N_plot + 1 : end);
            inds_together = inds_sort(end - N_plot + 1 : end);

            P_vec = diff_x_tog(:);
            pos_vals = vals_together(P_vec(inds_together) > 0);
            pos_inds = inds_together(P_vec(inds_together) > 0);

            neg_vals = -vals_together(P_vec(inds_together) < 0);
            neg_inds = inds_together(P_vec(inds_together) < 0);   
    end


    % cobmine the two cases
    % inds_together = [pos_inds; neg_inds];

    [pn_y_inds, pn_x_inds] = ind2sub(size(A), inds_together);

    y_inds_unique = unique(pn_y_inds);
    x_inds_unique = unique(pn_x_inds);

    yx_inds_unique = unique([y_inds_unique; x_inds_unique]);

    m = numel(yx_inds_unique);

    % ordered by absolute values
    if ~false
        figure, 
        set(gcf, 'color', 'w')
        set(gcf, 'Visible', 'on')
        % set(gcf, 'Visible', 'off')

        hold on
        aspect_ration = 10/16;

        per_image_size = 250;
        m = 5;
        n = 12;

        figure_width = per_image_size * n * 1.5;
        figure_height = round(figure_width / aspect_ration);
        border_width = 0.1;
        border_hight = 0.2;
        per_height = 0.14;
        per_width = 0.14;
        per_height_1 = 0.07;
        per_width_1 = 0.14;

        w_interval = 0.05;
        h_interval = 0.08;
        h_interval_1 = 0.04;

        set(gcf, 'Position', [1, 1, figure_width, figure_height]);
        axis off;

        cm = @jet;

        colors = cm(9);
        y_color = [0, 0, 0];

        figure_handles = {};
        curr_row = 0;
        curr_col = 0;

        plot_count = 1;
        for i = 1 : N_plot
            curr_y_ind = pn_y_inds(i);
            curr_x_ind = pn_x_inds(i);
            
            curr_row = ceil(plot_count / 4);
            curr_col = rem(plot_count, 4);
            if curr_col == 0
                curr_col = 4;
            end

            pn_y_node_name = kept_node_names{curr_y_ind};
            pn_x_node_name = kept_node_names{curr_x_ind};
            A_ind = sub2ind(size(A), curr_y_ind, curr_x_ind);
            if any(pos_inds == A_ind) 
                pn_flag = 'Pos';
            else
                pn_flag = 'Neg';
            end

            pn_y_plot = X(curr_y_ind, :);
            pn_x_plot = X(curr_x_ind, :);            

            A(curr_y_ind, curr_x_ind)

            curr_position = [border_width + ( w_interval + per_width) * (curr_col - 1), 1 - (border_hight + (h_interval + h_interval_1 + per_height + per_height_1) * (curr_row -1) + per_height), per_width, per_height];

            axes('Position', curr_position);

            [ax, p1, p2] = plotyy(current_timepoint, pn_x_plot, current_timepoint, pn_y_plot, 'plot', 'plot');    
            p1.Color = [0, 0, 0];
            p2.Color = [1, 0, 0];
            ax(1).YColor = 'k';
            ax(2).YColor = 'r';
            p2.LineStyle = ':';
            p1.LineWidth = 2;
            p2.LineWidth = 2;
            xlim(ax(1), [current_timepoint(1), current_timepoint(end)]);
            xlim(ax(2), [current_timepoint(1), current_timepoint(end)])

            % set(gca,'YTickLabel',[]);
            % axis off
            % set(gca, 'ylim', [min([ylim_1, ylim_2]), max([ylim_1, ylim_2])])
            switch causal_method
                case 'test'
                    title(sprintf('%d %s -> %d  Val: %0.4f', curr_x_ind, pn_flag, curr_y_ind, A(curr_y_ind, curr_x_ind)), 'FontSize', 9);
                otherwise
                    title(sprintf('%s %s -> %s  Val: %0.4f',  pn_x_node_name, pn_flag, pn_y_node_name, A(curr_y_ind, curr_x_ind)), 'FontSize', 9);
            end
            % plot cross correlation

            % [acor, lag] = xcorr(pn_x_plot, pn_y_plot, []);
            [acro, lag] = xcorr_compute(pn_y_plot, pn_x_plot);
    %             acor(lag == 0) = [];
    %             lag(lag == 0) = [];
            curr_position = [border_width + ( w_interval + per_width) * (curr_col - 1), 1 - (border_hight + (h_interval + h_interval_1 + per_height + per_height_1) * (curr_row -1) + h_interval_1 + per_height + per_height_1), per_width_1, per_height_1];
            axes('Position', curr_position);
            plot(lag(N_lag + 1 : end - N_lag), acro(N_lag + 1 : end - N_lag), 'bo', 'linewidth', 1.5)
            ax = gca;

            plot_count = plot_count + 1;
        end    

        hold off 
        dpi = 150;
        figure_filename = sprintf('%sclustering_ROI_top_ordered_kinetic_N_%d_%s_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, N_plot, pair_method, cluster_num, M, vol_thresh);
        export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% for lag time larger than plot all
N_plot_group = [8];
N_lag = min(round(0.6 * size(X, 2)), size(X, 2) - M - 2);

for i0 = 1 : numel(N_plot_group)
    N_plot = N_plot_group(i0);
    for j = 1 : M
        Aj = R_t_3d(:, :, j);
        switch pair_method
            case 'adjacent_mat'
                A_od = Aj;
                diagonal_inds = find(eye(size(Aj)) == 1);
                A_od(eye(size(Aj)) == 1) = 0;

                [A_vec_sort, inds_sort] = sort(abs(A_od(:)), 1, 'descend');
                vals_together = A_vec_sort(1 : N_plot);
                inds_together = inds_sort(1 : N_plot);

                A_vec = A_od(:);

                pos_vals = vals_together(A_vec(inds_together) > 0);
                pos_inds = inds_together(A_vec(inds_together) > 0);

                neg_vals = -vals_together(A_vec(inds_together) < 0);
                neg_inds = inds_together(A_vec(inds_together) < 0);
        end


        % cobmine the two cases
        % inds_together = [pos_inds; neg_inds];

        [pn_y_inds, pn_x_inds] = ind2sub(size(Aj), inds_together);

        y_inds_unique = unique(pn_y_inds);
        x_inds_unique = unique(pn_x_inds);

        yx_inds_unique = unique([y_inds_unique; x_inds_unique]);

        m = numel(yx_inds_unique);

        % set a new kind of figure

        % ordered by absolute values
        if ~false
            figure, 
            set(gcf, 'color', 'w')
            set(gcf, 'Visible', 'on')
            % set(gcf, 'Visible', 'off')

            hold on
            aspect_ration = 10/16;

            per_image_size = 250;
            m = 5;
            n = 12;

            figure_width = per_image_size * n * 1.5;
            figure_height = round(figure_width / aspect_ration);
            border_width = 0.1;
            border_hight = 0.2;
            per_height = 0.14;
            per_width = 0.14;
            per_height_1 = 0.07;
            per_width_1 = 0.14;

            w_interval = 0.05;
            h_interval = 0.08;
            h_interval_1 = 0.04;

            set(gcf, 'Position', [1, 1, figure_width, figure_height]);
            axis off;

            cm = @jet;

            colors = cm(9);
            y_color = [0, 0, 0];

            figure_handles = {};
            curr_row = 0;
            curr_col = 0;

            plot_count = 1;
            for i = 1 : N_plot
                curr_y_ind = pn_y_inds(i);
                curr_x_ind = pn_x_inds(i);

                curr_row = ceil(plot_count / 4);
                curr_col = rem(plot_count, 4);
                if curr_col == 0
                    curr_col = 4;
                end

                pn_y_node_name = kept_node_names{curr_y_ind};
                pn_x_node_name = kept_node_names{curr_x_ind};

                A_ind = sub2ind(size(A), curr_y_ind, curr_x_ind);
                if any(pos_inds == A_ind) 
                    pn_flag = 'Pos';
                else
                    pn_flag = 'Neg';
                end

                pn_y_plot = X(curr_y_ind, :);
                pn_x_plot = X(curr_x_ind, :);            

                Aj(curr_y_ind, curr_x_ind)

                curr_position = [border_width + ( w_interval + per_width) * (curr_col - 1), 1 - (border_hight + (h_interval + h_interval_1 + per_height + per_height_1) * (curr_row -1) + per_height), per_width, per_height];

                axes('Position', curr_position);

                [ax, p1, p2] = plotyy(current_timepoint, pn_x_plot, current_timepoint, pn_y_plot, 'plot', 'plot');    
                p1.Color = [0, 0, 0];
                p2.Color = [1, 0, 0];
                ax(1).YColor = 'k';
                ax(2).YColor = 'r';
                p2.LineStyle = ':';
                p1.LineWidth = 2;
                p2.LineWidth = 2;
                xlim(ax(1), [current_timepoint(1), current_timepoint(end)]);
                xlim(ax(2), [current_timepoint(1), current_timepoint(end)])

                % set(gca,'YTickLabel',[]);
                % axis off
                % set(gca, 'ylim', [min([ylim_1, ylim_2]), max([ylim_1, ylim_2])])
                switch causal_method
                    case 'test'
                        title(sprintf('%d %s -> %d  Val: %0.4f', curr_x_ind, pn_flag, curr_y_ind, Aj(curr_y_ind, curr_x_ind)), 'FontSize', 9);
                    otherwise
                        title(sprintf('%s %s -> %s  Val: %0.4f',  pn_x_node_name, pn_flag, pn_y_node_name, A(curr_y_ind, curr_x_ind)), 'FontSize', 9);
                end
                % plot cross correlation

                % [acor, lag] = xcorr(pn_x_plot, pn_y_plot, []);
                [acro, lag] = xcorr_compute(pn_y_plot, pn_x_plot);
        %             acor(lag == 0) = [];
        %             lag(lag == 0) = [];
                curr_position = [border_width + ( w_interval + per_width) * (curr_col - 1), 1 - (border_hight + (h_interval + h_interval_1 + per_height + per_height_1) * (curr_row -1) + h_interval_1 + per_height + per_height_1), per_width_1, per_height_1];
                axes('Position', curr_position);
                plot(lag(N_lag + 1 : end - N_lag), acro(N_lag + 1 : end - N_lag), 'bo', 'linewidth', 1.5)
                ax = gca;

                plot_count = plot_count + 1;
            end    

            hold off 
            dpi = 150;
            figure_filename = sprintf('%sclustering_ROI_A%d_top_ordered_kinetic_N_%d_%s_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, j, N_plot, pair_method, cluster_num, M, vol_thresh);
            export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         
        end
    end
end


end
