function [t_cell_info] = master_script_plot_network_ROC_curve(t_cell_info)
% compare networks for two methods
% 
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Date: Mar. 17, 2017
%
% copied from master_script_network_ROC_plot.m

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu





close all;

param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;
suffix = param.suffix;

graph_data = t_cell_info.graph_data;
kept_nodes = t_cell_info.graph_data.kept_nodes;
all_sensor_region_combine = graph_data.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);
unique_sensor_names = t_cell_info.model_data.unique_sensor_names;


model_location = t_cell_info.path_info.model_location;
model_sub_location = t_cell_info.path_info.model_sub_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load('Interaction_ground_truth_01232017.mat');

% visualize groundtruth network
G_raw = ground_truth;

if param.merge_proteins
    group_inds = param.group_inds;
    unique_sensor_names = cellfun(@(x) unique_sensor_names{min(x)}, group_inds, 'Uniformoutput', false);

    G_adj = zeros(numel(group_inds));
    for i = 1 : numel(group_inds)
        for j = 1 : numel(group_inds)
            if i == j 
                continue;
            end
            G_adj(i, j) = sum(reshape(G_raw(group_inds{i}, group_inds{j}), 1, [])) > 0;
        end
    end
else
    G_adj = G_raw;
end

labels = G_adj;
labels(eye(size(labels, 1)) == 1) = [];

G = graph(G_adj, unique_sensor_names);

figure,
set(gcf, 'color', 'w')
set(gcf, 'Visible', 'on')
% set(gcf, 'Visible', 'off')

figure_width = 500;
figure_height = 400;
dpi = 300;

set(gcf, 'Position', [1, 1, figure_width, figure_height]);
p1 = plot(G, 'LineWidth', 2, 'NodeLabel', G.Nodes.Name, 'Layout', 'layered');
axis off;
box off;
figure_filename = sprintf('%sclustering_ground_truth_network_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);     


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model_1_filename = sprintf('%sCGP_%s/clustering_ROI_results_CGP_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d.mat', model_location, opt_std, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh);
model_2_filename = sprintf('%sCENR_%s/clustering_ROI_results_CENR_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d.mat', model_location, opt_std, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh);

if ~exist(model_1_filename, 'file') 
    warning('%s does not exist!', model_1_filename);
    return;
end
    
if ~exist(model_2_filename, 'file') 
    warning('%s does not exist!', model_2_filename);
    return;
end
   
a = load(model_1_filename);
b = load(model_2_filename);

% for CGP, use A, and For elastic net, first use absolute value to see the
% hit rates. 

A_cgp = a.A;
R_t_3d = b.R_t_3d;
A_en = sign(sum(R_t_3d, 3));
A_en = max(A_en .* R_t_3d, [], 3) .* A_en;
% A_1 = A;
A_en(eye(size(A_en, 1)) == 1) = 0;
N_node = size(A_cgp, 1);

A_cgp_1 = abs(A_cgp(1 : 3 : end, 1 : 3 : end));
A_cgp_1(eye(N_node / 3) == 1) = [];
A_en_1 = abs(A_en(1 : 3 : end, 1 : 3 : end));
A_en_1(eye(N_node / 3) == 1) = [];
[Y_cgp, X_cgp] = roc(labels, A_cgp_1);
if Y_cgp(end) ~= 1 || X_cgp(end) ~= 1
    Y_cgp = [Y_cgp, 1];
    X_cgp = [X_cgp, 1];
end
AUC_cgp = Y_cgp(2 : end) * diff(X_cgp)'
[Y_en, X_en] = roc(labels, A_en_1);
if Y_en(end) ~= 1 || X_en(end) ~= 1
    Y_en = [Y_en, 1];
    X_en = [X_en, 1];
end
AUC_en = Y_en(2 : end) * diff(X_en)'


figure, 

set(gcf, 'Visible', 'off')
set(gcf, 'color', 'w')
set(gcf, 'Visible', 'on')
set(gcf, 'Renderer', 'opengl');
dpi = 300;
figure_height = 750;
figure_width = 900;
set(gcf, 'Position', [1920, 1, figure_width, figure_height]);
plot(X_cgp, Y_cgp)
hold on
plot(X_en, Y_en)
hold on 
plot(0 : 0.01 : 1, 0 : 0.01 : 1, '--')
hold off
legend({sprintf('CGP : %.4f', AUC_cgp), sprintf('Elastic net : %.4f', AUC_en)}, 'Location', 'southeast');
xlabel('False positive rate')
ylabel('True positive rate')
title('ROC for Network Reconstruction')
set(findobj(gca, '-property', 'LineWidth'), 'LineWidth', 3);
set(findobj(gca, '-property', 'FontSize'), 'FontSize', 21);
set(findobj(gca, '-property', 'FontWeight'), 'FontWeight', 'bold');

figure_filename = sprintf('%sclustering_AUC_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      


% consider all three regions. 
labels_1 = kron(G_adj, ones(cluster_num));
labels_1(eye(size(labels_1, 1)) == 1) = [];

A_cgp_1 = abs(A_cgp);
A_cgp_1(eye(N_node) == 1) = [];
A_en_1 = abs(A_en);
A_en_1(eye(N_node) == 1) = [];
[Y_cgp, X_cgp] = roc(labels_1, A_cgp_1);
if Y_cgp(end) ~= 1 || X_cgp(end) ~= 1
    Y_cgp = [Y_cgp, 1];
    X_cgp = [X_cgp, 1];
end
AUC_cgp = Y_cgp(2 : end) * diff(X_cgp)'
[Y_en, X_en] = roc(labels_1, A_en_1);
if Y_en(end) ~= 1 || X_en(end) ~= 1
    Y_en = [Y_en, 1];
    X_en = [X_en, 1];
end
AUC_en = Y_en(2 : end) * diff(X_en)'

figure, 
plot(X_cgp, Y_cgp)
hold on
plot(X_en, Y_en)
hold on 
plot(0 : 0.01 : 1, 0 : 0.01 : 1, '--')
hold off
legend({sprintf('CGP : %.4f', AUC_cgp), sprintf('Elastic net : %.4f', AUC_en)}, 'Location', 'southeast');
xlabel('False positive rate')
ylabel('True positive rate')
title('ROC for Classification')
figure_filename = sprintf('%sclustering_ROC_AUC_all_regions_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      

% only consider same region interaction

labels_1 = kron(G_adj, eye(cluster_num));
labels_1(eye(size(labels_1, 1)) == 1) = [];

A_cgp_1 = abs(A_cgp);
A_cgp_1(eye(N_node) == 1) = [];
A_en_1 = abs(A_en);
A_en_1(eye(N_node) == 1) = [];
[Y_cgp, X_cgp] = roc(labels_1, A_cgp_1);
if Y_cgp(end) ~= 1 || X_cgp(end) ~= 1
    Y_cgp = [Y_cgp, 1];
    X_cgp = [X_cgp, 1];
end
AUC_cgp = Y_cgp(2 : end) * diff(X_cgp)'
[Y_en, X_en] = roc(labels_1, A_en_1);
if Y_en(end) ~= 1 || X_en(end) ~= 1
    Y_en = [Y_en, 1];
    X_en = [X_en, 1];
end
AUC_en = Y_en(2 : end) * diff(X_en)'

figure, 
plot(X_cgp, Y_cgp)
hold on
plot(X_en, Y_en)
hold on 
plot(0 : 0.01 : 1, 0 : 0.01 : 1, '--')
hold off
legend({sprintf('CGP : %.4f', AUC_cgp), sprintf('Elastic net : %.4f', AUC_en)}, 'Location', 'southeast');
xlabel('False positive rate')
ylabel('True positive rate')
title('ROC for Classification')
figure_filename = sprintf('%sclustering_ROC_AUC_same_regions_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);      


% use mean for all regions to define the score

labels_1 = G_adj;
labels_1(eye(size(labels_1, 1)) == 1) = [];

A_cgp_1 = reshape(abs(A_cgp), cluster_num, size(G_adj, 1), cluster_num, size(G_adj, 2));
A_cgp_1 = squeeze(mean(mean(A_cgp_1, 3), 1));
A_cgp_1(eye(size(labels_1, 1)) == 1) = [];
A_en_1 = reshape(abs(A_en), cluster_num, size(G_adj, 1), cluster_num, size(G_adj, 2));
A_en_1 = squeeze(mean(mean(A_en_1, 3), 1));
A_en_1(eye(size(labels_1, 1)) == 1) = [];

[Y_cgp, X_cgp] = roc(labels_1, A_cgp_1);
if Y_cgp(end) ~= 1 || X_cgp(end) ~= 1
    Y_cgp = [Y_cgp, 1];
    X_cgp = [X_cgp, 1];
end
AUC_cgp = Y_cgp(2 : end) * diff(X_cgp)'
[Y_en, X_en] = roc(labels_1, A_en_1);
if Y_en(end) ~= 1 || X_en(end) ~= 1
    Y_en = [Y_en, 1];
    X_en = [X_en, 1];
end
AUC_en = Y_en(2 : end) * diff(X_en)'

figure, 
plot(X_cgp, Y_cgp)
hold on
plot(X_en, Y_en)
hold on 
plot(0 : 0.01 : 1, 0 : 0.01 : 1, '--')
hold off
legend({sprintf('CGP : %.4f', AUC_cgp), sprintf('Elastic net : %.4f', AUC_en)}, 'Location', 'southeast');
xlabel('False positive rate')
ylabel('True positive rate')
title('ROC for Classification')
figure_filename = sprintf('%sclustering_ROC_AUC_mean_score_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
export_fig(figure_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)]);        

end