function [t_cell_info] = master_script_plot_spatiotemporal_network(t_cell_info)
% plot the spatiotemporal network
%
% Author: Xiongtao Ruan
% Date: Mar. 17, 2017
%
% copied from master_script_CGP_temporal_cluster_postprocess_network_seqntl.m

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu





close all;

param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;
suffix = param.suffix;

% specific params
ct_method = 'xcorr';
critical_method = 'xcorr';


graph_data = t_cell_info.graph_data;
kept_nodes = t_cell_info.graph_data.kept_nodes;
all_sensor_region_combine = graph_data.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);

model_sub_location = t_cell_info.path_info.model_sub_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;

% [R_t_3d, A, c] = master_script_causal_graph_process(X, M) 
results_filename = sprintf('%sclustering_ROI_results_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d%s', model_sub_location, causal_method, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh, suffix);

results = load(results_filename);
A = results.A;
c = results.c;
X = t_cell_info.graph_data.X;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% put positive and negative effect together. 

N_influent = 20;

[A_vec_sort, inds_sort] = sort(A(:));
pos_vals = A_vec_sort(end - N_influent + 1 : end);
pos_inds = inds_sort(end - N_influent + 1 : end);

neg_vals = A_vec_sort(1 : N_influent);
neg_inds = inds_sort(1 : N_influent);

if min(abs(pos_vals)) < min(abs(neg_vals))
    neg_inds = find(A <= -min(abs(pos_vals)));
    neg_val = A(neg_inds);
end
if min(abs(pos_vals)) > min(abs(neg_vals))
    [neg_inds] = find(A <= -min(abs(pos_vals)));
    neg_val = A(neg_inds);    
end

inds_together = [pos_inds; neg_inds];

[pn_y_inds, pn_x_inds] = ind2sub(size(A), inds_together);

y_inds_unique = unique(pn_y_inds);
x_inds_unique = unique(pn_x_inds);

yx_inds_unique = unique([y_inds_unique; x_inds_unique]);

m = numel(yx_inds_unique);

A_cm = zeros(m);

pn_onset_time = zeros(size(inds_together, 1), 1);
pn_trend = zeros(size(inds_together, 1), 2);
for i = 1 : numel(pn_y_inds)
    I_y = X(pn_y_inds(i), :);
    I_x = X(pn_x_inds(i), :);
    param_ct.lag = M;
    param_ct.method = ct_method;
    if i <= numel(pos_inds)
        param_ct.direction = 'pos';
    else
        param_ct.direction = 'neg';
    end  
    switch critical_method
        case 'xcorr'
            % param_ct.method = 'xcorr';
            [pn_onset_time(i), pn_trend(i, :)] = critical_timepoint_extract(I_x, I_y, param_ct);   
        case 'lrpredict'
            % param_ct.method = 'overall';
            [pn_onset_time(i), pn_trend(i, :)] = critical_timepoint_extract_1(X, A, c, pn_x_inds(i), pn_y_inds(i), param_ct);   
    end
end

for i = 1 : numel(pn_y_inds)
    y_ind = pn_y_inds(i);
    x_ind = pn_x_inds(i);
    
    m_i = find(yx_inds_unique == y_ind);
    n_i = find(yx_inds_unique == x_ind);
    % A_cm(m_i, n_i) = neg_vals(i);   
    A_cm(m_i, n_i) = current_timepoint(pn_onset_time(i));    
    if current_timepoint(pn_onset_time(i)) == 0 
    % temporately disable some edges. 
    % if current_timepoint(pn_onset_time(i)) == 0 && m_i ~= 17 && m_i ~= 3 
        A_cm(m_i, n_i) = 0.1;
    end
end

% delete self connection if needed
if ~false
    A_cm(eye(size(A_cm, 1)) > 0) = 0;
end

% [curr_region_inds, curr_sensor_inds] = ind2sub([len_per_sensor, numel(unique_sensor_names)], 1 : size(A, 1));
curr_nodes_names = kept_node_names;

% convert the edges to node and time specific
nt = numel(current_timepoint);

self_inds = find(pn_y_inds == pn_x_inds);
pn_y_inds(self_inds) = [];
pn_x_inds(self_inds) = [];
pn_onset_time(self_inds) = [];

critical_times = zeros(numel(pn_y_inds), 1);

y_onset_time = pn_onset_time;
x_onset_time = pn_onset_time - M;

% remove self regulation

y_inds = pn_y_inds * (nt + 1) + y_onset_time;
x_inds = pn_x_inds * (nt + 1) + x_onset_time;

yx_inds_uniq_1 = unique([y_inds; x_inds]);

A_cell = {};

parents = cell(1, numel(yx_inds_uniq_1));
childs = cell(1, numel(yx_inds_uniq_1));

for i = 1 : numel(yx_inds_uniq_1)
    cur_ind = yx_inds_uniq_1(i);
    if any(x_inds == cur_ind)
        inds = find( x_inds == cur_ind);
        childs{i} = arrayfun(@(x) find(yx_inds_uniq_1 == x), y_inds(inds));
    end
    if any(y_inds == cur_ind)
        inds = find( y_inds == cur_ind);
        parents{i} = arrayfun(@(x) find(yx_inds_uniq_1 == x), x_inds(inds));
    end
end

% BFS to detect connected components, actually stack is a queue
visit_list = zeros(numel(yx_inds_uniq_1), 1);
stack = [];
ind = find(visit_list == 0, 1);
stack = [stack, ind];
graph_count = 1;
while ~all(visit_list > 0)
    A_cell{graph_count} = [];
    while ~isempty(stack)
        cur_ind = stack(1);
        stack(1) = [];
        if ~visit_list(cur_ind)
            visit_list(cur_ind) = 1;
            A_cell{graph_count} = [A_cell{graph_count}; cur_ind];

            if ~isempty(parents{cur_ind})
                stack = [stack; parents{cur_ind}];
            end
            if ~isempty(childs{cur_ind})
                stack = [stack; childs{cur_ind}];
            end 
        end
    end
    if isempty(stack)
        graph_count = graph_count + 1;
        ind = find(visit_list == 0, 1);
        stack = [stack; ind];
    end   
end

A_time_cell = cellfun(@(x) rem(yx_inds_uniq_1(x), (nt + 1)), A_cell, 'uniformoutput', false);
A_ind_cell = cellfun(@(x) floor(yx_inds_uniq_1(x) / (nt + 1)), A_cell, 'uniformoutput', false);

max_width = sum(cellfun(@(x) sum(x == mode(x)), A_time_cell));

min_time_ind = min(cellfun(@(x) min(x), A_time_cell));
max_time_ind = max(cellfun(@(x) max(x), A_time_cell));

max_height = max_time_ind - min_time_ind + 1;

m = max_width;
n = max_height;

scale_x = 60;
scale_y = 4;

rec_width = 0.8 * scale_x;
rec_height = 0.4 * scale_y;
rec_pos = @(x, y) [x - rec_width / 2, y - rec_height / 2, rec_width, rec_height];
rec_text = @(x, y, t) text(x, y, t,'HorizontalAlignment','center', 'fontsize', 12, 'fontweight', 'bold');

figure,

set(gcf, 'color', 'w')
set(gcf, 'Visible', 'on')
% set(gcf, 'Visible', 'off')

dpi = 300;
per_width = 150;
per_height = 120;
figure_width = per_width * m;
figure_height = per_height * n;

set(gcf, 'Position', [1, 1, figure_width, figure_height]);
axes('position', [0.05, 0.05, 0.9, 0.9]);
axis off;


axis([0, (m + 1) * scale_x + 5, 0, (n + 1)  * scale_y + 5]);
axis off;

nodes_handle = {};
edges_handle = {};

pos_func = @(x, y) [x * scale_x, (n - y + 1) * scale_y];

start_place = 0;
% place nodes 
node_positions = zeros(numel(yx_inds_uniq_1), 2);
for i = 1 : numel(A_ind_cell)
    cur_ind_graph = A_ind_cell{i};
    cur_time_graph = A_time_cell{i};
    cur_graph = A_cell{i};
    
    for j = 1 : numel(cur_ind_graph)
        cur_ind = cur_ind_graph(j);
        cur_time = cur_time_graph(j);
        cur_time_od = numel(find(cur_time_graph(1 : j) == cur_time));
        
        cur_x = (start_place + cur_time_od) * scale_x;
        cur_y = (n - cur_time + 1) * scale_y;
        node_positions(cur_graph(j), :) = [cur_x, cur_y];
        
        rectangle('position', rec_pos(cur_x, cur_y), 'Curvature',0.2, 'FaceColor',[238	238	0] / 255, 'EdgeColor','b');
        h = gca;
        rec_text(cur_x, cur_y, curr_nodes_names{cur_ind});
        nodes_handle{i, j} = h;
    end  
    start_place = start_place + sum(cur_time_graph == mode(cur_time_graph));
end

% place edges
hold on

for i = 1 : numel(A_ind_cell)
    cur_graph = A_cell{i};
    cur_ind_graph = A_ind_cell{i};
    cur_time_graph = A_time_cell{i};
    
    [~, inds] = sort(cur_time_graph);
    cur_ind_graph = cur_ind_graph(inds);
    cur_graph = cur_graph(inds);
    
    for j = 1 : numel(cur_ind_graph)
        cur_ind = cur_ind_graph(j);
        cur_uniq_ind = cur_graph(j);
        if ~isempty(childs{cur_uniq_ind})
            children_inds = childs{cur_uniq_ind};
            for k = 1 : numel(children_inds)
                h = draw_line(node_positions(cur_uniq_ind, :), node_positions(children_inds(k), :), rec_height);
                
                % color the lines by trend
                x_ind = cur_ind;
                y_ind = floor(yx_inds_uniq_1( children_inds(k)) / (nt + 1));
                yx_ind = sub2ind(size(A), y_ind, x_ind);      
                curr_trend = pn_trend(inds_together == yx_ind, :);
                if curr_trend(1) > 0 && curr_trend(2) > 0
                   h.Color = [0, 0, 0];
                elseif curr_trend(1) < 0 && curr_trend(2) < 0
                   h.Color = [1, 0, 0.0];
                elseif curr_trend(1) > 0 && curr_trend(2) < 0
                   h.Color = [0, 1, 0];
                elseif curr_trend(1) < 0 && curr_trend(2) > 0
                   h.Color = [0, 0, 1];
                end    
                h.LineWidth = 2;
                edges_handle{i, j} = h;
            end       
        end
    end
end

% put time points, the first time point is 0

for i = 1 : n
    text(0, (n - i + 1) * scale_y, int2str(current_timepoint(i + min_time_ind - 1)),'HorizontalAlignment','center', 'FontSize',21, 'fontweight', 'bold');
end
h = text( -0.5 * scale_x, ceil(n / 2) * scale_y, 'Critical time', 'HorizontalAlignment','center', 'FontSize',24, 'fontweight', 'bold');
set(h, 'rotation', 90);

figure_filename = sprintf('%sclustering_ROI_aligned_network_%s_%s_%s_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, opt_std, critical_method, ct_method, cluster_num, M, vol_thresh);
export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);         

end

function h = draw_line(pos_1, pos_2, rec_height)

x1 = pos_1(1);
y1 = pos_1(2);
x2 = pos_2(1);
y2 = pos_2(2);

pos_s = pos_1 - [0, rec_height / 2];
pos_e = pos_2 + [0, rec_height / 2];

pos_se = [pos_s; pos_e];

h = plot(pos_se(:, 1), pos_se(:, 2));
 
line2arrow(h);        

end




