function [param] = master_script_process_options(param)
% 03/15/2016 xruan: copied from t cell paper code

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set defaults and process options structure:
  
if ~false 
    nodes_to_exclude = '2-2[0-9]|2-30';
    [stat, hostname] = system('hostname');
    hostname = deblank(hostname);
    if regexp(hostname, nodes_to_exclude)
        fprintf('%s is not going to be used in the running and go to sleep until the end of the running.', hostname)
        pause(24 * 3600);
    end
end

default_param = master_script_get_default_params();

if ~exist('param', 'var')
    param = default_param;
else
    param_prior_to_processing = param;
    param = process_options_structure(default_param, param);
end

% First create a base set of options for some all_conditions variants:
% Use Excel files for all conditions:
all_conditions_base_options = param;

all_conditions_base_param.results_location = '/projects/cellorganizer/xruan/tcell_project/tcell_excel_results/';

if isfield(param, 'named_options_set')
    fprintf('Using named_options_set "%s"\n', param.named_options_set)
    switch param.named_options_set
        case 'CGP'     
            param.causal_method = 'CGP';
        case 'CENR'
            param.causal_method = 'CENR';
        case 'none'

        % Nothing here

      otherwise
        error('Unknown named_options_set value ''%s''', param.named_options_set)
    end
end

if strcmp(param.causal_method, 'CGP')
    switch param.opt_std
        case 'CV_M2'     
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-4, -1, 61), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -1, 31), 8);
            param.cv_param.self_prediction = false;
        case 'CV_M2_merge_214'
            param.merge_proteins = true;
            param.group_inds = {[1, 7], 2, [3, 5, 11, 12], 4, 6, 8, 9, 10};  
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -1, 31), 8);
            param.cv_param.self_prediction = false;      
            
        case 'CV_M2_merge_7'
            param.merge_proteins = true;
            param.group_inds = {[1, 2, 3, 5, 7, 11, 12], 4, 6, 8, 9, 10};   
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-4, -1, 61), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -1, 31), 8);
            param.cv_param.self_prediction = false;            
        case 'CV_M2_merge_25'
            param.merge_proteins = true;
            param.group_inds = {[1, 7], [2, 3, 5, 11, 12], 4, 6, 8, 9, 10};
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-4, -1, 61), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -1, 31), 8);
            param.cv_param.self_prediction = false;         
        case 'test'     
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 5), 8);
            param.cv_param.lambda_2_group = round(logspace(-4, -1, 2), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -1, 2), 8);
            param.cv_param.self_prediction = false;
            
      otherwise
        error('Unknown opt_std value ''%s''', param.opt_std)
    end
end

if strcmp(param.causal_method, 'CENR')
    param.residual_compute_method = 1;
    switch param.opt_std
        case 'CV_M2'     
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.alpha_group = 0.1 : 0.1 : 0.9;
            param.cv_param.b_group = -(0 : 0.1 : 1);
            param.cv_param.self_prediction = false;
        case 'CV_M2_merge_214'
            param.merge_proteins = true;
            param.group_inds = {[1, 7], 2, [3, 5, 11, 12], 4, 6, 8, 9, 10};  
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.alpha_group = 0.1 : 0.1 : 0.9;
            param.cv_param.b_group = -(0 : 0.1 : 1);
            param.cv_param.self_prediction = false;         
        case 'CV_M2_merge_7'
            param.merge_proteins = true;
            param.group_inds = {[1, 2, 3, 5, 7, 11, 12], 4, 6, 8, 9, 10};   
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.alpha_group = 0.1 : 0.1 : 0.9;
            param.cv_param.b_group = -(0 : 0.1 : 1);
            param.cv_param.self_prediction = false;      
        case 'CV_M2_merge_25'
            param.merge_proteins = true;
            param.group_inds = {[1, 7], [2, 3, 5, 11, 12], 4, 6, 8, 9, 10};
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.alpha_group = 0.1 : 0.1 : 0.9;
            param.cv_param.b_group = -(0 : 0.1 : 1);
            param.cv_param.self_prediction = false;        
      otherwise
        error('Unknown opt_std value ''%s''', param.opt_std)
    end
end

% March 19-20 2017
if strcmp(param.causal_method, 'CGP')
    param.residual_compute_method = 2;
    switch param.opt_std
        case 'CV_M2'     
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-4, -2, 21), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -2, 21), 8);
            param.cv_param.self_prediction = false;
        case 'CV_M2_merge_214'
            param.merge_proteins = true;
            param.group_inds = {[1, 7], 2, [3, 5, 11, 12], 4, 6, 8, 9, 10};  
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 21), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -2, 11), 8);
            param.cv_param.self_prediction = false;            
        case 'CV_M2_merge_7'
            param.merge_proteins = true;
            param.group_inds = {[1, 2, 3, 5, 7, 11, 12], 4, 6, 8, 9, 10};   
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-4, -2, 21), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -2, 11), 8);
            param.cv_param.self_prediction = false;            
        case 'CV_M2_merge_25'
            param.merge_proteins = true;
            param.group_inds = {[1, 7], [2, 3, 5, 11, 12], 4, 6, 8, 9, 10};
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-4, -2, 21), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -2, 11), 8);
            param.cv_param.self_prediction = false;         
        case 'test'     
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 5), 8);
            param.cv_param.lambda_2_group = round(logspace(-4, -1, 2), 8);
            param.cv_param.lambda_3_group = round(logspace(-4, -1, 2), 8);
            param.cv_param.self_prediction = false;
            
      otherwise
        error('Unknown opt_std value ''%s''', param.opt_std)
    end
end


% March 19-20 2017
if strcmp(param.causal_method, 'CENR')
    param.residual_compute_method = 1;
    switch param.opt_std
        case 'CV_M2'     
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.alpha_group = 0.1 : 0.1 : 0.9;
            param.cv_param.b_group = -(0.1 : 0.1 : 1);
            param.cv_param.self_prediction = false;
        case 'CV_M2_merge_214'
            param.merge_proteins = true;
            param.group_inds = {[1, 7], 2, [3, 5, 11, 12], 4, 6, 8, 9, 10};  
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -2, 21), 8);
            param.cv_param.alpha_group = 0.1 : 0.1 : 0.9;
            param.cv_param.b_group = -(0.1 : 0.1 : 1);
            param.cv_param.self_prediction = false;         
        case 'CV_M2_merge_7'
            param.merge_proteins = true;
            param.group_inds = {[1, 2, 3, 5, 7, 11, 12], 4, 6, 8, 9, 10};   
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.alpha_group = 0.1 : 0.1 : 0.9;
            param.cv_param.b_group = -(0.1 : 0.1 : 1);
            param.cv_param.self_prediction = false;      
        case 'CV_M2_merge_25'
            param.merge_proteins = true;
            param.group_inds = {[1, 7], [2, 3, 5, 11, 12], 4, 6, 8, 9, 10};
            
            param.cv_param.M_group = 2;
            param.cv_param.lambda_1_group = round(logspace(-4, -2, 41), 8);
            param.cv_param.lambda_2_group = round(logspace(-3, -1, 41), 8);
            param.cv_param.alpha_group = 0.1 : 0.1 : 0.9;
            param.cv_param.b_group = -(0.1 : 0.1 : 1);
            param.cv_param.self_prediction = false;        
      otherwise
        error('Unknown opt_std value ''%s''', param.opt_std)
    end
end

% param.debug_use_debug_switch_values_in_master_script = true;
% if param.debug_use_debug_switch_values_in_master_script
%     % For debugging, skip everything that is already (partially) done:
%     param.skip_window_saving = true;
%     param.skip_segmentation = true;
%     param.skip_alignment = true;
%     param.skip_morphing = true;
%     % % Takes maybe a minute, run every so often:
%     param.skip_morphing_inside_template = true;
%     param.skip_model_building = true;
%     param.skip_statistical_testing_across_conditions = true;
%     % param.skip_hierarchical_clustering = true;
%     param.skip_model_figure_generation = true;
%     param.skip_model_video_saving = true;
%     % param.skip_illustration_saving = true;
%     % param.skip_create_blender_figures = true;
% end
% 
% if ~param.skip_postprocess
%       param.check_morph_file_loadility = false;
%       param.skip_hierarchical_clustering = false;
% end
% options, pause
  
end
