function [t_cell_info] = master_script_visualize_model(t_cell_info)
% visualization of the graph 
% copied from master_script_CGP_temporal_clustering_ROI_postprocessing.m
% 
% Author: Xiongtao Ruan xruan@andrew.cmu.edu
% Date: Mar. 16, 2017

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




close all;

param = t_cell_info.param;
current_timepoint = param.current_timepoint;
M = param.lag_time;
cluster_num = param.cluster_num;
cluster_type = param.cluster_type;
vol_thresh = param.vol_thresh;
cluster_sensor = param.cluster_sensor;
data_type = param.data_type;
opt_std = param.opt_std;
causal_method = param.causal_method;
suffix = param.suffix;

graph_data = t_cell_info.graph_data;
kept_nodes = t_cell_info.graph_data.kept_nodes;
all_sensor_region_combine = graph_data.all_sensor_region_combine;
kept_node_names = all_sensor_region_combine(kept_nodes);

illustration_colormap_to_use = t_cell_info.param.illustration_colormap_to_use;

model_sub_location = t_cell_info.path_info.model_sub_location;
figure_sub_location = t_cell_info.path_info.figure_sub_location;

% [R_t_3d, A, c] = master_script_causal_graph_process(X, M) 
results_filename = sprintf('%sclustering_ROI_results_%s_std_%s_lag_%d_reltime_%d-%d_vol_thrd_%d%s', model_sub_location, causal_method, opt_std, M, current_timepoint(1), current_timepoint(end), vol_thresh, suffix);

results = load(results_filename);
A = results.A;
try
    c = results.c;
    R_t_3d = results.R_t_3d;
end

% plot for the figures in the paper
if ~false  
    close all;        
    figure, 
    set(gcf, 'Visible', 'off')
    set(gcf, 'Visible', 'on')
    set(gcf, 'Color', 'w')
    set(gcf, 'Renderer', 'opengl');
    dpi = 300;
    figure_height = 1200;
    figure_width = 1500;
    set(gcf, 'Position', [1, 1, figure_width, figure_height]);

    image_to_show = [];
    image_to_show = A;

    imagesc(image_to_show);
    colormap(illustration_colormap_to_use), caxis([min(image_to_show(:)), max(image_to_show(:))]);
    set(gca, 'YTick',  1 : size(A, 1), 'YTickLabel', kept_node_names);
    set(gca, 'XTick',  1 : size(A, 1), 'XTickLabel', kept_node_names);

    ylabel('Effect');
    xlabel('Cause');
    ax = gca;
    ax.FontWeight = 'bold'
    ax.FontSize = 21;
    ax.XTickLabelRotation=90;
    h = colorbar;

    xlabel(h, 'range', 'fontsize', 21, 'fontweight', 'bold');

    figure_filename = sprintf('%sclustering_ROI_A_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);          
end    
% for lag time larger than 1 visualize others

if ~false && M > 1
    for i = 1 : M
        close all;        
        figure, 
        set(gcf, 'Visible', 'off')
        set(gcf, 'Visible', 'on')
        set(gcf, 'Color', 'w')
        set(gcf, 'Renderer', 'opengl');
        dpi = 300;
        figure_height = 1200;
        figure_width = 1500;
        set(gcf, 'Position', [1, 1, figure_width, figure_height]);        
        
        image_to_show = [];
        image_to_show = R_t_3d(:, :, i);

        imagesc(image_to_show);
        colormap(illustration_colormap_to_use), caxis([min(image_to_show(:)), max(image_to_show(:))]);
        % axis off;
        % ylabel_group = arrayfun(@(x) num2str(x), -2 : 9, 'UniformOutput', false);
        set(gca, 'YTick',  1 : size(A, 1), 'YTickLabel', kept_node_names);
        set(gca, 'XTick',  1 : size(A, 1), 'XTickLabel', kept_node_names);
        ylabel('Effect');
        xlabel('Cause');
        ax = gca;
        ax.FontWeight = 'bold';
        ax.FontSize = 18;
        ax.XTickLabelRotation=90;
        title(sprintf('Adjacency matrix A_%d M - %d   Volume threshold - %d', i, M, vol_thresh), 'fontsize', 28);
        h = colorbar;
           
        figure_filename = sprintf('%sclustering_ROI_A%d_k-%d_M_%d_vol_thrsd_%d_%s', figure_sub_location, i, cluster_num, M, vol_thresh);
        export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);  
    end
    figure, 
    set(gcf, 'Visible', 'off')
    set(gcf, 'Visible', 'on')
    set(gcf, 'Color', 'w')
    set(gcf, 'Renderer', 'opengl');
    dpi = 300;
    figure_height = 500;
    figure_width = 1500;
    set(gcf, 'Position', [1, 1, figure_width, figure_height]);
    image_to_show = reshape(R_t_3d, size(R_t_3d, 1), []);
    imagesc(image_to_show);
    colormap(illustration_colormap_to_use), caxis([min(image_to_show(:)), max(image_to_show(:))]);
    ylabel('Effect');
    xlabel('Cause');

    ax = gca;
    ax.FontWeight = 'bold';
    ax.FontSize = 21;
    h = colorbar;
    figure_filename = sprintf('%sclustering_ROI_all_As_k-%d_M_%d_vol_thrsd_%d', figure_sub_location, cluster_num, M, vol_thresh);
    export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);  
end
% figures for the paper
if ~false && M > 1
    for i = 1 : M
        close all;        
        figure, 
        set(gcf, 'Visible', 'off')
        set(gcf, 'Visible', 'on')
        set(gcf, 'Color', 'w')
        set(gcf, 'Renderer', 'opengl');
        dpi = 300;
        figure_height = 1200;
        figure_width = 1500;
        set(gcf, 'Position', [1, 1, figure_width, figure_height]);       
        
        image_to_show = [];
        image_to_show = R_t_3d(:, :, i);
        
        imagesc(image_to_show);
        colormap(illustration_colormap_to_use), caxis([min(image_to_show(:)), max(image_to_show(:))]);
        % axis off;
        % ylabel_group = arrayfun(@(x) num2str(x), -2 : 9, 'UniformOutput', false);
        set(gca, 'YTick',  1 : size(A, 1), 'YTickLabel', kept_node_names);
        set(gca, 'XTick',  1 : size(A, 1), 'XTickLabel', kept_node_names);
        ylabel('Effect');
        xlabel('Cause');
        ax = gca;
        ax.FontWeight = 'bold';
        ax.FontSize = 21;
        ax.XTickLabelRotation=90; 
        h = colorbar;

        xlabel(h, 'range', 'FontSize', 21, 'FontWeight', 'bold');
         
        figure_filename = sprintf('%sclustering_ROI_A%d_k-%d_M_%d_vol_thrsd_%d_%s', figure_sub_location, i, cluster_num, M, vol_thresh);
        export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);  
    end  
end

end

