function [AUC] = model_AUC_computing(A, R_t_3d, param)
% compute AUC based on region 1. 
% March 20, 2017

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



causal_method = param.causal_method;
unique_sensor_names = param.unique_sensor_names;

load('Interaction_ground_truth_01232017.mat');

% visualize groundtruth network
G_raw = ground_truth;

if param.merge_proteins
    group_inds = param.group_inds;
    unique_sensor_names = cellfun(@(x) unique_sensor_names{min(x)}, group_inds, 'Uniformoutput', false);

    G_adj = zeros(numel(group_inds));
    for i = 1 : numel(group_inds)
        for j = 1 : numel(group_inds)
            if i == j 
                continue;
            end
            G_adj(i, j) = sum(reshape(G_raw(group_inds{i}, group_inds{j}), 1, [])) > 0;
        end
    end
else
    G_adj = G_raw;
end

labels = G_adj;
labels(eye(size(labels, 1)) == 1) = [];

G = graph(G_adj, unique_sensor_names);


switch causal_method
    case 'CGP'
        A_c = A;
    case 'CENR'
        A_en = sign(sum(R_t_3d, 3));
        A_en = max(A_en .* R_t_3d, [], 3) .* A_en;
        % A_1 = A;
        A_en(eye(size(A_en, 1)) == 1) = 0;
        A_c = A_en;
end

N_node = size(A_c, 1);

% default as th
A_c_1 = abs(A_c(1 : 3 : end, 1 : 3 : end));
A_c_1(eye(N_node / 3) == 1) = [];
[Y_c, X_c] = roc(labels, A_c_1);
if Y_c(end) ~= 1 || X_c(end) ~= 1
    Y_c = [Y_c, 1];
    X_c = [X_c, 1];
end
AUC_c = Y_c(2 : end) * diff(X_c)';
AUC = AUC_c;



end
