function [norm_diff_x] = most_influence_pairs_by_prediction(X, A, c, param)

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



% this function is for extraction of important time point for the time
% series that can represent the start of the event. 
% I_x is the time seris of effect, I_y is the time series of cause, lag
% is the proposed lag time for the analysis. 

% May 20, 2016
% extract trend based only on the maximum change of prediction. 
% copied from critical_timepoint_extract.m

t = 1;
if nargin < 4
    lag = 2;
    direction = 'pos';
    method = 'single';
else
    lag = param.lag;
    direction = param.direction;
    method = param.method;
end
M = lag;
[m, n] = size(X);
P_X = zeros(m, n - M);

switch method
    case 'overall'
        A_0_M = arrayfun(@(i) A ^ i, 0 : M, 'UniformOutput', false);
        A_0_M = cat(3, A_0_M{:});
        for i = 1 : n - M
            P_i_X_i = zeros(m, 1);
            for j = 1 : M            
                P_j = arrayfun(@(k) A_0_M(:, :, k) * c(j, k), 1 : j + 1, 'UniformOutput', false);
                P_j = sum(cat(3, P_j{:}), 3);
                P_j_X_k_j = P_j * X(:, i + M - j);
                P_i_X_i = P_i_X_i + P_j_X_k_j;
            end
            P_X(:, i) = P_i_X_i;
        end
        diff_x = diff(P_X(ind_x, :));
        [~, ind] = max(abs(diff_x));
        t = ind + M;
    case 'single'
        A_0_M = arrayfun(@(i) A ^ i, 0 : M, 'UniformOutput', false);
        A_0_M = cat(3, A_0_M{:});
        P_X_yx = zeros(m, m, n - M);
        for i_0 = 1 : m
            for i = 1 : n - M
                P_i_X_i = zeros(m, 1);
                for j = 1 : M            
                    P_j = arrayfun(@(k) A_0_M(:, :, k) * c(j, k), 1 : j + 1, 'UniformOutput', false);
                    P_j = sum(cat(3, P_j{:}), 3);
                    P_j_X_k_j = P_j(i_0, :)' * X(i_0, i + M - j);
                    P_i_X_i = P_i_X_i + P_j_X_k_j;
                end
                P_X_yx(i_0, :, i) = P_i_X_i;
            end
        end
        diff_x = diff(P_X_yx, 1, 3);
        % normalize by the average of actual time series.
        mean_X = mean(X(:, M + 1 : end), 2);
        norm_diff_x_cell = arrayfun(@(k) bsxfun(@rdivide, diff_x(:, k, :), mean_X), 1 : m, 'uniformoutput', false);
        norm_diff_x = cat(2, norm_diff_x_cell{:});
        
        [~, ind] = max(abs(diff_x));
    otherwise
        error('Unrecognized method.')
end 

% linear trend decision from the original data
% X_y = X(ind_y, t - M : t);
% if (t + M > n)
%     X_x = X(ind_x, n - M : n);
% else
%     X_x = X(ind_x, t : t + M);
% end
% 
% XT = [[1 : lag + 1]', ones(lag + 1, 1)];
% YT_y = X_y';
% YT_x = X_x';
% beta_y = (XT' * XT) \ XT' * YT_y;
% beta_x = (XT' * XT) \ XT' * YT_x;
% ts_trend = [beta_y(1), beta_x(1)];                

end