function h = network_comparison_plot_network(A_common, A_1_diff, A_2_diff, all_node_names)
% plot network comparison results. It aims to compare two networks,
% A_common is the common element, A_1_diff is the additional net for
% network 1, and A_2_diff is the additional net for network 2. 
%
% 
% Date: Jan. 08, 2017

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




A_total = A_common + A_1_diff + A_2_diff;


if ~false
    A_1 = A_total;
    A_1(eye(size(A_1, 1)) == 1) = 0;
    [A_vec_sort, inds_sort] = sort(abs(A_1(:)));
    Node_to_plot = sum(abs(A_1(:)) > 0);
    inds_sort = inds_sort(end - Node_to_plot + 1 : end);
    [y_inds, x_inds] = ind2sub(size(A_1), inds_sort);

    y_inds_unique = unique(y_inds);
    x_inds_unique = unique(x_inds);

    yx_inds_unique = unique([y_inds_unique; x_inds_unique]);

    m = numel(yx_inds_unique);

    A_cm = zeros(m);
    A_cm_1 = zeros(m);
    A_cm_2 = zeros(m);
    % [pos_region_inds, pos_sensor_inds] = ind2sub([len_per_sensor, numel(unique_sensor_names)], yx_inds_unique);
    nodes_names = all_node_names(yx_inds_unique)';
    
    % nodes_names = arrayfun(@(x, y) sprintf('%s-%d', unique_sensor_names{x}, y), pos_sensor_inds, pos_region_inds, 'uniformoutput', false);
    
    for i = 1 : numel(y_inds)
        y_ind = y_inds(i);
        x_ind = x_inds(i);
        
        m_i = yx_inds_unique == x_ind;
        n_i = yx_inds_unique == y_ind;
        A_cm(m_i, n_i) = A_1(y_ind, x_ind);   
        % A_cm(m_i, n_i) = 1;    
        if A_1_diff(y_ind, x_ind) ~= 0
           A_cm_1(m_i, n_i) = 1;
        end
        if A_2_diff(y_ind, x_ind) ~= 0
           A_cm_2(m_i, n_i) = 1;
        end
        
    end
    
    A_2 = A_cm;
    G = digraph(A_2);
    G.Nodes.Names = nodes_names;
    
    figure,
    set(gcf, 'color', 'w')
    set(gcf, 'Visible', 'on')
    % set(gcf, 'Visible', 'off')

    figure_width = 800;
    figure_height = 500;
    dpi = 300;

    set(gcf, 'Position', [1, 1, figure_width, figure_height]);
    axis tight
    
    p1 = plot(G,'EdgeLabel', round(G.Edges.Weight, 2), 'LineWidth', 2, 'NodeLabel', G.Nodes.Names, 'Layout', 'layered', 'ArrowSize', 28);
    set(p1, 'defaultAxesFontSize', 20)
    axis off;
    box off;
    [x1, y1] = ind2sub(size(A_cm_1), find(A_cm_1 == 1));
    highlight(p1, x1, y1, 'EdgeColor', 'r');
    [x2, y2] = ind2sub(size(A_cm_2), find(A_cm_2 == 1));
    highlight(p1, x2, y2, 'EdgeColor', 'g');
    ax = gca;
    ax.FontSize = 15;
    ax.FontWeight = 'bold';
    % figure_filename = sprintf('%sclustering_network_combine_k-%d_M_%d_vol_thrsd_%d', figure_dir, cluster_num, M, vol_thresh);
    % export_fig(figure_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);    
    h = p1;

end


end