 function [h, c] = plot_hierachical_cluster_figure(Y, A, node_names, metric)
 % convient function for hierachical clustering figure generation, Y:
 % distance, A: the figure that want to show, ususally the full matrix of
 % distance matrix, node_names: node names, metric: metric for distance,
 % just used for title. 

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


 
positions = [0.02, 0.25, 0.24, 0.5;...
             0.40, 0.25, 0.5, 0.5; ...
             0.91, 0.25, 0.025, 0.5];
current_tree = linkage(Y);

figure, 
set(gcf, 'Visible', 'off')
set(gcf, 'Visible', 'on')
set(gcf, 'Color', 'w')
set(gcf, 'Renderer', 'opengl');
dpi = 300;
figure_height = 1200;
figure_width = 1200;
set(gcf, 'Position', [1, 1, figure_width, figure_height]);

axes('Position', positions(1, :))
[H, T, current_tree_permutation] = dendrogram(current_tree, 0, 'orientation', 'left');
% dendrogram(current_tree, 'orientation', 'left');
axis off
set(H,'LineWidth',2)
dendrogram_plot_handle = gca;
ylim(dendrogram_plot_handle, [0.5,  size(A, 1) + 0.5]);
set(dendrogram_plot_handle,'YDir','reverse')

A_permute = (A(current_tree_permutation, current_tree_permutation))';
tick_values = 1 : size(A, 1);
ytrick_labels = node_names(current_tree_permutation);
xtrick_labels = node_names(current_tree_permutation);

% possible bug
hold on
axes('Position', positions(2, :))
imagesc(A_permute);
set(gca,'YDir','reverse')
matrix_plot_handle = gca;

set(matrix_plot_handle, 'YTick', tick_values, 'YTickLabel', ytrick_labels, 'fontsize', 15, 'fontweight', 'bold')

set(matrix_plot_handle, 'XTick', tick_values, 'XTickLabel', xtrick_labels, 'fontsize', 15, 'fontweight', 'bold')
matrix_plot_handle.XTickLabelRotation=90;

h = colorbar;
set(h, 'Position', positions(3, :))
hold off   

current_clusters = cluster(current_tree, 'MaxClust', 3);

c = cophenet(current_tree, Y)

% title(sprintf('Metric - %s CP: %.4f', metric, c), 'fontsize', 12, 'fontweight', 'bold')

 
 end