function [t_cell_info] = process_cross_validation_data_splitting(t_cell_info)

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



model_data = t_cell_info.model_data.model_data;
model_means =  t_cell_info.model_data.model_means;
unique_sensor_names = t_cell_info.model_data.unique_sensor_names;
used_model_names = t_cell_info.model_data.used_model_names;
used_model_reltime = t_cell_info.model_data.used_model_reltime;
used_model_conditions = t_cell_info.model_data.used_model_conditions;
interporlation_time = t_cell_info.model_data.interporlation_time;

current_condition = t_cell_info.param.current_condition;

cv_fold = t_cell_info.param.cv_param.cv_fold;

num_model_data_used = numel(unique_sensor_names);

cv_data_location = t_cell_info.path_info.cv_data_location;
data_location = t_cell_info.path_info.data_location; 

% set up partition over the data

s = RandStream('mt19937ar', 'Seed', 892734);
RandStream.setGlobalStream(s);

cv_means = {};
cv_means_mat = zeros([num_model_data_used, cv_fold, 2, numel(interporlation_time), size(model_means, 3)]);
cv_count_mat = zeros([num_model_data_used, cv_fold, 2, numel(interporlation_time)]);
rand_labels_cell = cell(cv_fold, 1);

cv_data_filename = sprintf('%sCV_data_%dfold.mat', cv_data_location, cv_fold);
compute_cv_data = true;

if compute_cv_data && ~exist(cv_data_filename, 'file')
    for i = 1 : numel(unique_sensor_names)
        current_sensor = unique_sensor_names{i};
        cur_model_data_filename = sprintf('%smodel_data_%s_%s', data_location, current_condition, current_sensor);
        a = load(cur_model_data_filename);
        cur_model_data = a.model_data_i;

        cur_M = numel(cur_model_data);
        rand_labels = ceil(randperm(cur_M) / floor(cur_M / cv_fold));

        if any(rand_labels == cv_fold + 1)
            upper_num = sum(rand_labels == cv_fold + 1);
            rand_labels(rand_labels == cv_fold + 1) = randperm(cv_fold, upper_num);
        end
        rand_labels_cell{i} = rand_labels;
        for j = 1 : cv_fold
            % cv_means{i, j} = cur_cv_means;
            [cv_means_mat(i, j, 1, :, :), cv_count_mat(i, j, 1, :)] = compute_time_series_mean_with_nan(cur_model_data(rand_labels ~= j));
            [cv_means_mat(i, j, 2, :, :), cv_count_mat(i, j, 2, :)] = compute_time_series_mean_with_nan(cur_model_data(rand_labels == j));
        end
    end
    save(cv_data_filename, 'cv_means_mat', 'cv_count_mat', 'cv_fold', 'unique_sensor_names', 'rand_labels_cell', 'interporlation_time');
    for j = 1 : cv_fold
        cv_data_filename_j = sprintf('%sCV_data_%dfold_%d.mat', cv_data_location, cv_fold, j);
        cv_means_mat_j = cv_means_mat(:, j, :, :, :); 
        save(cv_data_filename_j, 'cv_means_mat_j', 'cv_fold', 'unique_sensor_names', 'rand_labels_cell', 'interporlation_time');      
    end
else
    % load(cv_data_filename);
end

end