function [options] = process_options_structure(default_options, options, required_options, process_substructures)
% Return a structure the same as the structure options but with fields added from the structure default_options if they are not in options. Produces an error if any string in the cell array of strings required_options is not a field name in options. The second and third arguments are optional. Applies process_options_structure to fields of default_options that are structures if process_substructures = true.
% 
% Dependencies:
% 
% 2012-10-31 tebuck: Created.
% 2013-03-10 tebuck: Added optional third argument required_options.
% 2013-03-11 tebuck: Added optional fourth argument process_substructures.

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  if nargin == 1
    options = default_options; 
    return
  end
  if ~exist('required_options', 'var')
    required_options = {}; 
  end
  if ~exist('process_substructures', 'var')
    process_substructures = false; 
  end
  
  option_names = fieldnames(default_options); 
  for index = 1:length(option_names)
    current_option = option_names{index}; 
    if ~isfield(options, current_option)
      if any(strcmp(current_option, required_options))
        error('Option %s is required', current_option)
      else
        options = setfield(options, current_option, getfield(default_options, current_option)); 
      end
    else
      % fprintf('isfield!\n')
      % keyboard
      if process_substructures && isfield(default_options, current_option) && isstruct(default_options.(current_option))
        % a = options.(current_option)
        % b = default_options.(current_option)
        options.(current_option) = process_options_structure(default_options.(current_option), options.(current_option), {}, process_substructures); 
        % b = default_options.(current_option)
        % a = options.(current_option)
        % keyboard
      end
    end
  end

    
