function [mean_data] = read_sensors_model_mean_data(sensor_name, condition, timepoints, directory, model_type)

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



% this script is for reading the model means from the model files. 
% Output, a cell with each one contain model mean of all time points (-2 - 9 default)
% of a sensor in a condition 

% Author: Xiongtao Ruan
% Date: Feb. 12, 2016


if nargin < 5
    model_type = 'standardized_voxels';
end
if nargin < 4    
    directory = '/home/xruan/projects/tcell_simulation/data/Actin_two_point';
end
if nargin == 0
    directory = '/home/xruan/projects/tcell_simulation/data/Actin_two_point';
    sensor_name = {'Actin'};
    condition = {'Full Stimulus'};
end

filenames = dir_recursive(directory);
data_cell = {};
for i = 1 : numel(sensor_name)
    for j = 1 : numel(condition)
        data_cell{i, j} = [];
        for k = 1 : numel(timepoints)  
            curr_sensor = sensor_name{i};
            curr_condition = condition{j};
            curr_time = timepoints(k);
            curr_pattern = sprintf('%s_%s_reltime%3d_%s', curr_condition, curr_sensor, curr_time, model_type);
            inds = find(~cellfun(@isempty, regexp(filenames, curr_pattern)) == 1);
            curr_filenames = filenames{inds};
            try 
                data = load(curr_filenames);
            catch ME
                error(sprintf('File %s does not exist or is corrupted!', curr_filenames));                
            end
            data_cell{i, j} = [data_cell{i, j}; data.current_mean];           
        end
    end
end
                     
mean_data = data_cell;

end