function [region_centroids, region_labels, label_order, map_label_sorted] = region_centroid_compute(map_label)
% this function is for computing all centroid for all regions in the map
% labels for the sorting of the regions.
% Feb. 4, 2016
% Mar. 18, 2017 choosen by the variances so that it is consistent. And
% synapse: region 1, nuclear: region 2, and cyto: region 3. 

%
% Author: Xiongtao Ruan
%
% Copyright (C) 2012-2017 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



[m, n, k] = size(map_label);
unique_labels = unique(map_label(:));
% remove 0 
unique_labels(1) = [];
region_labels = unique_labels;
num_regions = numel(unique_labels);

region_centroids = zeros(num_regions, 3);
region_variances = zeros(num_regions, 3);
for i = 1 : numel(unique_labels)
    inds_i = find(map_label == unique_labels(i));
    [y, x, z] = ind2sub([m, n, k], inds_i);
    centroid_i = mean([y, x, z]);
    var_i = var([y, x, z]);
    region_centroids(i, :) = centroid_i;
    region_variances(i, :) = var_i;
end

[B, sort_inds] = sortrows(region_centroids);
[B, sort_inds] = sortrows(region_variances);
label_order = sort_inds;

map_label_sorted = zeros([m, n, k]);

for i = 1 : num_regions
    map_label_sorted(find(map_label == unique_labels(sort_inds(i)))) = i;    
end


end



